@@ -12,8 +12,6 @@
  * This file is dual licensed under LGPL v2.1 and the Scintilla license (http://www.scintilla.org/License.txt).
  */
 
-#include <sys/time.h>
-
 #include <cstddef>
 #include <cstdlib>
 #include <cassert>
@@ -21,9 +19,14 @@
 #include <cstdio>
 
 #include <stdexcept>
+#include <string_view>
 #include <vector>
 #include <map>
+#include <functional>
 #include <memory>
+#include <numeric>
+
+#include <sys/time.h>
 
 #import <Foundation/NSGeometry.h>
 
@@ -31,6 +34,7 @@
 
 #include "StringCopy.h"
 #include "XPM.h"
+#include "UniConversion.h"
 
 #import "ScintillaView.h"
 #import "ScintillaCocoa.h"
@@ -69,23 +73,9 @@ inline CGRect PRectangleToCGRect(PRectangle &rc) {
 	return CGRectMake(rc.left, rc.top, rc.Width(), rc.Height());
 }
 
-//--------------------------------------------------------------------------------------------------
-
-/**
- * Converts a Quartz-style rectangle to a PRectangle structure as used by Scintilla.
- */
-inline PRectangle CGRectToPRectangle(const CGRect &rect) {
-	PRectangle rc;
-	rc.left = (int)(rect.origin.x + 0.5);
-	rc.top = (int)(rect.origin.y + 0.5);
-	rc.right = (int)(rect.origin.x + rect.size.width + 0.5);
-	rc.bottom = (int)(rect.origin.y + rect.size.height + 0.5);
-	return rc;
-}
-
 //----------------- Font ---------------------------------------------------------------------------
 
-Font::Font(): fid(0) {
+Font::Font() noexcept : fid(0) {
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -96,10 +86,18 @@ Font::~Font() {
 
 //--------------------------------------------------------------------------------------------------
 
-static int FontCharacterSet(Font &f) {
-	return static_cast<QuartzTextStyle *>(f.GetID())->getCharacterSet();
+static QuartzTextStyle *TextStyleFromFont(const Font &f) {
+	return static_cast<QuartzTextStyle *>(f.GetID());
 }
 
+//--------------------------------------------------------------------------------------------------
+
+static int FontCharacterSet(Font &f) {
+	return TextStyleFromFont(f)->getCharacterSet();
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Creates a CTFontRef with the given properties.
  */
@@ -123,6 +121,209 @@ void Font::Release() {
 	fid = 0;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+// Bidirectional text support for Arabic and Hebrew.
+
+namespace {
+
+CFIndex IndexFromPosition(std::string_view text, size_t position) {
+	const std::string_view textUptoPosition = text.substr(0, position);
+	return UTF16Length(textUptoPosition);
+}
+
+// Handling representations and tabs
+
+struct Blob {
+	XYPOSITION width;
+	Blob(XYPOSITION width_) : width(width_) {
+	}
+};
+
+static void BlobDealloc(void *refCon) {
+	Blob *blob = static_cast<Blob *>(refCon);
+	delete blob;
+}
+
+static CGFloat BlobGetWidth(void *refCon) {
+	Blob *blob = static_cast<Blob *>(refCon);
+	return blob->width;
+}
+
+class ScreenLineLayout : public IScreenLineLayout {
+	CTLineRef line = NULL;
+	const std::string text;
+public:
+	ScreenLineLayout(const IScreenLine *screenLine);
+	~ScreenLineLayout();
+	// IScreenLineLayout implementation
+	size_t PositionFromX(XYPOSITION xDistance, bool charPosition) override;
+	XYPOSITION XFromPosition(size_t caretPosition) override;
+	std::vector<Interval> FindRangeIntervals(size_t start, size_t end) override;
+};
+
+ScreenLineLayout::ScreenLineLayout(const IScreenLine *screenLine) : text(screenLine->Text()) {
+	const UInt8 *puiBuffer = reinterpret_cast<const UInt8 *>(text.data());
+
+	// Start with an empty mutable attributed string and add each character to it.
+	CFMutableAttributedStringRef mas = CFAttributedStringCreateMutable(NULL, 0);
+
+	for (size_t bp=0; bp<text.length();) {
+		const unsigned char uch = text[bp];
+		const unsigned int byteCount = UTF8BytesOfLead[uch];
+		XYPOSITION repWidth = screenLine->RepresentationWidth(bp);
+		if (uch == '\t') {
+			// Find the size up to the tab
+			NSMutableAttributedString *nas = (__bridge NSMutableAttributedString *)mas;
+			const NSSize sizeUpTo = [nas size];
+			const XYPOSITION nextTab = screenLine->TabPositionAfter(sizeUpTo.width);
+			repWidth = nextTab - sizeUpTo.width;
+		}
+		CFAttributedStringRef as = NULL;
+		if (repWidth > 0.0f) {
+			CTRunDelegateCallbacks callbacks = {
+				.version = kCTRunDelegateVersion1,
+				.dealloc = BlobDealloc,
+				.getWidth = BlobGetWidth
+			};
+			CTRunDelegateRef runDelegate = CTRunDelegateCreate(&callbacks, new Blob(repWidth));
+			NSMutableAttributedString *masBlob = [[NSMutableAttributedString alloc] initWithString:@"X"];
+			NSRange rangeX = NSMakeRange(0, 1);
+			[masBlob addAttribute: (NSString *)kCTRunDelegateAttributeName value: (__bridge id)runDelegate range:rangeX];
+			CFRelease(runDelegate);
+			as = (CFAttributedStringRef)CFBridgingRetain(masBlob);
+		} else {
+			CFStringRef piece = CFStringCreateWithBytes(NULL,
+								    &puiBuffer[bp],
+								    byteCount,
+								    kCFStringEncodingUTF8,
+								    false);
+			QuartzTextStyle *qts = static_cast<QuartzTextStyle *>(screenLine->FontOfPosition(bp)->GetID());
+			CFMutableDictionaryRef pieceAttributes = qts->getCTStyle();
+			as = CFAttributedStringCreate(NULL, piece, pieceAttributes);
+			CFRelease(piece);
+		}
+		CFAttributedStringReplaceAttributedString(mas,
+							  CFRangeMake(CFAttributedStringGetLength(mas), 0),
+							  as);
+		bp += byteCount;
+		CFRelease(as);
+	}
+
+	line = CTLineCreateWithAttributedString(mas);
+	CFRelease(mas);
+}
+
+ScreenLineLayout::~ScreenLineLayout() {
+	CFRelease(line);
+}
+
+size_t ScreenLineLayout::PositionFromX(XYPOSITION xDistance, bool charPosition) {
+	if (!line) {
+		return 0;
+	}
+	const CGPoint ptDistance = CGPointMake(xDistance, 0);
+	const CFIndex offset = CTLineGetStringIndexForPosition(line, ptDistance);
+	if (offset == kCFNotFound) {
+		return 0;
+	}
+	// Convert back to UTF-8 positions
+	return UTF8PositionFromUTF16Position(text, offset);
+}
+
+XYPOSITION ScreenLineLayout::XFromPosition(size_t caretPosition) {
+	if (!line) {
+		return 0.0;
+	}
+	// Convert from UTF-8 position
+	const CFIndex caretIndex = IndexFromPosition(text, caretPosition);
+
+	const CGFloat distance = CTLineGetOffsetForStringIndex(line, caretIndex, nullptr);
+	return distance;
+}
+
+void AddToIntervalVector(std::vector<Interval> &vi, XYPOSITION left, XYPOSITION right) {
+	const Interval interval = {left, right};
+	if (vi.empty()) {
+		vi.push_back(interval);
+	} else {
+		Interval &last = vi.back();
+		if (fabs(last.right-interval.left) < 0.01) {
+			// If new left is very close to previous right then extend last item
+			last.right = interval.right;
+		} else {
+			vi.push_back(interval);
+		}
+	}
+}
+
+std::vector<Interval> ScreenLineLayout::FindRangeIntervals(size_t start, size_t end) {
+	if (!line) {
+		return {};
+	}
+
+	std::vector<Interval> ret;
+
+	// Convert from UTF-8 position
+	const CFIndex startIndex = IndexFromPosition(text, start);
+	const CFIndex endIndex = IndexFromPosition(text, end);
+
+	CFArrayRef runs = CTLineGetGlyphRuns(line);
+	const CFIndex runCount = CFArrayGetCount(runs);
+	for (CFIndex run=0; run<runCount; run++) {
+		CTRunRef aRun = static_cast<CTRunRef>(CFArrayGetValueAtIndex(runs, run));
+		const CFIndex glyphCount = CTRunGetGlyphCount(aRun);
+		const CFRange rangeAll = CFRangeMake(0, glyphCount);
+		std::vector<CFIndex> indices(glyphCount);
+		CTRunGetStringIndices(aRun, rangeAll, indices.data());
+		std::vector<CGPoint> positions(glyphCount);
+		CTRunGetPositions(aRun, rangeAll, positions.data());
+		std::vector<CGSize> advances(glyphCount);
+		CTRunGetAdvances(aRun, rangeAll, advances.data());
+		for (CFIndex glyph=0; glyph<glyphCount; glyph++) {
+			const CFIndex glyphIndex = indices[glyph];
+			const XYPOSITION xPosition = positions[glyph].x;
+			const XYPOSITION width = advances[glyph].width;
+			if ((glyphIndex >= startIndex) && (glyphIndex < endIndex)) {
+				AddToIntervalVector(ret, xPosition, xPosition + width);
+			}
+		}
+	}
+	return ret;
+}
+
+// Helper for SurfaceImpl::MeasureWidths that examines the glyph runs in a layout
+
+void GetPositions(CTLineRef line, std::vector<CGFloat> &positions) {
+
+	// Find the advances of the text
+	std::vector<CGFloat> lineAdvances(positions.size());
+	CFArrayRef runs = CTLineGetGlyphRuns(line);
+	const CFIndex runCount = CFArrayGetCount(runs);
+	for (CFIndex run=0; run<runCount; run++) {
+		CTRunRef aRun = static_cast<CTRunRef>(CFArrayGetValueAtIndex(runs, run));
+		const CFIndex glyphCount = CTRunGetGlyphCount(aRun);
+		const CFRange rangeAll = CFRangeMake(0, glyphCount);
+		std::vector<CFIndex> indices(glyphCount);
+		CTRunGetStringIndices(aRun, rangeAll, indices.data());
+		std::vector<CGSize> advances(glyphCount);
+		CTRunGetAdvances(aRun, rangeAll, advances.data());
+		for (CFIndex glyph=0; glyph<glyphCount; glyph++) {
+			const CFIndex glyphIndex = indices[glyph];
+			if (glyphIndex >= positions.size()) {
+				return;
+			}
+			lineAdvances[glyphIndex] = advances[glyph].width;
+		}
+	}
+
+	// Accumulate advances into positions
+	std::partial_sum(lineAdvances.begin(), lineAdvances.end(),
+			 positions.begin(), std::plus<CGFloat>());
+}
+
+}
+
 //----------------- SurfaceImpl --------------------------------------------------------------------
 
 SurfaceImpl::SurfaceImpl() {
@@ -131,7 +332,7 @@ SurfaceImpl::SurfaceImpl() {
 	y = 0;
 	gc = NULL;
 
-	textLayout.reset(new QuartzTextLayout(nullptr));
+	textLayout.reset(new QuartzTextLayout());
 	codePage = 0;
 	verticalDeviceResolution = 0;
 
@@ -145,13 +346,12 @@ SurfaceImpl::SurfaceImpl() {
 //--------------------------------------------------------------------------------------------------
 
 SurfaceImpl::~SurfaceImpl() {
-	Release();
+	Clear();
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void SurfaceImpl::Release() {
-	textLayout->setContext(nullptr);
+void SurfaceImpl::Clear() {
 	if (bitmapData) {
 		bitmapData.reset();
 		// We only "own" the graphics context if we are a bitmap context
@@ -168,6 +368,12 @@ void SurfaceImpl::Release() {
 
 //--------------------------------------------------------------------------------------------------
 
+void SurfaceImpl::Release() {
+	Clear();
+}
+
+//--------------------------------------------------------------------------------------------------
+
 bool SurfaceImpl::Initialised() {
 	// We are initalised if the graphics context is not null
 	return gc != NULL;// || port != NULL;
@@ -191,7 +397,6 @@ void SurfaceImpl::Init(SurfaceID sid, WindowID) {
 	Release();
 	gc = static_cast<CGContextRef>(sid);
 	CGContextSetLineWidth(gc, 1.0);
-	textLayout->setContext(gc);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -227,7 +432,6 @@ void SurfaceImpl::InitPixMap(int width, int height, Surface *surface_, WindowID
 		// and we have no use for the bitmap without the context
 		bitmapData.reset();
 	}
-	textLayout->setContext(gc);
 
 	// the context retains the color space, so we can release it
 	CGColorSpaceRelease(colorSpace);
@@ -253,7 +457,7 @@ void SurfaceImpl::InitPixMap(int width, int height, Surface *surface_, WindowID
 
 void SurfaceImpl::PenColour(ColourDesired fore) {
 	if (gc) {
-		ColourDesired colour(fore.AsLong());
+		ColourDesired colour(fore.AsInteger());
 
 		// Set the Stroke color to match
 		CGContextSetRGBStrokeColor(gc, colour.GetRed() / 255.0, colour.GetGreen() / 255.0,
@@ -265,7 +469,7 @@ void SurfaceImpl::PenColour(ColourDesired fore) {
 
 void SurfaceImpl::FillColour(const ColourDesired &back) {
 	if (gc) {
-		ColourDesired colour(back.AsLong());
+		ColourDesired colour(back.AsInteger());
 
 		// Set the Fill color to match
 		CGContextSetRGBFillColor(gc, colour.GetRed() / 255.0, colour.GetGreen() / 255.0,
@@ -278,7 +482,7 @@ void SurfaceImpl::FillColour(const ColourDesired &back) {
 CGImageRef SurfaceImpl::GetImage() {
 	// For now, assume that GetImage can only be called on PixMap surfaces.
 	if (!bitmapData)
-		return nullptr;
+		return NULL;
 
 	CGContextFlush(gc);
 
@@ -287,8 +491,8 @@ CGImageRef SurfaceImpl::GetImage() {
 	if (colorSpace == NULL)
 		return NULL;
 
-	const int bitmapBytesPerRow = ((int) bitmapWidth * BYTES_PER_PIXEL);
-	const int bitmapByteCount = (bitmapBytesPerRow * (int) bitmapHeight);
+	const int bitmapBytesPerRow = bitmapWidth * BYTES_PER_PIXEL;
+	const int bitmapByteCount = bitmapBytesPerRow * bitmapHeight;
 
 	// Make a copy of the bitmap data for the image creation and divorce it
 	// From the SurfaceImpl lifetime
@@ -374,12 +578,12 @@ void SurfaceImpl::LineTo(int x_, int y_) {
 
 //--------------------------------------------------------------------------------------------------
 
-void SurfaceImpl::Polygon(Scintilla::Point *pts, int npts, ColourDesired fore,
+void SurfaceImpl::Polygon(Scintilla::Point *pts, size_t npts, ColourDesired fore,
 			  ColourDesired back) {
 	// Allocate memory for the array of points.
 	std::vector<CGPoint> points(npts);
 
-	for (int i = 0; i < npts; i++) {
+	for (size_t i = 0; i < npts; i++) {
 		// Quartz floating point issues: plot the MIDDLE of the pixels
 		points[i].x = pts[i].x + 0.5;
 		points[i].y = pts[i].y + 0.5;
@@ -642,6 +846,50 @@ void Scintilla::SurfaceImpl::AlphaRectangle(PRectangle rc, int cornerSize, Colou
 	}
 }
 
+void Scintilla::SurfaceImpl::GradientRectangle(PRectangle rc, const std::vector<ColourStop> &stops, GradientOptions options) {
+	if (!gc) {
+		return;
+	}
+
+	CGPoint ptStart = CGPointMake(rc.left, rc.top);
+	CGPoint ptEnd = CGPointMake(rc.left, rc.bottom);
+	if (options == GradientOptions::leftToRight) {
+		ptEnd = CGPointMake(rc.right, rc.top);
+	}
+
+	std::vector<CGFloat> components;
+	std::vector<CGFloat> locations;
+	for (const ColourStop &stop : stops) {
+		locations.push_back(stop.position);
+		components.push_back(stop.colour.GetRedComponent());
+		components.push_back(stop.colour.GetGreenComponent());
+		components.push_back(stop.colour.GetBlueComponent());
+		components.push_back(stop.colour.GetAlphaComponent());
+	}
+
+	CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
+	if (!colorSpace) {
+		return;
+	}
+
+	CGGradientRef gradiantRef = CGGradientCreateWithColorComponents(colorSpace,
+									components.data(),
+									locations.data(),
+									locations.size());
+	if (gradiantRef) {
+		CGContextSaveGState(gc);
+		CGRect rect = PRectangleToCGRect(rc);
+		CGContextClipToRect(gc, rect);
+		CGContextBeginPath(gc);
+		CGContextAddRect(gc, rect);
+		CGContextClosePath(gc);
+		CGContextDrawLinearGradient(gc, gradiantRef, ptStart, ptEnd, 0);
+		CGGradientRelease(gradiantRef);
+		CGContextRestoreGState(gc);
+	}
+	CGColorSpaceRelease(colorSpace);
+}
+
 static void ProviderReleaseData(void *, const void *data, size_t) {
 	const unsigned char *pixels = static_cast<const unsigned char *>(data);
 	delete []pixels;
@@ -653,8 +901,8 @@ static CGImageRef ImageCreateFromRGBA(int width, int height, const unsigned char
 	// Create an RGB color space.
 	CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
 	if (colorSpace) {
-		const int bitmapBytesPerRow = ((int) width * 4);
-		const int bitmapByteCount = (bitmapBytesPerRow * (int) height);
+		const int bitmapBytesPerRow = width * 4;
+		const int bitmapByteCount = bitmapBytesPerRow * height;
 
 		// Create a data provider.
 		CGDataProviderRef dataProvider = 0;
@@ -724,8 +972,8 @@ void SurfaceImpl::CopyImageRectangle(Surface &surfaceSource, PRectangle srcRect,
 	CGRect dst = PRectangleToCGRect(dstRect);
 
 	/* source from QuickDrawToQuartz2D.pdf on developer.apple.com */
-	float w = (float) CGImageGetWidth(image);
-	float h = (float) CGImageGetHeight(image);
+	const float w = static_cast<float>(CGImageGetWidth(image));
+	const float h = static_cast<float>(CGImageGetHeight(image));
 	CGRect drawRect = CGRectMake(0, 0, w, h);
 	if (!CGRectEqualToRect(src, dst)) {
 		CGFloat sx = CGRectGetWidth(dst) / CGRectGetWidth(src);
@@ -776,19 +1024,27 @@ void SurfaceImpl::Copy(PRectangle rc, Scintilla::Point from, Surface &surfaceSou
 
 //--------------------------------------------------------------------------------------------------
 
-void SurfaceImpl::DrawTextNoClip(PRectangle rc, Font &font_, XYPOSITION ybase, const char *s, int len,
-				 ColourDesired fore, ColourDesired back) {
-	FillRectangle(rc, back);
-	DrawTextTransparent(rc, font_, ybase, s, len, fore);
+// Bidirectional text support for Arabic and Hebrew.
+
+std::unique_ptr<IScreenLineLayout> SurfaceImpl::Layout(const IScreenLine *screenLine) {
+	return std::make_unique<ScreenLineLayout>(screenLine);
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void SurfaceImpl::DrawTextClipped(PRectangle rc, Font &font_, XYPOSITION ybase, const char *s, int len,
+void SurfaceImpl::DrawTextNoClip(PRectangle rc, Font &font_, XYPOSITION ybase, std::string_view text,
+				 ColourDesired fore, ColourDesired back) {
+	FillRectangle(rc, back);
+	DrawTextTransparent(rc, font_, ybase, text, fore);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void SurfaceImpl::DrawTextClipped(PRectangle rc, Font &font_, XYPOSITION ybase, std::string_view text,
 				  ColourDesired fore, ColourDesired back) {
 	CGContextSaveGState(gc);
 	CGContextClipToRect(gc, PRectangleToCGRect(rc));
-	DrawTextNoClip(rc, font_, ybase, s, len, fore, back);
+	DrawTextNoClip(rc, font_, ybase, text, fore, back);
 	CGContextRestoreGState(gc);
 }
 
@@ -849,53 +1105,53 @@ CFStringEncoding EncodingFromCharacterSet(bool unicode, int characterSet) {
 	}
 }
 
-void SurfaceImpl::DrawTextTransparent(PRectangle rc, Font &font_, XYPOSITION ybase, const char *s, int len,
+void SurfaceImpl::DrawTextTransparent(PRectangle rc, Font &font_, XYPOSITION ybase, std::string_view text,
 				      ColourDesired fore) {
 	CFStringEncoding encoding = EncodingFromCharacterSet(unicodeMode, FontCharacterSet(font_));
-	ColourDesired colour(fore.AsLong());
+	ColourDesired colour(fore.AsInteger());
 	CGColorRef color = CGColorCreateGenericRGB(colour.GetRed()/255.0, colour.GetGreen()/255.0, colour.GetBlue()/255.0, 1.0);
 
-	QuartzTextStyle *style = static_cast<QuartzTextStyle *>(font_.GetID());
+	QuartzTextStyle *style = TextStyleFromFont(font_);
 	style->setCTStyleColor(color);
 
 	CGColorRelease(color);
 
-	textLayout->setText(reinterpret_cast<const UInt8 *>(s), len, encoding, *static_cast<QuartzTextStyle *>(font_.GetID()));
-	textLayout->draw(rc.left, ybase);
-}
-
-static size_t utf8LengthFromLead(unsigned char uch) {
-	if (uch >= (0x80 + 0x40 + 0x20 + 0x10)) {
-		return 4;
-	} else if (uch >= (0x80 + 0x40 + 0x20)) {
-		return 3;
-	} else if (uch >= (0x80)) {
-		return 2;
-	} else {
-		return 1;
-	}
+	textLayout->setText(text, encoding, *style);
+	textLayout->draw(gc, rc.left, ybase);
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len, XYPOSITION *positions) {
+void SurfaceImpl::MeasureWidths(Font &font_, std::string_view text, XYPOSITION *positions) {
 	CFStringEncoding encoding = EncodingFromCharacterSet(unicodeMode, FontCharacterSet(font_));
-	textLayout->setText(reinterpret_cast<const UInt8 *>(s), len, encoding, *static_cast<QuartzTextStyle *>(font_.GetID()));
+	const CFStringEncoding encodingUsed =
+		textLayout->setText(text, encoding, *TextStyleFromFont(font_));
 
 	CTLineRef mLine = textLayout->getCTLine();
-	assert(mLine != NULL);
+	assert(mLine);
+
+	if (encodingUsed != encoding) {
+		// Switched to MacRoman to make work so treat as single byte encoding.
+		for (int i=0; i<text.length(); i++) {
+			CGFloat xPosition = CTLineGetOffsetForStringIndex(mLine, i+1, nullptr);
+			positions[i] = static_cast<XYPOSITION>(xPosition);
+		}
+		return;
+	}
 
 	if (unicodeMode) {
 		// Map the widths given for UTF-16 characters back onto the UTF-8 input string
 		CFIndex fit = textLayout->getStringLength();
 		int ui=0;
-		const unsigned char *us = reinterpret_cast<const unsigned char *>(s);
 		int i=0;
+		std::vector<CGFloat> linePositions(fit);
+		GetPositions(mLine, linePositions);
 		while (ui<fit) {
-			size_t lenChar = utf8LengthFromLead(us[i]);
-			size_t codeUnits = (lenChar < 4) ? 1 : 2;
-			CGFloat xPosition = CTLineGetOffsetForStringIndex(mLine, ui+codeUnits, NULL);
-			for (unsigned int bytePos=0; (bytePos<lenChar) && (i<len); bytePos++) {
+			const unsigned char uch = text[i];
+			const unsigned int byteCount = UTF8BytesOfLead[uch];
+			const int codeUnits = UTF16LengthFromUTF8ByteCount(byteCount);
+			const CGFloat xPosition = linePositions[ui];
+			for (unsigned int bytePos=0; (bytePos<byteCount) && (i<text.length()); bytePos++) {
 				positions[i++] = static_cast<XYPOSITION>(xPosition);
 			}
 			ui += codeUnits;
@@ -903,21 +1159,21 @@ void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len, XYPOSITION
 		XYPOSITION lastPos = 0.0f;
 		if (i > 0)
 			lastPos = positions[i-1];
-		while (i<len) {
+		while (i<text.length()) {
 			positions[i++] = lastPos;
 		}
 	} else if (codePage) {
 		int ui = 0;
-		for (int i=0; i<len;) {
-			size_t lenChar = DBCSIsLeadByte(codePage, s[i]) ? 2 : 1;
+		for (int i=0; i<text.length();) {
+			size_t lenChar = DBCSIsLeadByte(codePage, text[i]) ? 2 : 1;
 			CGFloat xPosition = CTLineGetOffsetForStringIndex(mLine, ui+1, NULL);
-			for (unsigned int bytePos=0; (bytePos<lenChar) && (i<len); bytePos++) {
+			for (unsigned int bytePos=0; (bytePos<lenChar) && (i<text.length()); bytePos++) {
 				positions[i++] = static_cast<XYPOSITION>(xPosition);
 			}
 			ui++;
 		}
 	} else {	// Single byte encoding
-		for (int i=0; i<len; i++) {
+		for (int i=0; i<text.length(); i++) {
 			CGFloat xPosition = CTLineGetOffsetForStringIndex(mLine, i+1, NULL);
 			positions[i] = static_cast<XYPOSITION>(xPosition);
 		}
@@ -925,27 +1181,16 @@ void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len, XYPOSITION
 
 }
 
-XYPOSITION SurfaceImpl::WidthText(Font &font_, const char *s, int len) {
+XYPOSITION SurfaceImpl::WidthText(Font &font_, std::string_view text) {
 	if (font_.GetID()) {
 		CFStringEncoding encoding = EncodingFromCharacterSet(unicodeMode, FontCharacterSet(font_));
-		textLayout->setText(reinterpret_cast<const UInt8 *>(s), len, encoding, *static_cast<QuartzTextStyle *>(font_.GetID()));
+		textLayout->setText(text, encoding, *TextStyleFromFont(font_));
 
 		return static_cast<XYPOSITION>(textLayout->MeasureStringWidth());
 	}
 	return 1;
 }
 
-XYPOSITION SurfaceImpl::WidthChar(Font &font_, char ch) {
-	char str[2] = { ch, '\0' };
-	if (font_.GetID()) {
-		CFStringEncoding encoding = EncodingFromCharacterSet(unicodeMode, FontCharacterSet(font_));
-		textLayout->setText(reinterpret_cast<const UInt8 *>(str), 1, encoding, *static_cast<QuartzTextStyle *>(font_.GetID()));
-
-		return textLayout->MeasureStringWidth();
-	} else
-		return 1;
-}
-
 // This string contains a good range of characters to test for size.
 const char sizeString[] = "`~!@#$%^&*()-_=+\\|[]{};:\"\'<,>.?/1234567890"
 			  "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
@@ -954,7 +1199,7 @@ XYPOSITION SurfaceImpl::Ascent(Font &font_) {
 	if (!font_.GetID())
 		return 1;
 
-	float ascent = static_cast<QuartzTextStyle *>(font_.GetID())->getAscent();
+	float ascent = TextStyleFromFont(font_)->getAscent();
 	return ascent + 0.5f;
 
 }
@@ -963,7 +1208,7 @@ XYPOSITION SurfaceImpl::Descent(Font &font_) {
 	if (!font_.GetID())
 		return 1;
 
-	float descent = static_cast<QuartzTextStyle *>(font_.GetID())->getDescent();
+	float descent = TextStyleFromFont(font_)->getDescent();
 	return descent + 0.5f;
 
 }
@@ -982,10 +1227,9 @@ XYPOSITION SurfaceImpl::AverageCharWidth(Font &font_) {
 	if (!font_.GetID())
 		return 1;
 
-	const int sizeStringLength = ELEMENTS(sizeString);
-	XYPOSITION width = WidthText(font_, sizeString, sizeStringLength);
+	XYPOSITION width = WidthText(font_, sizeString);
 
-	return (int)((width / (float) sizeStringLength) + 0.5);
+	return round(width / strlen(sizeString));
 }
 
 void SurfaceImpl::SetClip(PRectangle rc) {
@@ -1005,6 +1249,9 @@ void SurfaceImpl::SetDBCSMode(int codePage_) {
 		codePage = codePage_;
 }
 
+void SurfaceImpl::SetBidiR2L(bool) {
+}
+
 Surface *Surface::Allocate(int) {
 	return new SurfaceImpl();
 }
@@ -1028,7 +1275,7 @@ static CGFloat ScreenMax() {
 
 //--------------------------------------------------------------------------------------------------
 
-PRectangle Window::GetPosition() {
+PRectangle Window::GetPosition() const {
 	if (wid) {
 		NSRect rect;
 		id idWin = (__bridge id)(wid);
@@ -1080,8 +1327,8 @@ void Window::SetPosition(PRectangle rc) {
 
 //--------------------------------------------------------------------------------------------------
 
-void Window::SetPositionRelative(PRectangle rc, Window window) {
-	PRectangle rcOther = window.GetPosition();
+void Window::SetPositionRelative(PRectangle rc, const Window *window) {
+	PRectangle rcOther = window->GetPosition();
 	rc.left += rcOther.left;
 	rc.right += rcOther.left;
 	rc.top += rcOther.top;
@@ -1091,7 +1338,7 @@ void Window::SetPositionRelative(PRectangle rc, Window window) {
 
 //--------------------------------------------------------------------------------------------------
 
-PRectangle Window::GetClientPosition() {
+PRectangle Window::GetClientPosition() const {
 	// This means, in MacOS X terms, get the "frame bounds". Call GetPosition, just like on Win32.
 	return GetPosition();
 }
@@ -1455,7 +1702,7 @@ void ListBoxImpl::Create(Window & /*parent*/, int /*ctrlID*/, Scintilla::Point p
 
 	NSRect lbRect = NSMakeRect(pt.x, pt.y, 120, lineHeight * desiredVisibleRows);
 	NSWindow *winLB = [[NSWindow alloc] initWithContentRect: lbRect
-						      styleMask: NSBorderlessWindowMask
+						      styleMask: NSWindowStyleMaskBorderless
 							backing: NSBackingStoreBuffered
 							  defer: NO];
 	[winLB setLevel: NSFloatingWindowLevel];
@@ -1495,7 +1742,7 @@ void ListBoxImpl::Create(Window & /*parent*/, int /*ctrlID*/, Scintilla::Point p
 void ListBoxImpl::SetFont(Font &font_) {
 	// NSCell setFont takes an NSFont* rather than a CTFontRef but they
 	// are the same thing toll-free bridged.
-	QuartzTextStyle *style = static_cast<QuartzTextStyle *>(font_.GetID());
+	QuartzTextStyle *style = TextStyleFromFont(font_);
 	font.Release();
 	font.SetID(new QuartzTextStyle(*style));
 	NSFont *pfont = (__bridge NSFont *)style->getFontRef();
@@ -1533,7 +1780,7 @@ PRectangle ListBoxImpl::GetDesiredRect() {
 
 	if (Length() > rows) {
 		[scroller setHasVerticalScroller: YES];
-		rcDesired.right += [NSScroller scrollerWidthForControlSize: NSRegularControlSize
+		rcDesired.right += [NSScroller scrollerWidthForControlSize: NSControlSizeRegular
 							     scrollerStyle: NSScrollerStyleLegacy];
 	} else {
 		[scroller setHasVerticalScroller: NO];
@@ -1572,7 +1819,7 @@ void ListBoxImpl::Append(char *s, int type) {
 	ld.Add(count, type, s);
 
 	Scintilla::SurfaceImpl surface;
-	XYPOSITION width = surface.WidthText(font, s, static_cast<int>(strlen(s)));
+	XYPOSITION width = surface.WidthText(font, s);
 	if (width > maxItemWidth) {
 		maxItemWidth = width;
 		colText.width = maxItemWidth;
@@ -1593,7 +1840,7 @@ void ListBoxImpl::SetList(const char *list, char separator, char typesep) {
 	size_t count = strlen(list) + 1;
 	std::vector<char> words(list, list+count);
 	char *startword = words.data();
-	char *numword = NULL;
+	char *numword = nullptr;
 	int i = 0;
 	for (; words[i]; i++) {
 		if (words[i] == separator) {
@@ -1602,7 +1849,7 @@ void ListBoxImpl::SetList(const char *list, char separator, char typesep) {
 				*numword = '\0';
 			Append(startword, numword?atoi(numword + 1):-1);
 			startword = words.data() + i + 1;
-			numword = NULL;
+			numword = nullptr;
 		} else if (words[i] == typesep) {
 			numword = words.data() + i;
 		}
@@ -1703,7 +1950,7 @@ void ListBoxImpl::SelectionChange() {
 
 // ListBox is implemented by the ListBoxImpl class.
 
-ListBox::ListBox() {
+ListBox::ListBox() noexcept {
 }
 
 ListBox::~ListBox() {
@@ -1724,7 +1971,7 @@ void Window::Destroy() {
 	if (wid) {
 		id idWin = (__bridge id)(wid);
 		if ([idWin isKindOfClass: [NSWindow class]]) {
-			CFBridgingRelease(wid);
+			[idWin close];
 		}
 	}
 	wid = nullptr;
@@ -1753,7 +2000,7 @@ NSMenu
 
 //----------------- Menu ---------------------------------------------------------------------------
 
-Menu::Menu()
+Menu::Menu() noexcept
 	: mid(0) {
 }
 
@@ -1778,34 +2025,6 @@ void Menu::Show(Point, Window &) {
 	// takes care to show it properly.
 }
 
-//----------------- ElapsedTime --------------------------------------------------------------------
-
-// ElapsedTime is used for precise performance measurements during development
-// and not for anything a user sees.
-
-ElapsedTime::ElapsedTime() {
-	struct timeval curTime;
-	gettimeofday(&curTime, NULL);
-
-	bigBit = curTime.tv_sec;
-	littleBit = curTime.tv_usec;
-}
-
-double ElapsedTime::Duration(bool reset) {
-	struct timeval curTime;
-	gettimeofday(&curTime, NULL);
-	long endBigBit = curTime.tv_sec;
-	long endLittleBit = curTime.tv_usec;
-	double result = 1000000.0 * (endBigBit - bigBit);
-	result += endLittleBit - littleBit;
-	result /= 1000000.0;
-	if (reset) {
-		bigBit = endBigBit;
-		littleBit = endLittleBit;
-	}
-	return result;
-}
-
 //----------------- Platform -----------------------------------------------------------------------
 
 ColourDesired Platform::Chrome() {
@@ -1912,11 +2131,11 @@ void Platform::Assert(const char *c, const char *file, int line) {
  * Implements the platform specific part of library loading.
  *
  * @param modulePath The path to the module to load.
- * @return A library instance or NULL if the module could not be found or another problem occurred.
+ * @return A library instance or nullptr if the module could not be found or another problem occurred.
  */
 DynamicLibrary *DynamicLibrary::Load(const char * /* modulePath */) {
 	// Not implemented.
-	return NULL;
+	return nullptr;
 }
 
 //--------------------------------------------------------------------------------------------------
