@@ -5,26 +5,28 @@
 // Copyright 1998-2006 by Neil Hodgson <neilh@scintilla.org>
 // The License.txt file describes the conditions under which this software may be distributed.
 
-#include <stdlib.h>
-#include <string.h>
-#include <ctype.h>
-#include <stdio.h>
-#include <stddef.h>
-#include <fcntl.h>
-#include <sys/stat.h>
-#include <time.h>
+#include <cstddef>
+#include <cstdlib>
+#include <cstring>
+#include <cctype>
+#include <cstdio>
+#include <ctime>
 
 #include <string>
 #include <vector>
-#include <set>
 #include <map>
+#include <set>
 #include <memory>
 #include <sstream>
 
-#include "Scintilla.h"
+#include <fcntl.h>
+#include <sys/stat.h>
+
 #include "ILexer.h"
+#include "Scintilla.h"
 
 #include "GUI.h"
+#include "ScintillaWindow.h"
 
 #include "StringList.h"
 #include "StringHelpers.h"
@@ -68,7 +70,7 @@
 
 struct PDFStyle {
 	std::string fore;
-	int font;
+	int font=0;
 };
 
 static const char *PDFfontNames[] = {
@@ -118,7 +120,7 @@ void SciTEBase::SaveToPDF(const FilePath &saveName) {
 		~PDFObjectTracker() {
 		}
 		void write(const char *objectData) {
-			size_t length = strlen(objectData);
+			const size_t length = strlen(objectData);
 			// note binary write used, open with "wb"
 			fwrite(objectData, sizeof(char), length, fp);
 		}
@@ -180,7 +182,7 @@ void SciTEBase::SaveToPDF(const FilePath &saveName) {
 		long pageWidth, pageHeight;
 		GUI::Rectangle pageMargin;
 		//
-		PDFRender() {
+		PDFRender() : buffer{} {
 			pageStarted = false;
 			firstLine = false;
 			pageCount = 0;
@@ -514,8 +516,8 @@ void SciTEBase::SaveToPDF(const FilePath &saveName) {
 	} else {
 		int lineIndex = 0;
 		for (int i = 0; i < lengthDoc; i++) {
-			char ch = acc[i];
-			int style = acc.StyleAt(i);
+			const char ch = acc[i];
+			const int style = acc.StyleAt(i);
 
 			if (ch == '\t') {
 				// expand tabs
