@@ -5,26 +5,28 @@
 // Copyright 1998-2006 by Neil Hodgson <neilh@scintilla.org>
 // The License.txt file describes the conditions under which this software may be distributed.
 
-#include <stdlib.h>
-#include <string.h>
-#include <ctype.h>
-#include <stdio.h>
-#include <stddef.h>
-#include <fcntl.h>
-#include <sys/stat.h>
-#include <time.h>
+#include <cstddef>
+#include <cstdlib>
+#include <cstring>
+#include <cctype>
+#include <cstdio>
+#include <ctime>
 
 #include <string>
 #include <vector>
-#include <set>
 #include <map>
+#include <set>
 #include <memory>
 #include <sstream>
 
-#include "Scintilla.h"
+#include <fcntl.h>
+#include <sys/stat.h>
+
 #include "ILexer.h"
+#include "Scintilla.h"
 
 #include "GUI.h"
+#include "ScintillaWindow.h"
 
 #include "StringList.h"
 #include "StringHelpers.h"
@@ -97,7 +99,7 @@ void SciTEBase::SaveToXML(const FilePath &saveName) {
 
 		fputs("<document xmlns='http://www.scintila.org/scite.rng'", fp);
 		fprintf(fp, " filename='%s'",
-		        static_cast<const char *>(filePath.Name().AsUTF8().c_str()));
+		        filePath.Name().AsUTF8().c_str());
 		fprintf(fp, " type='%s'", "unknown");
 		fprintf(fp, " version='%s'", "1.0");
 		fputs(">\n", fp);
