@@ -5,26 +5,30 @@
 // Copyright 1998-2005 by Neil Hodgson <neilh@scintilla.org>
 // The License.txt file describes the conditions under which this software may be distributed.
 
-#include <stdlib.h>
-#include <string.h>
-#include <ctype.h>
-#include <stdio.h>
-#include <fcntl.h>
-#include <sys/stat.h>
-#include <time.h>
+#include <cstdlib>
+#include <cstring>
+#include <cctype>
+#include <cstdio>
+#include <ctime>
+#include <cerrno>
 
 #include <string>
+#include <string_view>
 #include <vector>
 #include <algorithm>
 
+#include <fcntl.h>
+
 #if defined(__unix__)
 
 #include <unistd.h>
-
 #include <dirent.h>
-#include <errno.h>
 
-#else
+#endif
+
+#include <sys/stat.h>
+
+#if !defined(__unix__)
 
 #include <io.h>
 
@@ -41,6 +45,7 @@
 #include "Scintilla.h"
 
 #include "GUI.h"
+#include "ScintillaWindow.h"
 
 #include "FilePath.h"
 
@@ -49,8 +54,6 @@ const GUI::gui_char pathSepString[] = "/";
 const GUI::gui_char pathSepChar = '/';
 const GUI::gui_char listSepString[] = ":";
 const GUI::gui_char configFileVisibilityString[] = ".";
-const GUI::gui_char fileRead[] = "rb";
-const GUI::gui_char fileWrite[] = "wb";
 #endif
 #ifdef WIN32
 // Windows
@@ -58,9 +61,15 @@ const GUI::gui_char pathSepString[] = GUI_TEXT("\\");
 const GUI::gui_char pathSepChar = '\\';
 const GUI::gui_char listSepString[] = GUI_TEXT(";");
 const GUI::gui_char configFileVisibilityString[] = GUI_TEXT("");
+#endif
+
 const GUI::gui_char fileRead[] = GUI_TEXT("rb");
 const GUI::gui_char fileWrite[] = GUI_TEXT("wb");
-#endif
+
+namespace {
+const GUI::gui_char currentDirectory[] = GUI_TEXT(".");
+const GUI::gui_char parentDirectory[] = GUI_TEXT("..");
+}
 
 FilePath::FilePath(const GUI::gui_char *fileName_) : fileName(fileName_ ? fileName_ : GUI_TEXT("")) {}
 
@@ -95,7 +104,7 @@ void FilePath::SetDirectory(FilePath const &directory) {
 }
 
 void FilePath::Init() {
-	fileName = GUI_TEXT("");
+	fileName.clear();
 }
 
 bool FilePath::operator==(const FilePath &other) const {
@@ -124,7 +133,7 @@ bool FilePath::IsSet() const {
 }
 
 bool FilePath::IsUntitled() const {
-	size_t dirEnd = fileName.rfind(pathSepChar);
+	const size_t dirEnd = fileName.rfind(pathSepChar);
 	return (dirEnd == GUI::gui_string::npos) || (!fileName[dirEnd+1]);
 }
 
@@ -170,7 +179,7 @@ std::string FilePath::AsUTF8() const {
 }
 
 FilePath FilePath::Name() const {
-	size_t dirEnd = fileName.rfind(pathSepChar);
+	const size_t dirEnd = fileName.rfind(pathSepChar);
 	if (dirEnd != GUI::gui_string::npos)
 		return fileName.substr(dirEnd + 1);
 	else
@@ -178,8 +187,8 @@ FilePath FilePath::Name() const {
 }
 
 FilePath FilePath::BaseName() const {
-	size_t dirEnd = fileName.rfind(pathSepChar);
-	size_t extStart = fileName.rfind('.');
+	const size_t dirEnd = fileName.rfind(pathSepChar);
+	const size_t extStart = fileName.rfind('.');
 	if (dirEnd != GUI::gui_string::npos) {
 		if (extStart > dirEnd) {
 			return FilePath(fileName.substr(dirEnd + 1, extStart - dirEnd - 1));
@@ -227,22 +236,6 @@ static size_t strlen(const wchar_t *str) {
 	return wcslen(str);
 }
 
-static wchar_t *strcpy(wchar_t *strDestination, const wchar_t *strSource) {
-	return wcscpy(strDestination, strSource);
-}
-
-static int strcmp(const wchar_t *a, const wchar_t *b) {
-	return wcscmp(a,b);
-}
-
-static wchar_t *strchr(wchar_t *str, wchar_t c) {
-	return wcschr(str, c);
-}
-
-static wchar_t *strrchr(wchar_t *str, wchar_t c) {
-	return wcsrchr(str, c);
-}
-
 static wchar_t *getcwd(wchar_t *buffer, int maxlen) {
 	return _wgetcwd(buffer, maxlen);
 }
@@ -279,44 +272,48 @@ FilePath FilePath::NormalizePath() const {
 	if (fileName.empty()) {
 		return FilePath();
 	}
-	GUI::gui_string path = fileName;
+	GUI::gui_string path(fileName);
 #ifdef WIN32
 	// Convert unix path separators to Windows
 	std::replace(path.begin(), path.end(), L'/', pathSepChar);
 #endif
-	GUI::gui_string absPathString(fileName.length() + 1, 0);
-	GUI::gui_char *absPath = &absPathString[0];
-	GUI::gui_char *cur = absPath;
-	*cur = '\0';
-	GUI::gui_char *part = &path[0];
-	if (*part == pathSepChar) {
-		*cur++ = pathSepChar;
-		*cur = '\0';
-		part++;
+	GUI::gui_string_view source = path;
+	GUI::gui_string absPathString;
+	// Result is always same size or shorter so can allocate once for maximum
+	// possible and avoid a reallocation for common path lengths.
+	absPathString.reserve(fileName.length());
+	if (source.front() == pathSepChar) {
+		absPathString.push_back(pathSepChar);
+		source.remove_prefix(1);
 	}
 	// Split into components and remove x/.. and .
-	while (part) {
-		GUI::gui_char *next = strchr(part, pathSepChar);
-		if (next)
-			*next++ = 0;
-		GUI::gui_char *last;
-		if (strcmp(part, GUI_TEXT(".")) == 0)
-			;
-		else if (strcmp(part, GUI_TEXT("..")) == 0 && (last = strrchr(absPath, pathSepChar)) != NULL) {
-			if (last > absPath)
-				cur = last;
-			else
-				cur = last + 1;
-			*cur = '\0';
-		} else {
-			if (cur > absPath && *(cur - 1) != pathSepChar)
-				*cur++ = pathSepChar;
-			strcpy(cur, part);
-			cur += strlen(part);
+	while (true) {
+		const size_t separator = source.find_first_of(pathSepChar);
+		// If no pathSepChar then separator == npos, so substr -> rest of string, OK
+		const GUI::gui_string_view part = source.substr(0, separator);
+		if (part != currentDirectory) {
+			bool appendPart = true;
+			if (part == parentDirectory) {
+				const size_t last = absPathString.find_last_of(pathSepChar);
+				if (last != GUI::gui_string::npos) {
+					// Erase the last component from the path separator, unless that would erase
+					// the entire string, in which case leave a single path separator.
+					const size_t truncPoint = (last > 0) ? last : last + 1;
+					absPathString.erase(truncPoint);
+					appendPart = false;
+				}
+			}
+			if (appendPart) {
+				if (!absPathString.empty() && (absPathString.back() != pathSepChar))
+					absPathString.push_back(pathSepChar);
+				absPathString.append(part);
+			}
 		}
-		part = next;
+		if (separator == GUI::gui_string::npos)	// Consumed last part
+			break;
+		source.remove_prefix(separator+1);
 	}
-	return FilePath(absPath);
+	return FilePath(absPathString);
 }
 
 /**
@@ -370,11 +367,13 @@ void FilePath::List(FilePathSet &directories, FilePathSet &files) const {
 	if (hFind != INVALID_HANDLE_VALUE) {
 		bool complete = false;
 		while (!complete) {
-			if ((strcmp(findFileData.cFileName, GUI_TEXT(".")) != 0) && (strcmp(findFileData.cFileName, GUI_TEXT("..")) != 0)) {
+			const std::wstring_view entryName = findFileData.cFileName;
+			if ((entryName != currentDirectory) && (entryName != parentDirectory)) {
+				FilePath pathFull(AsInternal(), findFileData.cFileName);
 				if (findFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
-					directories.push_back(FilePath(AsInternal(), findFileData.cFileName));
+					directories.push_back(pathFull);
 				} else {
-					files.push_back(FilePath(AsInternal(), findFileData.cFileName));
+					files.push_back(pathFull);
 				}
 			}
 			if (!::FindNextFileW(hFind, &findFileData)) {
@@ -392,7 +391,8 @@ void FilePath::List(FilePathSet &directories, FilePathSet &files) const {
 	}
 	struct dirent *ent;
 	while ((ent = readdir(dp)) != NULL) {
-		if ((strcmp(ent->d_name, ".") != 0) && (strcmp(ent->d_name, "..") != 0)) {
+		std::string_view entryName = ent->d_name;
+		if ((entryName != currentDirectory) && (entryName != parentDirectory)) {
 			FilePath pathFull(AsInternal(), ent->d_name);
 			if (pathFull.IsDirectory()) {
 				directories.push_back(pathFull);
@@ -419,14 +419,14 @@ FILE *FilePath::Open(const GUI::gui_char *mode) const {
 /// Size of block for file reading.
 static const size_t readBlockSize = 64 * 1024;
 
-std::vector<char> FilePath::Read() const {
-	std::vector<char> data;
+std::string FilePath::Read() const {
+	std::string data;
 	FILE *fp = Open(fileRead);
 	if (fp) {
-		std::vector<char> block(readBlockSize);
+		std::string block(readBlockSize, '\0');
 		size_t lenBlock = fread(&block[0], 1, block.size(), fp);
 		while (lenBlock > 0) {
-			data.insert(data.end(), block.begin(), block.begin() + lenBlock);
+			data.append(block, 0, lenBlock);
 			lenBlock = fread(&block[0], 1, block.size(), fp);
 		}
 		fclose(fp);
@@ -514,18 +514,47 @@ bool FilePath::IsDirectory() const {
 		return false;
 }
 
+namespace {
+
 #ifdef _WIN32
-static void Lowercase(GUI::gui_string &s) {
-	int chars = ::LCMapString(LOCALE_SYSTEM_DEFAULT, LCMAP_LOWERCASE, s.c_str(), static_cast<int>(s.size())+1, NULL, 0);
+void Lowercase(GUI::gui_string &s) {
+	const int chars = ::LCMapString(LOCALE_SYSTEM_DEFAULT, LCMAP_LOWERCASE, s.c_str(), static_cast<int>(s.size())+1, NULL, 0);
 	std::vector<wchar_t> vc(chars);
 	::LCMapString(LOCALE_SYSTEM_DEFAULT, LCMAP_LOWERCASE, s.c_str(), static_cast<int>(s.size())+1, &vc[0], chars);
 	s = &vc[0];
 }
 #endif
 
-static bool EndMatches(GUI::gui_string const &s, GUI::gui_string const &end) {
-	return (s.size() >= end.size()) &&
-		(std::equal(s.begin() + s.size() - end.size(), s.end(), end.begin()));
+bool PatternMatch(GUI::gui_string_view pattern, GUI::gui_string_view text) {
+	if (pattern == text) {
+		return true;
+	} else if (pattern.empty()) {
+		return false;
+	} else if (pattern.front() == '*') {
+		pattern.remove_prefix(1);
+		if (pattern.empty()) {
+			return true;
+		}
+		while (!text.empty()) {
+			if (PatternMatch(pattern, text)) {
+				return true;
+			}
+			text.remove_prefix(1);
+		}
+	} else if (text.empty()) {
+		return false;
+	} else if (pattern.front() == '?') {
+		pattern.remove_prefix(1);
+		text.remove_prefix(1);
+		return PatternMatch(pattern, text);
+	} else if (pattern.front() == text.front()) {
+		pattern.remove_prefix(1);
+		text.remove_prefix(1);
+		return PatternMatch(pattern, text);
+	}
+	return false;
+}
+
 }
 
 bool FilePath::Matches(const GUI::gui_char *pattern) const {
@@ -539,14 +568,8 @@ bool FilePath::Matches(const GUI::gui_char *pattern) const {
 	size_t start = 0;
 	while (start < pat.length()) {
 		const GUI::gui_char *patElement = pat.c_str() + start;
-		if (patElement[0] == '*') {
-			if (EndMatches(nameCopy, patElement + 1)) {
-				return true;
-			}
-		} else {
-			if (nameCopy == patElement) {
-				return true;
-			}
+		if (PatternMatch(patElement, nameCopy)) {
+			return true;
 		}
 		start += strlen(patElement) + 1;
 	}
@@ -576,7 +599,7 @@ static bool MakeLongPath(const GUI::gui_char* shortPath, GUI::gui_string &longPa
 		HMODULE hModule = ::GetModuleHandle(TEXT("kernel32.dll"));
 		if (hModule) {
 			// attempt to get GetLongPathNameW implemented in Windows 2000 or newer
-			pfnGetLong = (GetLongSig)::GetProcAddress(hModule, "GetLongPathNameW");
+			pfnGetLong = reinterpret_cast<GetLongSig>(::GetProcAddress(hModule, "GetLongPathNameW"));
 		}
 	}
 
@@ -622,6 +645,16 @@ void FilePath::FixName() {
 #endif
 }
 
+bool FilePath::CaseSensitive() {
+#if defined (__APPLE__)
+	return false;
+#elif defined(__unix__)
+	return true;
+#else
+	return false;
+#endif
+}
+
 std::string CommandExecute(const GUI::gui_char *command, const GUI::gui_char *directoryForRun) {
 	std::string output;
 #ifdef _WIN32
