@@ -13,7 +13,7 @@ extern const GUI::gui_char fileRead[];
 extern const GUI::gui_char fileWrite[];
 
 #if defined(__unix__)
-#include <limits.h>
+#include <climits>
 #ifdef PATH_MAX
 #define MAX_PATH PATH_MAX
 #else
@@ -31,11 +31,16 @@ public:
 	FilePath(const GUI::gui_char *fileName_ = GUI_TEXT(""));
 	FilePath(const GUI::gui_string &fileName_);
 	FilePath(FilePath const &directory, FilePath const &name);
+	FilePath(FilePath const &) = default;
+	FilePath(FilePath &&) = default;
+	FilePath &operator=(FilePath const &) = default;
+	FilePath &operator=(FilePath &&) = default;
+	virtual ~FilePath() = default;
 	void Set(const GUI::gui_char *fileName_);
 	void Set(FilePath const &other);
 	void Set(FilePath const &directory, FilePath const &name);
 	void SetDirectory(FilePath const &directory);
-	void Init();
+	virtual void Init();
 	bool SameNameAs(const GUI::gui_char *other) const;
 	bool SameNameAs(const FilePath &other) const;
 	bool operator==(const FilePath &other) const;
@@ -58,13 +63,14 @@ public:
 	bool SetWorkingDirectory() const;
 	void List(FilePathSet &directories, FilePathSet &files) const;
 	FILE *Open(const GUI::gui_char *mode) const;
-	std::vector<char> Read() const;
+	std::string Read() const;
 	void Remove() const;
 	time_t ModifiedTime() const;
 	long long GetFileLength() const;
 	bool Exists() const;
 	bool IsDirectory() const;
 	bool Matches(const GUI::gui_char *pattern) const;
+	static bool CaseSensitive();
 };
 
 std::string CommandExecute(const GUI::gui_char *command, const GUI::gui_char *directoryForRun);
