@@ -5,9 +5,9 @@
 // Copyright 2011 by Neil Hodgson <neilh@scintilla.org>
 // The License.txt file describes the conditions under which this software may be distributed.
 
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
+#include <cstdlib>
+#include <cstring>
+#include <cstdio>
 
 #include <string>
 #include <vector>
@@ -27,14 +27,14 @@
 
 #endif
 
+#include "ILoader.h"
 #include "Scintilla.h"
-#include "ILexer.h"
-
-#include "Mutex.h"
 
 #include "GUI.h"
+#include "ScintillaWindow.h"
 
 #include "FilePath.h"
+#include "Mutex.h"
 #include "Cookie.h"
 #include "Worker.h"
 #include "FileWorker.h"
@@ -74,7 +74,7 @@ void FileLoader::Execute() {
 			::Sleep(sleepTime);
 #endif
 			lenFile = convert.convert(&data[0], lenFile);
-			char *dataBlock = convert.getNewBuf();
+			const char *dataBlock = convert.getNewBuf();
 			err = pLoader->AddData(dataBlock, static_cast<int>(lenFile));
 			IncrementProgress(static_cast<int>(lenFile));
 			if (et.Duration() > nextProgress) {
@@ -84,9 +84,9 @@ void FileLoader::Execute() {
 			lenFile = fread(&data[0], 1, blockSize, fp);
 			if ((lenFile == 0) && (err == 0)) {
 				// Handle case where convert is holding a lead surrogate but no more data
-				size_t lenFileTrail = convert.convert(NULL, lenFile);
+				const size_t lenFileTrail = convert.convert(NULL, lenFile);
 				if (lenFileTrail) {
-					char *dataTrail = convert.getNewBuf();
+					const char *dataTrail = convert.getNewBuf();
 					err = pLoader->AddData(dataTrail, static_cast<int>(lenFileTrail));
 				}
 			}
@@ -133,7 +133,7 @@ void FileStorer::Execute() {
 		}
 		convert.setfile(fp);
 		std::vector<char> data(blockSize + 1);
-		size_t lengthDoc = size;
+		const size_t lengthDoc = size;
 		size_t grabSize;
 		for (size_t i = 0; i < lengthDoc && (!Cancelling()); i += grabSize) {
 #ifdef __unix__
