@@ -10,8 +10,6 @@
 #ifndef GUI_H
 #define GUI_H
 
-#define ELEMENTS(a) (sizeof(a) / sizeof(a[0]))
-
 namespace GUI {
 
 class Point {
@@ -19,7 +17,7 @@ public:
 	int x;
 	int y;
 
-	explicit Point(int x_=0, int y_=0) : x(x_), y(y_) {
+	explicit Point(int x_=0, int y_=0) noexcept : x(x_), y(y_) {
 	}
 };
 
@@ -30,16 +28,16 @@ public:
 	int right;
 	int bottom;
 
-	Rectangle(int left_=0, int top_=0, int right_=0, int bottom_ = 0) :
+	Rectangle(int left_=0, int top_=0, int right_=0, int bottom_ = 0) noexcept :
 		left(left_), top(top_), right(right_), bottom(bottom_) {
 	}
-	bool Contains(Point pt) const {
+	bool Contains(Point pt) const noexcept {
 		return (pt.x >= left) && (pt.x <= right) &&
 			(pt.y >= top) && (pt.y <= bottom);
 	}
-	int Width() const { return right - left; }
-	int Height() const { return bottom - top; }
-	bool operator==(const Rectangle &other) const {
+	int Width() const noexcept { return right - left; }
+	int Height() const noexcept { return bottom - top; }
+	bool operator==(const Rectangle &other) const noexcept {
 		return (left == other.left) &&
 			(top == other.top) &&
 			(right == other.right) &&
@@ -53,6 +51,7 @@ public:
 
 typedef char gui_char;
 typedef std::string gui_string;
+typedef std::string_view gui_string_view;
 
 #define GUI_TEXT(q) q
 
@@ -62,6 +61,7 @@ typedef std::string gui_string;
 
 typedef wchar_t gui_char;
 typedef std::wstring gui_string;
+typedef std::wstring_view gui_string_view;
 
 #define GUI_TEXT(q) L##q
 
@@ -74,24 +74,32 @@ gui_string StringFromInteger(long i);
 gui_string StringFromLongLong(long long i);
 gui_string HexStringFromInteger(long i);
 
+std::string LowerCaseUTF8(std::string_view sv);
+
 typedef void *WindowID;
 class Window {
 protected:
 	WindowID wid;
 public:
-	Window() : wid(0) {
+	Window() noexcept : wid(0) {
 	}
-	Window &operator=(WindowID wid_) {
+	Window(Window const &) = default;
+	Window(Window &&) = default;
+	Window &operator=(Window const &) = default;
+	Window &operator=(Window &&) = default;
+	Window &operator=(WindowID wid_) noexcept {
 		wid = wid_;
 		return *this;
 	}
-	WindowID GetID() const {
+	virtual ~Window() = default;
+
+	WindowID GetID() const noexcept {
 		return wid;
 	}
-	void SetID(WindowID wid_) {
+	void SetID(WindowID wid_) noexcept {
 		wid = wid_;
 	}
-	bool Created() const {
+	bool Created() const noexcept {
 		return wid != 0;
 	}
 	void Destroy();
@@ -108,9 +116,9 @@ typedef void *MenuID;
 class Menu {
 	MenuID mid;
 public:
-	Menu() : mid(0) {
+	Menu() noexcept : mid(0) {
 	}
-	MenuID GetID() const {
+	MenuID GetID() const noexcept {
 		return mid;
 	}
 	void CreatePopUp();
@@ -126,67 +134,8 @@ public:
 	double Duration(bool reset=false);
 };
 
-struct ScintillaFailure {
-	sptr_t status;
-	explicit ScintillaFailure(sptr_t status_) : status(status_) {
-	}
-};
-
-class ScintillaWindow : public Window {
-	// Deleted so ScintillaWindow objects can not be copied.
-	ScintillaWindow(const ScintillaWindow &source) = delete;
-	ScintillaWindow &operator=(const ScintillaWindow &) = delete;
-	SciFnDirect fn;
-	sptr_t ptr;
+class ScintillaPrimitive : public Window {
 public:
-	sptr_t status;
-	ScintillaWindow() : fn(0), ptr(0), status() {
-	}
-	void SetID(WindowID wid_) {
-		wid = wid_;
-		fn = 0;
-		ptr = 0;
-		if (wid) {
-			fn = reinterpret_cast<SciFnDirect>(
-				Send(SCI_GETDIRECTFUNCTION, 0, 0));
-			ptr = Send(SCI_GETDIRECTPOINTER, 0, 0);
-		}
-	}
-	bool CanCall() const {
-		return wid && fn && ptr;
-	}
-	int Call(unsigned int msg, uptr_t wParam=0, sptr_t lParam=0) {
-		switch (msg) {
-		case SCI_CREATEDOCUMENT:
-		case SCI_CREATELOADER:
-		case SCI_PRIVATELEXERCALL:
-		case SCI_GETDIRECTFUNCTION:
-		case SCI_GETDIRECTPOINTER:
-		case SCI_GETDOCPOINTER:
-		case SCI_GETCHARACTERPOINTER:
-			throw ScintillaFailure(SC_STATUS_FAILURE);
-		}
-		if (!fn)
-			throw ScintillaFailure(SC_STATUS_FAILURE);
-		const sptr_t retVal = fn(ptr, msg, wParam, lParam);
-		status = fn(ptr, SCI_GETSTATUS, 0, 0);
-		if (status > 0 && status < SC_STATUS_WARN_START)
-			throw ScintillaFailure(status);
-		return static_cast<int>(retVal);
-	}
-	sptr_t CallReturnPointer(unsigned int msg, uptr_t wParam=0, sptr_t lParam=0) {
-		sptr_t retVal = fn(ptr, msg, wParam, lParam);
-		status = fn(ptr, SCI_GETSTATUS, 0, 0);
-		if (status > 0 && status < SC_STATUS_WARN_START)
-			throw ScintillaFailure(status);
-		return retVal;
-	}
-	int CallPointer(unsigned int msg, uptr_t wParam, void *s) {
-		return Call(msg, wParam, reinterpret_cast<sptr_t>(s));
-	}
-	int CallString(unsigned int msg, uptr_t wParam, const char *s) {
-		return Call(msg, wParam, reinterpret_cast<sptr_t>(s));
-	}
 	// Send is the basic method and can be used between threads on Win32
 	sptr_t Send(unsigned int msg, uptr_t wParam=0, sptr_t lParam=0);
 };
