@@ -11,6 +11,7 @@
 #include "Scintilla.h"
 
 #include "GUI.h"
+#include "ScintillaWindow.h"
 
 #include "MatchMarker.h"
 
@@ -29,14 +30,14 @@ std::vector<LineRange> LinesBreak(GUI::ScintillaWindow *pSci) {
 			rangePriority.lineEnd = lineEnd;
 		lineRanges.push_back(rangePriority);
 		if (rangePriority.lineEnd < lineEnd)
-			lineRanges.push_back(LineRange(rangePriority.lineEnd, lineEnd));
+			lineRanges.emplace_back(rangePriority.lineEnd, lineEnd);
 		if (rangePriority.lineStart > 0)
-			lineRanges.push_back(LineRange(0, rangePriority.lineStart));
+			lineRanges.emplace_back(0, rangePriority.lineStart);
 	}
 	return lineRanges;
 }
 
-MatchMarker::MatchMarker() : 
+MatchMarker::MatchMarker() :
 	pSci(0), styleMatch(-1), flagsMatch(0), indicator(0), bookMark(-1) {
 }
 
@@ -68,7 +69,7 @@ void MatchMarker::Continue() {
 	// Remove old indicators if any exist.
 	pSci->Call(SCI_SETINDICATORCURRENT, indicator);
 
-	LineRange rangeSearch = lineRanges[0];
+	const LineRange rangeSearch = lineRanges[0];
 	int lineEndSegment = rangeSearch.lineStart + segment;
 	if (lineEndSegment > rangeSearch.lineEnd)
 		lineEndSegment = rangeSearch.lineEnd;
