@@ -56,7 +56,15 @@ public:
 	RecentFile() {
 		scrollPosition = 0;
 	}
-	void Init() {
+	RecentFile(const FilePath &path_, SelectedRange selection_, int scrollPosition_) :
+		FilePath(path_), selection(selection_), scrollPosition(scrollPosition_) {
+	}
+	RecentFile(RecentFile const &) = default;
+	RecentFile(RecentFile &&) = default;
+	RecentFile &operator=(RecentFile const &) = default;
+	RecentFile &operator=(RecentFile &&) = default;
+	~RecentFile() override = default;
+	void Init() override {
 		FilePath::Init();
 		selection.position = INVALID_POSITION;
 		selection.anchor = INVALID_POSITION;
@@ -64,8 +72,9 @@ public:
 	}
 };
 
-struct BufferState : public RecentFile {
+struct BufferState {
 public:
+	RecentFile file;
 	std::vector<int> foldState;
 	std::vector<int> bookmarks;
 };
@@ -78,8 +87,9 @@ public:
 
 struct FileWorker;
 
-class Buffer : public RecentFile {
+class Buffer {
 public:
+	RecentFile file;
 	sptr_t doc;
 	bool isDirty;
 	bool isReadOnly;
@@ -98,12 +108,13 @@ public:
 	PropSetFile props;
 	enum FutureDo { fdNone=0, fdFinishSave=1 } futureDo;
 	Buffer() :
-			RecentFile(), doc(0), isDirty(false), isReadOnly(false), failedSave(false), useMonoFont(false), lifeState(empty),
+			file(), doc(0), isDirty(false), isReadOnly(false), failedSave(false), useMonoFont(false), lifeState(empty),
 			unicodeMode(uni8Bit), fileModTime(0), fileModLastAsk(0), documentModTime(0),
 			findMarks(fmNone), pFileWorker(0), futureDo(fdNone) {}
 
+	~Buffer() = default;
 	void Init() {
-		RecentFile::Init();
+		file.Init();
 		isDirty = false;
 		isReadOnly = false;
 		failedSave = false;
@@ -122,7 +133,7 @@ public:
 	}
 
 	void SetTimeFromFile() {
-		fileModTime = ModifiedTime();
+		fileModTime = file.ModifiedTime();
 		fileModLastAsk = fileModTime;
 		documentModTime = fileModTime;
 		failedSave = false;
@@ -268,7 +279,7 @@ public:
 	bool read;
 	Localization() : PropSetFile(true), read(false) {
 	}
-	GUI::gui_string Text(const char *s, bool retainIfNotFound=true);
+	GUI::gui_string Text(const char *s, bool retainIfNotFound=true) override;
 	void SetMissing(const std::string &missing_) {
 		missing = missing_;
 	}
@@ -343,6 +354,8 @@ public:
 	}
 };
 
+class IEditorConfig;
+
 class SciTEBase : public ExtensionAPI, public Searcher, public WorkerListener {
 protected:
 	bool needIdle;
@@ -389,6 +402,10 @@ protected:
 	int diagnosticStyleStart;
 	enum { diagnosticStyles=4};
 
+	bool stripTrailingSpaces;
+	bool ensureFinalLineEnd;
+	bool ensureConsistentLineEnds;
+
 	bool indentOpening;
 	bool indentClosing;
 	bool indentMaintain;
@@ -523,6 +540,8 @@ protected:
 
 	PropSetFile propsStatus;	// Not attached to a file but need SetInteger method.
 
+	std::unique_ptr<IEditorConfig> editorConfig;
+
 	enum { bufferMax = IDM_IMPORT - IDM_BUFFER };
 	BufferList buffers;
 
@@ -676,7 +695,7 @@ protected:
 	void UserStripShow(const char * /* description */) override {}
 	void UserStripSet(int /* control */, const char * /* value */) override {}
 	void UserStripSetList(int /* control */, const char * /* value */) override {}
-	const char *UserStripValue(int /* control */) override { return 0; }
+	std::string UserStripValue(int /* control */) override { return std::string(); }
 	virtual void ShowBackgroundProgress(const GUI::gui_string & /* explanation */, size_t /* size */, size_t /* progress */) {}
 	Sci_CharacterRange GetSelection();
 	SelectedRange GetSelectedRange();
@@ -717,7 +736,7 @@ protected:
 	void FailedSaveMessageBox(const FilePath &filePathSaving);
 	virtual void FindMessageBox(const std::string &msg, const std::string *findItem = 0) = 0;
 	bool FindReplaceAdvanced() const;
-	int FindInTarget(std::string findWhatText, int startPosition, int endPosition);
+	int FindInTarget(const std::string &findWhatText, int startPosition, int endPosition);
 	// Implement Searcher
 	void SetFindText(const char *sFind) override;
 	void SetFind(const char *sFind) override;
@@ -727,7 +746,7 @@ protected:
 	void MoveBack() override;
 	void ScrollEditorIfNeeded() override;
 	int FindNext(bool reverseDirection, bool showWarnings=true, bool allowRegExp=true) override;
-	virtual void HideMatch() override;
+	void HideMatch() override;
 	virtual void FindIncrement() = 0;
 	int IncrementSearchMode();
 	virtual void FindInFiles() = 0;
@@ -787,6 +806,7 @@ protected:
 	void UpdateStatusBar(bool bUpdateSlowData) override;
 	int GetLineLength(int line);
 	int GetCurrentLineNumber();
+	int GetCurrentColumnNumber();
 	int GetCurrentScrollPosition();
 	virtual void AddCommand(const std::string &cmd, const std::string &dir,
 	        JobSubsystem jobType, const std::string &input = "",
@@ -846,7 +866,7 @@ protected:
 	void DeleteFileStackMenu();
 	void SetFileStackMenu();
 	bool AddFileToBuffer(const BufferState &bufferState);
-	void AddFileToStack(const FilePath &file, SelectedRange selection, int scrollPos);
+	void AddFileToStack(const RecentFile &file);
 	void RemoveFileFromStack(const FilePath &file);
 	RecentFile GetFilePosition();
 	void DisplayAround(const RecentFile &rf);
@@ -916,7 +936,7 @@ protected:
 	void AskMacroList();
 	bool StartMacroList(const char *words);
 	void ContinueMacroList(const char *stext);
-	bool ProcessCommandLine(GUI::gui_string &args, int phase);
+	bool ProcessCommandLine(const GUI::gui_string &args, int phase);
 	virtual bool IsStdinBlocked();
 	void OpenFromStdin(bool UseOutputPane);
 	void OpenFilesFromStdin();
@@ -932,10 +952,9 @@ protected:
 	void SendOneProperty(const char *kind, const char *key, const char *val);
 	void PropertyFromDirector(const char *arg);
 	void PropertyToDirector(const char *arg);
-
 	// ExtensionAPI
 	sptr_t Send(Pane p, unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0) override;
-	char *Range(Pane p, int start, int end) override;
+	std::string Range(Pane p, int start, int end) override;
 	void Remove(Pane p, int start, int end) override;
 	void Insert(Pane p, int pos, const char *s) override;
 	void Trace(const char *s) override;
@@ -962,9 +981,11 @@ public:
 
 	explicit SciTEBase(Extension *ext = 0);
 	// Deleted copy-constructor and assignment operator.
-	explicit SciTEBase(const SciTEBase&) = delete;
-	void operator=(const SciTEBase&) = delete;
-	virtual ~SciTEBase();
+	SciTEBase(const SciTEBase &) = delete;
+	SciTEBase(SciTEBase &&) = delete;
+	void operator=(const SciTEBase &) = delete;
+	void operator=(SciTEBase &&) = delete;
+	~SciTEBase() override;
 
 	void Finalise();
 
@@ -972,7 +993,7 @@ public:
 
 	virtual bool PerformOnNewThread(Worker *pWorker) = 0;
 	// WorkerListener
-	virtual void PostOnMainThread(int cmd, Worker *pWorker) override = 0;
+	void PostOnMainThread(int cmd, Worker *pWorker) override = 0;
 	virtual void WorkerCommand(int cmd, Worker *pWorker);
 };
 
