@@ -5,26 +5,27 @@
 // Copyright 1998-2010 by Neil Hodgson <neilh@scintilla.org>
 // The License.txt file describes the conditions under which this software may be distributed.
 
-#include <stdlib.h>
-#include <string.h>
-#include <ctype.h>
-#include <stdio.h>
-#include <stddef.h>
-#include <time.h>
-#include <assert.h>
+#include <cstddef>
+#include <cstdlib>
+#include <cassert>
+#include <cstring>
+#include <cctype>
+#include <cstdio>
+#include <ctime>
 
 #include <string>
 #include <vector>
-#include <set>
 #include <map>
+#include <set>
 #include <algorithm>
 #include <memory>
 
+#include "ILoader.h"
 #include "Scintilla.h"
 #include "SciLexer.h"
-#include "ILexer.h"
 
 #include "GUI.h"
+#include "ScintillaWindow.h"
 
 #include "StringList.h"
 #include "StringHelpers.h"
@@ -50,7 +51,7 @@ void Buffer::DocumentModified() {
 
 bool Buffer::NeedsSave(int delayBeforeSave) const {
 	const time_t now = time(0);
-	return now && documentModTime && isDirty && !pFileWorker && (now-documentModTime > delayBeforeSave) && !IsUntitled() && !failedSave;
+	return now && documentModTime && isDirty && !pFileWorker && (now-documentModTime > delayBeforeSave) && !file.IsUntitled() && !failedSave;
 }
 
 void Buffer::CompleteLoading() {
@@ -128,7 +129,7 @@ int BufferList::GetDocumentByName(const FilePath &filename, bool excludeCurrent)
 		return -1;
 	}
 	for (int i = 0;i < length;i++) {
-		if ((!excludeCurrent || i != current) && buffers[i].SameNameAs(filename)) {
+		if ((!excludeCurrent || i != current) && buffers[i].file.SameNameAs(filename)) {
 			return i;
 		}
 	}
@@ -150,7 +151,7 @@ void BufferList::RemoveInvisible(int index) {
 
 void BufferList::RemoveCurrent() {
 	// Delete and move up to fill gap but ensure doc pointer is saved.
-	sptr_t currentDoc = buffers[current].doc;
+	const sptr_t currentDoc = buffers[current].doc;
 	buffers[current].CompleteLoading();
 	for (int i = current;i < length - 1;i++) {
 		buffers[i] = buffers[i + 1];
@@ -298,7 +299,7 @@ BackgroundActivities BufferList::CountBackgroundActivities() const {
 					bg.loaders++;
 				else
 					bg.storers++;
-				bg.fileNameLast = buffers[i].AsInternal();
+				bg.fileNameLast = buffers[i].file.AsInternal();
 				bg.totalWork += buffers[i].pFileWorker->SizeJob();
 				bg.totalProgress += buffers[i].pFileWorker->ProgressMade();
 			}
@@ -367,7 +368,7 @@ void SciTEBase::SwitchDocumentAt(int index, sptr_t pdoc) {
 	if (index < 0 || index >= buffers.size()) {
 		return;
 	}
-	sptr_t pdocOld = buffers.buffers[index].doc;
+	const sptr_t pdocOld = buffers.buffers[index].doc;
 	buffers.buffers[index].doc = pdoc;
 	if (pdocOld) {
 		wEditor.Call(SCI_RELEASEDOCUMENT, 0, pdocOld);
@@ -401,12 +402,12 @@ void SciTEBase::SetDocumentAt(int index, bool updateStack) {
 			extender->InitBuffer(0);
 	}
 
-	Buffer bufferNext = buffers.buffers[buffers.Current()];
-	SetFileName(bufferNext);
+	const Buffer &bufferNext = buffers.buffers[buffers.Current()];
+	SetFileName(bufferNext.file);
 	propsDiscovered = bufferNext.props;
 	propsDiscovered.superPS = &propsLocal;
 	wEditor.Call(SCI_SETDOCPOINTER, 0, GetDocumentAt(buffers.Current()));
-	bool restoreBookmarks = bufferNext.lifeState == Buffer::readAll;
+	const bool restoreBookmarks = bufferNext.lifeState == Buffer::readAll;
 	PerformDeferredTasks();
 	if (bufferNext.lifeState == Buffer::readAll) {
 		CompleteOpen(ocCompleteSwitch);
@@ -420,7 +421,7 @@ void SciTEBase::SetDocumentAt(int index, bool updateStack) {
 	if (lineNumbers && lineNumbersExpand)
 		SetLineNumberWidth();
 
-	DisplayAround(bufferNext);
+	DisplayAround(bufferNext.file);
 	if (restoreBookmarks) {
 		// Restoring a session does not restore the scroll position
 		// so make the selection visible.
@@ -437,15 +438,15 @@ void SciTEBase::SetDocumentAt(int index, bool updateStack) {
 }
 
 void SciTEBase::UpdateBuffersCurrent() {
-	int currentbuf = buffers.Current();
+	const int currentbuf = buffers.Current();
 
 	if ((buffers.length > 0) && (currentbuf >= 0) && (buffers.GetVisible(currentbuf))) {
 		Buffer &bufferCurrent = buffers.buffers[currentbuf];
-		bufferCurrent.Set(filePath);
+		bufferCurrent.file.Set(filePath);
 		if (bufferCurrent.lifeState != Buffer::reading && bufferCurrent.lifeState != Buffer::readAll) {
-			bufferCurrent.selection.position = wEditor.Call(SCI_GETCURRENTPOS);
-			bufferCurrent.selection.anchor = wEditor.Call(SCI_GETANCHOR);
-			bufferCurrent.scrollPosition = GetCurrentScrollPosition();
+			bufferCurrent.file.selection.position = wEditor.Call(SCI_GETCURRENTPOS);
+			bufferCurrent.file.selection.anchor = wEditor.Call(SCI_GETANCHOR);
+			bufferCurrent.file.scrollPosition = GetCurrentScrollPosition();
 
 			// Retrieve fold state and store in buffer state info
 
@@ -564,7 +565,7 @@ void SciTEBase::LoadSessionFile(const GUI::gui_char *sessionName) {
 }
 
 void SciTEBase::RestoreRecentMenu() {
-	SelectedRange sr(0,0);
+	const SelectedRange sr(0,0);
 
 	DeleteFileStackMenu();
 
@@ -573,7 +574,7 @@ void SciTEBase::RestoreRecentMenu() {
 		std::string propStr = propsSession.GetString(propKey.c_str());
 		if (propStr == "")
 			continue;
-		AddFileToStack(GUI::StringFromUTF8(propStr), sr, 0);
+		AddFileToStack(RecentFile(GUI::StringFromUTF8(propStr), sr, 0));
 	}
 }
 
@@ -614,7 +615,7 @@ std::string StringFromLines(const std::vector<int> &lines) {
 void SciTEBase::RestoreFromSession(const Session &session) {
 	for (const BufferState &buffer : session.buffers)
 		AddFileToBuffer(buffer);
-	int iBuffer = buffers.GetDocumentByName(session.pathActive);
+	const int iBuffer = buffers.GetDocumentByName(session.pathActive);
 	if (iBuffer >= 0)
 		SetDocumentAt(iBuffer);
 }
@@ -650,21 +651,21 @@ void SciTEBase::RestoreSession() {
 			continue;
 
 		BufferState bufferState;
-		bufferState.Set(GUI::StringFromUTF8(propStr));
+		bufferState.file.Set(GUI::StringFromUTF8(propStr));
 
 		propKey = IndexPropKey("buffer", i, "current");
 		if (propsSession.GetInt(propKey.c_str()))
-			session.pathActive = bufferState;
+			session.pathActive = bufferState.file;
 
 		propKey = IndexPropKey("buffer", i, "scroll");
 		const int scroll = propsSession.GetInt(propKey.c_str());
-		bufferState.scrollPosition = scroll;
+		bufferState.file.scrollPosition = scroll;
 
 		propKey = IndexPropKey("buffer", i, "position");
 		const int pos = propsSession.GetInt(propKey.c_str());
 
-		bufferState.selection.anchor = pos - 1;
-		bufferState.selection.position = bufferState.selection.anchor;
+		bufferState.file.selection.anchor = pos - 1;
+		bufferState.file.selection.position = bufferState.file.selection.anchor;
 
 		if (props.GetInt("session.bookmarks")) {
 			propKey = IndexPropKey("buffer", i, "bookmarks");
@@ -757,16 +758,16 @@ void SciTEBase::SaveSessionFile(const GUI::gui_char *sessionName) {
 	if (props.GetInt("buffers") && (!defaultSession || props.GetInt("save.session"))) {
 		const int curr = buffers.Current();
 		for (int i = 0; i < buffers.lengthVisible; i++) {
-			if (buffers.buffers[i].IsSet() && !buffers.buffers[i].IsUntitled()) {
-				Buffer &buff = buffers.buffers[i];
+			const Buffer &buff = buffers.buffers[i];
+			if (buff.file.IsSet() && !buff.file.IsUntitled()) {
 				std::string propKey = IndexPropKey("buffer", i, "path");
-				fprintf(sessionFile, "\n%s=%s\n", propKey.c_str(), buff.AsUTF8().c_str());
+				fprintf(sessionFile, "\n%s=%s\n", propKey.c_str(), buff.file.AsUTF8().c_str());
 
-				const int pos = buff.selection.position + 1;
+				const int pos = buff.file.selection.position + 1;
 				propKey = IndexPropKey("buffer", i, "position");
 				fprintf(sessionFile, "%s=%d\n", propKey.c_str(), pos);
 
-				const int scroll = buff.scrollPosition;
+				const int scroll = buff.file.scrollPosition;
 				propKey = IndexPropKey("buffer", i, "scroll");
 				fprintf(sessionFile, "%s=%d\n", propKey.c_str(), scroll);
 
@@ -805,9 +806,9 @@ void SciTEBase::SaveSessionFile(const GUI::gui_char *sessionName) {
 
 void SciTEBase::SetIndentSettings() {
 	// Get default values
-	int useTabs = props.GetInt("use.tabs", 1);
-	int tabSize = props.GetInt("tabsize");
-	int indentSize = props.GetInt("indent.size");
+	const int useTabs = props.GetInt("use.tabs", 1);
+	const int tabSize = props.GetInt("tabsize");
+	const int indentSize = props.GetInt("indent.size");
 	// Either set the settings related to the extension or the default ones
 	std::string fileNameForExtension = ExtensionFileName();
 	std::string useTabsChars = props.GetNewExpandString("use.tabs.",
@@ -850,10 +851,8 @@ void SciTEBase::New() {
 
 	propsDiscovered.Clear();
 
-	if ((buffers.size() == 1) && (!buffers.buffers[0].IsUntitled())) {
-		AddFileToStack(buffers.buffers[0],
-		        buffers.buffers[0].selection,
-		        buffers.buffers[0].scrollPosition);
+	if ((buffers.size() == 1) && (!buffers.buffers[0].file.IsUntitled())) {
+		AddFileToStack(buffers.buffers[0].file);
 	}
 
 	// If the current buffer is the initial untitled, clean buffer then overwrite it,
@@ -861,14 +860,14 @@ void SciTEBase::New() {
 	if ((buffers.length > 1) ||
 	        (buffers.Current() != 0) ||
 	        (buffers.buffers[0].isDirty) ||
-	        (!buffers.buffers[0].IsUntitled())) {
+	        (!buffers.buffers[0].file.IsUntitled())) {
 		if (buffers.size() == buffers.length) {
 			Close(false, false, true);
 		}
 		buffers.SetCurrent(buffers.Add());
 	}
 
-	sptr_t doc = GetDocumentAt(buffers.Current());
+	const sptr_t doc = GetDocumentAt(buffers.Current());
 	wEditor.Call(SCI_SETDOCPOINTER, 0, doc);
 
 	FilePath curDirectory(filePath.Directory());
@@ -915,7 +914,7 @@ void SciTEBase::RestoreState(const Buffer &buffer, bool restoreBookmarks) {
 
 void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew) {
 	bool closingLast = true;
-	int index = buffers.Current();
+	const int index = buffers.Current();
 	if ((index >= 0) && buffers.initialised) {
 		buffers.buffers[index].CancelLoad();
 	}
@@ -935,8 +934,7 @@ void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew)
 	} else if (buffers.size() > 1) {
 		if (buffers.Current() >= 0 && buffers.Current() < buffers.length) {
 			UpdateBuffersCurrent();
-			Buffer buff = buffers.buffers[buffers.Current()];
-			AddFileToStack(buff, buff.selection, buff.scrollPosition);
+			AddFileToStack(CurrentBufferConst()->file);
 		}
 		closingLast = (buffers.lengthVisible == 1) && !buffers.buffers[0].pFileWorker;
 		if (closingLast) {
@@ -959,12 +957,12 @@ void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew)
 			if (extender && !makingRoomForNew)
 				extender->ActivateBuffer(buffers.Current());
 		}
-		Buffer bufferNext = buffers.buffers[buffers.Current()];
+		const Buffer &bufferNext = buffers.buffers[buffers.Current()];
 
 		if (updateUI)
-			SetFileName(bufferNext);
+			SetFileName(bufferNext.file);
 		else
-			filePath = bufferNext;
+			filePath = bufferNext.file;
 		propsDiscovered = bufferNext.props;
 		propsDiscovered.superPS = &propsLocal;
 		wEditor.Call(SCI_SETDOCPOINTER, 0, GetDocumentAt(buffers.Current()));
@@ -983,7 +981,7 @@ void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew)
 			CheckReload();
 		if (updateUI) {
 			RestoreState(bufferNext, false);
-			DisplayAround(bufferNext);
+			DisplayAround(bufferNext.file);
 		}
 	}
 
@@ -1002,14 +1000,14 @@ void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew)
 }
 
 void SciTEBase::CloseTab(int tab) {
-	int tabCurrent = buffers.Current();
+	const int tabCurrent = buffers.Current();
 	if (tab == tabCurrent) {
 		if (SaveIfUnsure() != saveCancelled) {
 			Close();
 			WindowSetFocus(wEditor);
 		}
 	} else {
-		FilePath fpCurrent = buffers.buffers[tabCurrent].AbsolutePath();
+		FilePath fpCurrent = buffers.buffers[tabCurrent].file.AbsolutePath();
 		SetDocumentAt(tab);
 		if (SaveIfUnsure() != saveCancelled) {
 			Close();
@@ -1032,7 +1030,7 @@ void SciTEBase::CloseAllBuffers(bool loadingSession) {
 SciTEBase::SaveResult SciTEBase::SaveAllBuffers(bool alwaysYes) {
 	SaveResult choice = saveCompleted;
 	UpdateBuffersCurrent();
-	int currentBuffer = buffers.Current();
+	const int currentBuffer = buffers.Current();
 	for (int i = 0; (i < buffers.lengthVisible) && (choice != saveCancelled); i++) {
 		if (buffers.buffers[i].isDirty) {
 			SetDocumentAt(i);
@@ -1051,9 +1049,9 @@ SciTEBase::SaveResult SciTEBase::SaveAllBuffers(bool alwaysYes) {
 
 void SciTEBase::SaveTitledBuffers() {
 	UpdateBuffersCurrent();
-	int currentBuffer = buffers.Current();
+	const int currentBuffer = buffers.Current();
 	for (int i = 0; i < buffers.lengthVisible; i++) {
-		if (buffers.buffers[i].isDirty && !buffers.buffers[i].IsUntitled()) {
+		if (buffers.buffers[i].isDirty && !buffers.buffers[i].file.IsUntitled()) {
 			SetDocumentAt(i);
 			Save();
 		}
@@ -1085,12 +1083,12 @@ void SciTEBase::ShiftTab(int indexFrom, int indexTo) {
 
 	TabSelect(indexTo);
 
-	DisplayAround(buffers.buffers[buffers.Current()]);
+	DisplayAround(buffers.buffers[buffers.Current()].file);
 }
 
 void SciTEBase::MoveTabRight() {
 	if (buffers.lengthVisible < 2) return;
-	int indexFrom = buffers.Current();
+	const int indexFrom = buffers.Current();
 	int indexTo = indexFrom + 1;
 	if (indexTo >= buffers.lengthVisible) indexTo = 0;
 	ShiftTab(indexFrom, indexTo);
@@ -1098,7 +1096,7 @@ void SciTEBase::MoveTabRight() {
 
 void SciTEBase::MoveTabLeft() {
 	if (buffers.lengthVisible < 2) return;
-	int indexFrom = buffers.Current();
+	const int indexFrom = buffers.Current();
 	int indexTo = indexFrom - 1;
 	if (indexTo < 0) indexTo = buffers.lengthVisible - 1;
 	ShiftTab(indexFrom, indexTo);
@@ -1141,10 +1139,10 @@ void SciTEBase::SetBuffersMenu() {
 		DestroyMenuItem(menuBuffers, IDM_BUFFER + pos);
 	}
 	if (buffers.size() > 1) {
-		int menuStart = 4;
+		const int menuStart = 4;
 		SetMenuItem(menuBuffers, menuStart, IDM_BUFFERSEP, GUI_TEXT(""));
 		for (pos = 0; pos < buffers.lengthVisible; pos++) {
-			int itemID = bufferCmdID + pos;
+			const int itemID = bufferCmdID + pos;
 			GUI::gui_string entry;
 			GUI::gui_string titleTab;
 
@@ -1163,13 +1161,13 @@ void SciTEBase::SetBuffersMenu() {
 			}
 #endif
 
-			if (buffers.buffers[pos].IsUntitled()) {
+			if (buffers.buffers[pos].file.IsUntitled()) {
 				GUI::gui_string untitled = localiser.Text("Untitled");
 				entry += untitled;
 				titleTab += untitled;
 			} else {
-				GUI::gui_string path = buffers.buffers[pos].AsInternal();
-				GUI::gui_string filename = buffers.buffers[pos].Name().AsInternal();
+				GUI::gui_string path = buffers.buffers[pos].file.AsInternal();
+				GUI::gui_string filename = buffers.buffers[pos].file.Name().AsInternal();
 
 				EscapeFilePathsForMenu(path);
 #if defined(_WIN32)
@@ -1185,8 +1183,8 @@ void SciTEBase::SetBuffersMenu() {
 			//strcat(entry, cpDirEnd + 1);
 
 			if (buffers.buffers[pos].isReadOnly && props.GetInt("read.only.indicator"))  {
-				entry=entry+GUI_TEXT(" |");
-				titleTab=titleTab+GUI_TEXT(" |");
+				entry += GUI_TEXT(" |");
+				titleTab += GUI_TEXT(" |");
 			}
 
 			if (buffers.buffers[pos].isDirty) {
@@ -1225,7 +1223,7 @@ void SciTEBase::SetFileStackMenu() {
 	if (recentFileStack[0].IsSet()) {
 		SetMenuItem(menuFile, MRU_START, IDM_MRU_SEP, GUI_TEXT(""));
 		for (int stackPos = 0; stackPos < fileStackMax; stackPos++) {
-			int itemID = fileStackCmdID + stackPos;
+			const int itemID = fileStackCmdID + stackPos;
 			if (recentFileStack[stackPos].IsSet()) {
 				GUI::gui_string sEntry;
 
@@ -1248,20 +1246,20 @@ void SciTEBase::SetFileStackMenu() {
 bool SciTEBase::AddFileToBuffer(const BufferState &bufferState) {
 	// Return whether file loads successfully
 	bool opened = false;
-	if (bufferState.Exists()) {
-		opened = Open(bufferState, static_cast<OpenFlags>(ofForceLoad));
+	if (bufferState.file.Exists()) {
+		opened = Open(bufferState.file, static_cast<OpenFlags>(ofForceLoad));
 		// If forced synchronous should set up position, foldState and bookmarks
 		if (opened) {
-			int iBuffer = buffers.GetDocumentByName(bufferState, false);
+			const int iBuffer = buffers.GetDocumentByName(bufferState.file, false);
 			if (iBuffer >= 0) {
-				buffers.buffers[iBuffer].scrollPosition = bufferState.scrollPosition;
-				buffers.buffers[iBuffer].selection = bufferState.selection;
+				buffers.buffers[iBuffer].file.scrollPosition = bufferState.file.scrollPosition;
+				buffers.buffers[iBuffer].file.selection = bufferState.file.selection;
 				buffers.buffers[iBuffer].foldState = bufferState.foldState;
 				buffers.buffers[iBuffer].bookmarks = bufferState.bookmarks;
 				if (buffers.buffers[iBuffer].lifeState == Buffer::open) {
 					// File was opened synchronously
 					RestoreState(buffers.buffers[iBuffer], true);
-					DisplayAround(buffers.buffers[iBuffer]);
+					DisplayAround(buffers.buffers[iBuffer].file);
 					wEditor.Call(SCI_SCROLLCARET);
 				}
 			}
@@ -1270,22 +1268,19 @@ bool SciTEBase::AddFileToBuffer(const BufferState &bufferState) {
 	return opened;
 }
 
-void SciTEBase::AddFileToStack(const FilePath &file, SelectedRange selection, int scrollPos) {
+void SciTEBase::AddFileToStack(const RecentFile &file) {
 	if (!file.IsSet())
 		return;
 	DeleteFileStackMenu();
 	// Only stack non-empty names
 	if (file.IsSet() && !file.IsUntitled()) {
-		int stackPos;
 		int eqPos = fileStackMax - 1;
-		for (stackPos = 0; stackPos < fileStackMax; stackPos++)
+		for (int stackPos = 0; stackPos < fileStackMax; stackPos++)
 			if (recentFileStack[stackPos].SameNameAs(file))
 				eqPos = stackPos;
-		for (stackPos = eqPos; stackPos > 0; stackPos--)
+		for (int stackPos = eqPos; stackPos > 0; stackPos--)
 			recentFileStack[stackPos] = recentFileStack[stackPos - 1];
-		recentFileStack[0].Set(file);
-		recentFileStack[0].selection = selection;
-		recentFileStack[0].scrollPosition = scrollPos;
+		recentFileStack[0] = file;
 	}
 	SetFileStackMenu();
 }
@@ -1317,8 +1312,8 @@ void SciTEBase::DisplayAround(const RecentFile &rf) {
 	if ((rf.selection.position != INVALID_POSITION) && (rf.selection.anchor != INVALID_POSITION)) {
 		SetSelection(rf.selection.anchor, rf.selection.position);
 
-		int curTop = wEditor.Call(SCI_GETFIRSTVISIBLELINE);
-		int lineTop = wEditor.Call(SCI_VISIBLEFROMDOCLINE, rf.scrollPosition);
+		const int curTop = wEditor.Call(SCI_GETFIRSTVISIBLELINE);
+		const int lineTop = wEditor.Call(SCI_VISIBLEFROMDOCLINE, rf.scrollPosition);
 		wEditor.Call(SCI_LINESCROLL, 0, lineTop - curTop);
 		wEditor.Call(SCI_CHOOSECARETX, 0, 0);
 	}
@@ -1389,8 +1384,8 @@ void SciTEBase::StackMenu(int pos) {
 				RecentFile rf = recentFileStack[pos];
 				// Already asked user so don't allow Open to ask again.
 				Open(rf, ofNoSaveIfDirty);
-				CurrentBuffer()->scrollPosition = rf.scrollPosition;
-				CurrentBuffer()->selection = rf.selection;
+				CurrentBuffer()->file.scrollPosition = rf.scrollPosition;
+				CurrentBuffer()->file.selection = rf.selection;
 				DisplayAround(rf);
 			}
 		}
@@ -1430,7 +1425,7 @@ void SciTEBase::SetToolsMenu() {
 	RemoveToolsMenu();
 	int menuPos = TOOLS_START;
 	for (int item = 0; item < toolMax; item++) {
-		int itemID = IDM_TOOLS + item;
+		const int itemID = IDM_TOOLS + item;
 		std::string prefix = "command.name.";
 		prefix += StdStringFromInteger(item);
 		prefix += ".";
@@ -1517,7 +1512,7 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 				startPath++;
 				const char *endPath = strchr(startPath, '\"');
 				if (endPath) {
-					ptrdiff_t length = endPath - startPath;
+					const ptrdiff_t length = endPath - startPath;
 					sourcePath.assign(startPath, length);
 					endPath++;
 					while (*endPath && !isdigitchar(*endPath)) {
@@ -1569,7 +1564,7 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 					endPath = strchr(endPath + 1, '(');
 				}
 				if (endPath) {
-					ptrdiff_t length = endPath - start;
+					const ptrdiff_t length = endPath - start;
 					sourcePath.assign(start, length);
 					endPath++;
 					return atoi(endPath) - 1;
@@ -1606,7 +1601,7 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 						space2--;
 					}
 
-					ptrdiff_t length = space2 - space;
+					const ptrdiff_t length = space2 - space;
 
 					if (length > 0) {
 						sourcePath.assign(space, length);
@@ -1620,7 +1615,7 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 			// perl
 			const char *at = strstr(cdoc, " at ");
 			const char *line = strstr(cdoc, " line ");
-			ptrdiff_t length = line - (at + 4);
+			const ptrdiff_t length = line - (at + 4);
 			if (at && line && length > 0) {
 				sourcePath.assign(at + 4, length);
 				line += 6;
@@ -1651,7 +1646,7 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 			if (line && file) {
 				const char *fileStart = file + lenFile + 1;
 				const char *quote = strstr(fileStart, "'");
-				size_t length = quote - fileStart;
+				const size_t length = quote - fileStart;
 				if (quote && length > 0) {
 					sourcePath.assign(fileStart, length);
 				}
@@ -1694,7 +1689,7 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 			const char *file = strstr(cdoc, idFile);
 			if (line && file && (line > file)) {
 				file += lenFile;
-				size_t length = line - file;
+				const size_t length = line - file;
 				sourcePath.assign(file, length);
 				line += lenLine;
 				return atoi(line) - 1;
@@ -1717,7 +1712,7 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 					while (*file && !isspacechar(*file)) {
 						file++;
 					}
-					size_t length = strlen(file);
+					const size_t length = strlen(file);
 					sourcePath.assign(file, length);
 					return atoi(line) - 1;
 				}
@@ -1741,7 +1736,7 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 				if (file) {
 					file++;
 					const char *endfile = strchr(file, ')');
-					size_t length = endfile - file;
+					const size_t length = endfile - file;
 					sourcePath.assign(file, length);
 					line++;
 					return atoi(line) - 1;
@@ -1762,7 +1757,7 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 			//const char *column = strstr(cdoc, idColumn);
 			if (line && file && (line > file)) {
 				file += lenFile;
-				size_t length = line - file;
+				const size_t length = line - file;
 				sourcePath.assign(file, length);
 				line += lenLine;
 				return atoi(line) - 1;
@@ -1783,7 +1778,7 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 			const char *lineend = strrchr(cdoc, ':');
 			if (line && file && (line > file)) {
 				file += lenFile;
-				size_t length = line - file;
+				const size_t length = line - file;
 				sourcePath.assign(file, length);
 				line += lenLine;
 				if ((lineend > line)) {
@@ -1824,7 +1819,7 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 				 */
 			const char *startPath = strrchr(cdoc, '(') + 1;
 			const char *endPath = strchr(startPath, ':');
-			ptrdiff_t length = endPath - startPath;
+			const ptrdiff_t length = endPath - startPath;
 			if (length > 0) {
 				sourcePath.assign(startPath, length);
 				const int sourceNumber = atoi(endPath + 1) - 1;
@@ -1839,7 +1834,7 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 			const char *startPath = cdoc + 4;
 			const char *endPath = strpbrk(startPath, "\t\r\n");
 			if (endPath) {
-				ptrdiff_t length = endPath - startPath;
+				const ptrdiff_t length = endPath - startPath;
 				sourcePath.assign(startPath, length);
 				return 0;
 			}
@@ -1882,7 +1877,7 @@ void SciTEBase::ShowMessages(int line) {
 		line--;
 	const int maxLine = wOutput.Call(SCI_GETLINECOUNT);
 	while ((line < maxLine) && (acc.StyleAt(acc.LineStart(line)) != SCE_ERR_CMD)) {
-		int startPosLine = wOutput.Call(SCI_POSITIONFROMLINE, line, 0);
+		const int startPosLine = wOutput.Call(SCI_POSITIONFROMLINE, line, 0);
 		const int lineEnd = wOutput.Call(SCI_GETLINEENDPOSITION, line, 0);
 		std::string message = GetRangeString(wOutput, startPosLine, lineEnd);
 		std::string source;
@@ -1893,7 +1888,7 @@ void SciTEBase::ShowMessages(int line) {
 			RemoveEscSeq(message);
 			style = SCE_ERR_GCC;
 		}
-		int sourceLine = DecodeMessage(message.c_str(), source, style, column);
+		const int sourceLine = DecodeMessage(message.c_str(), source, style, column);
 		Chomp(message, ':');
 		if (style == SCE_ERR_GCC) {
 			Chomp(message, ':');
@@ -1911,7 +1906,7 @@ void SciTEBase::ShowMessages(int line) {
 					}
 				}
 			}
-			int lenCurrent = wEditor.CallString(SCI_ANNOTATIONGETTEXT, sourceLine, NULL);
+			const int lenCurrent = wEditor.CallString(SCI_ANNOTATIONGETTEXT, sourceLine, NULL);
 			std::string msgCurrent(lenCurrent, '\0');
 			std::string stylesCurrent(lenCurrent, '\0');
 			if (lenCurrent) {
@@ -1943,7 +1938,7 @@ void SciTEBase::GoMessage(int dir) {
 	Sci_CharacterRange crange;
 	crange.cpMin = wOutput.Call(SCI_GETSELECTIONSTART);
 	crange.cpMax = wOutput.Call(SCI_GETSELECTIONEND);
-	long selStart = static_cast<long>(crange.cpMin);
+	const long selStart = static_cast<long>(crange.cpMin);
 	const int curLine = wOutput.Call(SCI_LINEFROMPOSITION, selStart);
 	const int maxLine = wOutput.Call(SCI_GETLINECOUNT);
 	int lookLine = curLine + dir;
@@ -1953,7 +1948,7 @@ void SciTEBase::GoMessage(int dir) {
 		lookLine = 0;
 	TextReader acc(wOutput);
 	while ((dir == 0) || (lookLine != curLine)) {
-		int startPosLine = wOutput.Call(SCI_POSITIONFROMLINE, lookLine, 0);
+		const int startPosLine = wOutput.Call(SCI_POSITIONFROMLINE, lookLine, 0);
 		const int lineLength = wOutput.Call(SCI_LINELENGTH, lookLine, 0);
 		int style = acc.StyleAt(startPosLine);
 		if (style != SCE_ERR_DEFAULT &&
@@ -1995,8 +1990,8 @@ void SciTEBase::GoMessage(int dir) {
 					} else {
 						// Look through buffers for name match
 						for (int i = buffers.lengthVisible - 1; i >= 0; i--) {
-							if (sourcePath.Name().SameNameAs(buffers.buffers[i].Name())) {
-								messagePath = buffers.buffers[i];
+							if (sourcePath.Name().SameNameAs(buffers.buffers[i].file.Name())) {
+								messagePath = buffers.buffers[i].file;
 								bExists = true;
 							}
 						}
@@ -2054,7 +2049,7 @@ void SciTEBase::GoMessage(int dir) {
 				        "error.marker.back", ColourRGB(0xff, 0xff, 0)));
 				wEditor.Call(SCI_MARKERADD, sourceLine, 0);
 				int startSourceLine = wEditor.Call(SCI_POSITIONFROMLINE, sourceLine, 0);
-				int endSourceline = wEditor.Call(SCI_POSITIONFROMLINE, sourceLine + 1, 0);
+				const int endSourceline = wEditor.Call(SCI_POSITIONFROMLINE, sourceLine + 1, 0);
 				if (column >= 0) {
 					// Get the position in line according to current tab setting
 					startSourceLine = wEditor.Call(SCI_FINDCOLUMN, sourceLine, column);
