@@ -5,28 +5,30 @@
 // Copyright 1998-2010 by Neil Hodgson <neilh@scintilla.org>
 // The License.txt file describes the conditions under which this software may be distributed.
 
-#include <stdlib.h>
+#include <cstddef>
+#include <cstdlib>
 // Older versions of GNU stdint.h require this definition to be able to see INT32_MAX
 #define __STDC_LIMIT_MACROS
-#include <stdint.h>
-#include <string.h>
-#include <ctype.h>
-#include <stdio.h>
-#include <stddef.h>
-#include <fcntl.h>
-#include <time.h>
+#include <cstdint>
+#include <cstring>
+#include <cctype>
+#include <cstdio>
+#include <ctime>
 
 #include <string>
 #include <vector>
-#include <set>
 #include <map>
+#include <set>
 #include <algorithm>
 #include <memory>
 
+#include <fcntl.h>
+
+#include "ILoader.h"
 #include "Scintilla.h"
-#include "ILexer.h"
 
 #include "GUI.h"
+#include "ScintillaWindow.h"
 
 #include "StringList.h"
 #include "StringHelpers.h"
@@ -86,7 +88,7 @@ void SciTEBase::SetFileName(const FilePath &openName, bool fixCase) {
 
 	SetWindowName();
 	if (buffers.buffers.size() > 0)
-		buffers.buffers[buffers.Current()].Set(filePath);
+		CurrentBuffer()->file.Set(filePath);
 }
 
 // See if path exists.
@@ -150,7 +152,7 @@ void SciTEBase::DiscoverEOLSetting() {
 
 // Look inside the first line for a #! clue regarding the language
 std::string SciTEBase::DiscoverLanguage() {
-	int length = Minimum(LengthDocument(), 64 * 1024);
+	const int length = Minimum(LengthDocument(), 64 * 1024);
 	std::string buf = GetRangeString(wEditor, 0, length);
 	std::string languageOverride = "";
 	std::string l1 = ExtractLine(buf.c_str(), length);
@@ -276,7 +278,19 @@ void SciTEBase::OpenCurrentFile(long long fileSize, bool suppressMessage, bool a
 		assert(CurrentBufferConst()->pFileWorker == NULL);
 		ILoader *pdocLoad;
 		try {
-			pdocLoad = reinterpret_cast<ILoader *>(wEditor.CallReturnPointer(SCI_CREATELOADER, static_cast<uptr_t>(fileSize) + 1000));
+			sptr_t docOptions = SC_DOCUMENTOPTION_DEFAULT;
+
+			const long long sizeLarge = props.GetLongLong("file.size.large");
+			if (sizeLarge && (fileSize > sizeLarge))
+				docOptions |= SC_DOCUMENTOPTION_TEXT_LARGE;
+
+			const long long sizeNoStyles = props.GetLongLong("file.size.no.styles");
+			if (sizeNoStyles && (fileSize > sizeNoStyles))
+				docOptions |= SC_DOCUMENTOPTION_STYLES_NONE;
+
+			pdocLoad = reinterpret_cast<ILoader *>(
+				wEditor.CallReturnPointer(SCI_CREATELOADER, static_cast<uptr_t>(fileSize) + 1000,
+					docOptions));
 		} catch (...) {
 			wEditor.Call(SCI_SETSTATUS, 0);
 			return;
@@ -293,14 +307,14 @@ void SciTEBase::OpenCurrentFile(long long fileSize, bool suppressMessage, bool a
 		const UniMode umCodingCookie = CodingCookieValue(&data[0], lenFile);
 		while (lenFile > 0) {
 			lenFile = convert.convert(&data[0], lenFile);
-			char *dataBlock = convert.getNewBuf();
+			const char *dataBlock = convert.getNewBuf();
 			wEditor.CallString(SCI_ADDTEXT, lenFile, dataBlock);
 			lenFile = fread(&data[0], 1, data.size(), fp);
 			if (lenFile == 0) {
 				// Handle case where convert is holding a lead surrogate but no more data
-				size_t lenFileTrail = convert.convert(NULL, lenFile);
+				const size_t lenFileTrail = convert.convert(NULL, lenFile);
 				if (lenFileTrail) {
-					char *dataTrail = convert.getNewBuf();
+					const char *dataTrail = convert.getNewBuf();
 					wEditor.CallString(SCI_ADDTEXT, lenFileTrail, dataTrail);
 				}
 			}
@@ -321,7 +335,7 @@ void SciTEBase::OpenCurrentFile(long long fileSize, bool suppressMessage, bool a
 
 void SciTEBase::TextRead(FileWorker *pFileWorker) {
 	FileLoader *pFileLoader = static_cast<FileLoader *>(pFileWorker);
-	int iBuffer = buffers.GetDocumentByWorker(pFileLoader);
+	const int iBuffer = buffers.GetDocumentByWorker(pFileLoader);
 	// May not be found if load cancelled
 	if (iBuffer >= 0) {
 		buffers.buffers[iBuffer].unicodeMode = pFileLoader->unicodeMode;
@@ -333,15 +347,15 @@ void SciTEBase::TextRead(FileWorker *pFileWorker) {
 			buffers.buffers[iBuffer].lifeState = Buffer::empty;
 		}
 		// Switch documents
-		sptr_t pdocLoading = reinterpret_cast<sptr_t>(pFileLoader->pLoader->ConvertToDocument());
+		const sptr_t pdocLoading = reinterpret_cast<sptr_t>(pFileLoader->pLoader->ConvertToDocument());
 		pFileLoader->pLoader = 0;
 		SwitchDocumentAt(iBuffer, pdocLoading);
 		if (iBuffer == buffers.Current()) {
 			CompleteOpen(ocCompleteCurrent);
 			if (extender)
-				extender->OnOpen(buffers.buffers[iBuffer].AsUTF8().c_str());
+				extender->OnOpen(buffers.buffers[iBuffer].file.AsUTF8().c_str());
 			RestoreState(buffers.buffers[iBuffer], true);
-			DisplayAround(buffers.buffers[iBuffer]);
+			DisplayAround(buffers.buffers[iBuffer].file);
 			wEditor.Call(SCI_SCROLLCARET);
 		}
 	}
@@ -408,8 +422,8 @@ void SciTEBase::CompleteOpen(OpenCompletion oc) {
 }
 
 void SciTEBase::TextWritten(FileWorker *pFileWorker) {
-	FileStorer *pFileStorer = static_cast<FileStorer *>(pFileWorker);
-	int iBuffer = buffers.GetDocumentByWorker(pFileStorer);
+	const FileStorer *pFileStorer = static_cast<const FileStorer *>(pFileWorker);
+	const int iBuffer = buffers.GetDocumentByWorker(pFileStorer);
 
 	FilePath pathSaved = pFileStorer->path;
 	const int errSaved = pFileStorer->err;
@@ -433,11 +447,11 @@ void SciTEBase::TextWritten(FileWorker *pFileWorker) {
 			}
 			if (iBuffer == buffers.Current()) {
 				wEditor.Call(SCI_SETREADONLY, CurrentBuffer()->isReadOnly);
-				if (pathSaved.SameNameAs(CurrentBuffer()->AsInternal())) {
+				if (pathSaved.SameNameAs(CurrentBuffer()->file)) {
 					wEditor.Call(SCI_SETSAVEPOINT);
 				}
 				if (extender)
-					extender->OnSave(buffers.buffers[iBuffer].AsUTF8().c_str());
+					extender->OnSave(buffers.buffers[iBuffer].file.AsUTF8().c_str());
 			} else {
 				buffers.buffers[iBuffer].isDirty = false;
 				buffers.buffers[iBuffer].failedSave = false;
@@ -505,7 +519,7 @@ bool SciTEBase::Open(const FilePath &file, OpenFlags of) {
 		return false;
 	}
 
-	int index = buffers.GetDocumentByName(absPath);
+	const int index = buffers.GetDocumentByName(absPath);
 	if (index >= 0) {
 		buffers.SetVisible(index, true);
 		SetDocumentAt(index);
@@ -522,7 +536,7 @@ bool SciTEBase::Open(const FilePath &file, OpenFlags of) {
 	}
 
 	const long long fileSize = absPath.IsUntitled() ? 0 : absPath.GetFileLength();
-	if (fileSize > INT32_MAX) {
+	if (fileSize > INTPTR_MAX) {
 		const GUI::gui_string sSize = GUI::StringFromLongLong(fileSize);
 		const GUI::gui_string msg = LocaliseMessage("File '^0' is ^1 bytes long, "
 			"larger than 2GB which is the largest SciTE can open.",
@@ -548,7 +562,7 @@ bool SciTEBase::Open(const FilePath &file, OpenFlags of) {
 	}
 
 	if (buffers.size() == buffers.length) {
-		AddFileToStack(filePath, GetSelectedRange(), GetCurrentScrollPosition());
+		AddFileToStack(RecentFile(filePath, GetSelectedRange(), GetCurrentScrollPosition()));
 		ClearDocument();
 		CurrentBuffer()->lifeState = Buffer::open;
 		if (extender)
@@ -665,11 +679,11 @@ bool SciTEBase::OpenSelected() {
 		// A bit of duplicate work with DecodeMessage, but we don't know
 		// here the format of the line, so we do guess work.
 		// Can't do much for space separated line numbers anyway...
-		size_t endPath = selName.find("(");
+		size_t endPath = selName.find('(');
 		if (endPath != std::string::npos) {	// Visual Studio error message: F:\scite\src\SciTEBase.h(312):	bool Exists(
 			lineNumber = atol(selName.c_str() + endPath + 1);
 		} else {
-			endPath = selName.find(":", 2);	// Skip Windows' drive separator
+			endPath = selName.find(':', 2);	// Skip Windows' drive separator
 			if (endPath != std::string::npos) {	// grep -n line, perhaps gcc too: F:\scite\src\SciTEBase.h:312:	bool Exists(
 				lineNumber = atol(selName.c_str() + endPath + 1);
 			}
@@ -697,7 +711,7 @@ bool SciTEBase::OpenSelected() {
 			            "openpath.", fileNameForExtension.c_str()));
 			while (openPath.length()) {
 				GUI::gui_string tryPath(openPath);
-				size_t sepIndex = tryPath.find(listSepString);
+				const size_t sepIndex = tryPath.find(listSepString);
 				if ((sepIndex != GUI::gui_string::npos) && (sepIndex != 0)) {
 					tryPath.erase(sepIndex);
 					openPath.erase(0, sepIndex + 1);
@@ -714,7 +728,7 @@ bool SciTEBase::OpenSelected() {
 	FilePath pathReturned;
 	if (Exists(path.AsInternal(), selFN.c_str(), &pathReturned)) {
 		// Open synchronously if want to seek line number or search tag
-		OpenFlags of = ((lineNumber > 0) || (cTag.length() != 0)) ? ofSynchronous : ofNone;
+		const OpenFlags of = ((lineNumber > 0) || (cTag.length() != 0)) ? ofSynchronous : ofNone;
 		if (Open(pathReturned, of)) {
 			if (lineNumber > 0) {
 				wEditor.Call(SCI_GOTOLINE, lineNumber - 1);
@@ -747,10 +761,10 @@ void SciTEBase::Revert() {
 void SciTEBase::CheckReload() {
 	if (props.GetInt("load.on.activate")) {
 		// Make a copy of fullPath as otherwise it gets aliased in Open
-		time_t newModTime = filePath.ModifiedTime();
+		const time_t newModTime = filePath.ModifiedTime();
 		if ((newModTime != 0) && (newModTime != CurrentBuffer()->fileModTime)) {
 			RecentFile rf = GetFilePosition();
-			OpenFlags of = props.GetInt("reload.preserves.undo") ? ofPreserveUndo : ofNone;
+			const OpenFlags of = props.GetInt("reload.preserves.undo") ? ofPreserveUndo : ofNone;
 			if (CurrentBuffer()->isDirty || props.GetInt("are.you.sure.on.reload") != 0) {
 				if ((0 == dialogsOnScreen) && (newModTime != CurrentBuffer()->fileModLastAsk)) {
 					GUI::gui_string msg;
@@ -861,7 +875,7 @@ SciTEBase::SaveResult SciTEBase::SaveIfUnsureAll() {
 	if (props.GetInt("save.recent")) {
 		for (int i = 0; i < buffers.lengthVisible; ++i) {
 			Buffer buff = buffers.buffers[i];
-			AddFileToStack(buff, buff.selection, buff.scrollPosition);
+			AddFileToStack(buff.file);
 		}
 	}
 	if (props.GetInt("save.session") || props.GetInt("save.position") || props.GetInt("save.recent")) {
@@ -904,12 +918,108 @@ SciTEBase::SaveResult SciTEBase::SaveIfUnsureForBuilt() {
 	}
 	return saveCompleted;
 }
+/**
+	Selection saver and restorer.
+
+	If virtual space is disabled, the class does nothing.
+
+	If virtual space is enabled, constructor saves all selections using (line, column) coordinates,
+	destructor restores all the saved selections.
+**/
+class SelectionKeeper {
+public:
+	explicit SelectionKeeper(GUI::ScintillaWindow &editor) : wEditor(editor) {
+		const int mask = SCVS_RECTANGULARSELECTION | SCVS_USERACCESSIBLE;
+		if (wEditor.Call(SCI_GETVIRTUALSPACEOPTIONS, 0, 0) & mask) {
+			const int n = wEditor.Call(SCI_GETSELECTIONS, 0, 0);
+			for (int i = 0; i < n; ++i) {
+				selections.push_back(LocFromPos(GetSelection(i)));
+			}
+		}
+	}
+
+	~SelectionKeeper() {
+		int i = 0;
+		for (auto const &sel : selections) {
+			SetSelection(i, PosFromLoc(sel));
+			++i;
+		}
+	}
+
+private:
+	struct Position {
+		Position(int pos_, int virt_ = 0) : pos(pos_), virt(virt_) {};
+		int pos;
+		int virt;
+	};
+
+	struct Location {
+		Location(int line_, int col_) : line(line_), col(col_) {};
+		int line;
+		int col;
+	};
+
+	Position GetAnchor(int i) {
+		const int pos  = wEditor.Call(SCI_GETSELECTIONNANCHOR, i, 0);
+		const int virt = wEditor.Call(SCI_GETSELECTIONNANCHORVIRTUALSPACE, i, 0);
+		return Position(pos, virt);
+	}
+
+	Position GetCaret(int i) {
+		const int pos  = wEditor.Call(SCI_GETSELECTIONNCARET, i, 0);
+		const int virt = wEditor.Call(SCI_GETSELECTIONNCARETVIRTUALSPACE, i, 0);
+		return Position(pos, virt);
+	}
+
+	std::pair<Position, Position> GetSelection(int i) {
+		return {GetAnchor(i), GetCaret(i)};
+	};
+
+	Location LocFromPos(Position const &pos) {
+		const int line = wEditor.Call(SCI_LINEFROMPOSITION, pos.pos, 0);
+		const int col  = wEditor.Call(SCI_GETCOLUMN, pos.pos, 0) + pos.virt;
+		return Location(line, col);
+	}
+
+	std::pair<Location, Location> LocFromPos(std::pair<Position, Position> const &pos) {
+		return {LocFromPos(pos.first), LocFromPos(pos.second)};
+	}
+
+	Position PosFromLoc(Location const &loc) {
+		const int pos = wEditor.Call(SCI_FINDCOLUMN, loc.line, loc.col);
+		const int col = wEditor.Call(SCI_GETCOLUMN, pos, 0);
+		return Position(pos, loc.col - col);
+	}
+
+	std::pair<Position, Position> PosFromLoc(std::pair<Location, Location> const &loc) {
+		return {PosFromLoc(loc.first), PosFromLoc(loc.second)};
+	}
+
+	void SetAnchor(int i, Position const &pos) {
+		wEditor.Call(SCI_SETSELECTIONNANCHOR, i, pos.pos);
+		wEditor.Call(SCI_SETSELECTIONNANCHORVIRTUALSPACE, i, pos.virt);
+	};
+
+	void SetCaret(int i, Position const &pos) {
+		wEditor.Call(SCI_SETSELECTIONNCARET, i, pos.pos);
+		wEditor.Call(SCI_SETSELECTIONNCARETVIRTUALSPACE, i, pos.virt);
+	}
+
+	void SetSelection(int i, std::pair<Position, Position> const &pos) {
+		SetAnchor(i, pos.first);
+		SetCaret(i, pos.second);
+	}
+
+	GUI::ScintillaWindow &wEditor;
+	std::vector<std::pair<Location, Location>> selections;
+};
 
 void SciTEBase::StripTrailingSpaces() {
 	const int maxLines = wEditor.Call(SCI_GETLINECOUNT);
+	SelectionKeeper keeper(wEditor);
 	for (int line = 0; line < maxLines; line++) {
 		const int lineStart = wEditor.Call(SCI_POSITIONFROMLINE, line);
-		int lineEnd = wEditor.Call(SCI_GETLINEENDPOSITION, line);
+		const int lineEnd = wEditor.Call(SCI_GETLINEENDPOSITION, line);
 		int i = lineEnd - 1;
 		char ch = static_cast<char>(wEditor.Call(SCI_GETCHARAT, i));
 		while ((i >= lineStart) && ((ch == ' ') || (ch == '\t'))) {
@@ -925,9 +1035,9 @@ void SciTEBase::StripTrailingSpaces() {
 }
 
 void SciTEBase::EnsureFinalNewLine() {
-	int maxLines = wEditor.Call(SCI_GETLINECOUNT);
+	const int maxLines = wEditor.Call(SCI_GETLINECOUNT);
 	bool appendNewLine = maxLines == 1;
-	int endDocument = wEditor.Call(SCI_POSITIONFROMLINE, maxLines);
+	const int endDocument = wEditor.Call(SCI_POSITIONFROMLINE, maxLines);
 	if (maxLines > 1) {
 		appendNewLine = endDocument > wEditor.Call(SCI_POSITIONFROMLINE, maxLines - 1);
 	}
@@ -950,15 +1060,11 @@ bool SciTEBase::PrepareBufferForSave(const FilePath &saveName) {
 	bool retVal = false;
 	// Perform clean ups on text before saving
 	wEditor.Call(SCI_BEGINUNDOACTION);
-	std::string useStripTrailingSpaces = props.GetNewExpandString("strip.trailing.spaces.", ExtensionFileName().c_str());
-	if (useStripTrailingSpaces.length() > 0) {
-		if (atoi(useStripTrailingSpaces.c_str()))
-			StripTrailingSpaces();
-	} else if (props.GetInt("strip.trailing.spaces"))
+	if (stripTrailingSpaces)
 		StripTrailingSpaces();
-	if (props.GetInt("ensure.final.line.end"))
+	if (ensureFinalLineEnd)
 		EnsureFinalNewLine();
-	if (props.GetInt("ensure.consistent.line.ends"))
+	if (ensureConsistentLineEnds)
 		wEditor.Call(SCI_CONVERTEOLS, wEditor.Call(SCI_GETEOLMODE));
 
 	if (extender)
@@ -979,7 +1085,7 @@ bool SciTEBase::SaveBuffer(const FilePath &saveName, SaveFlags sf) {
 
 		FILE *fp = saveName.Open(fileWrite);
 		if (fp) {
-			size_t lengthDoc = LengthDocument();
+			const size_t lengthDoc = LengthDocument();
 			if (!(sf & sfSynchronous)) {
 				wEditor.Call(SCI_SETREADONLY, 1);
 				const char *documentBytes = reinterpret_cast<const char *>(wEditor.CallReturnPointer(SCI_GETCHARACTERPOINTER));
@@ -1102,7 +1208,7 @@ bool SciTEBase::Save(SaveFlags sf) {
 		return true;
 	} else {
 		if (props.GetString("save.path.suggestion").length()) {
-			time_t t = time(NULL);
+			const time_t t = time(NULL);
 			char timeBuff[15];
 			strftime(timeBuff, sizeof(timeBuff), "%Y%m%d%H%M%S",  localtime(&t));
 			PropSetFile propsSuggestion;
@@ -1115,7 +1221,7 @@ bool SciTEBase::Save(SaveFlags sf) {
 				filePath = FilePath(GUI::StringFromUTF8(savePathSuggestion)).NormalizePath();
 			}
 		}
-		bool ret = SaveAsDialog();
+		const bool ret = SaveAsDialog();
 		if (!ret)
 			filePath.Set(GUI_TEXT(""));
 		return ret;
@@ -1304,8 +1410,7 @@ class FileReader {
 	std::string lineToShow;
 	bool caseSensitive;
 public:
-	FileReader(const FilePath &fPath, bool caseSensitive_) {
-		bf.reset(new BufferedFile(fPath));
+	FileReader(const FilePath &fPath, bool caseSensitive_) : bf(std::make_unique<BufferedFile>(fPath)) {
 		lineNum = 0;
 		lastWasCR = false;
 		caseSensitive = caseSensitive_;
@@ -1320,7 +1425,7 @@ public:
 		}
 		lineToShow.clear();
 		while (!bf->Exhausted()) {
-			int ch = bf->NextByte();
+			const int ch = bf->NextByte();
 			if (lastWasCR && ch == '\n' && lineToShow.empty()) {
 				lastWasCR = false;
 			} else if (ch == '\r' || ch == '\n') {
@@ -1362,6 +1467,7 @@ bool SciTEBase::GrepIntoDirectory(const FilePath &directory) {
 }
 
 void SciTEBase::GrepRecursive(GrepFlags gf, const FilePath &baseDir, const char *searchString, const GUI::gui_char *fileTypes) {
+	const int checkAfterLines = 10'000;
 	FilePathSet directories;
 	FilePathSet files;
 	baseDir.List(directories, files);
@@ -1376,6 +1482,8 @@ void SciTEBase::GrepRecursive(GrepFlags gf, const FilePath &baseDir, const char
 			FileReader fr(fPath, gf & grepMatchCase);
 			if ((gf & grepBinary) || !fr.BufferContainsNull()) {
 				while (const char *line = fr.Next()) {
+					if (((fr.LineNumber() % checkAfterLines) == 0) && jobQueue.Cancelled())
+						return;
 					const char *match = strstr(line, searchString);
 					if (match) {
 						if (gf & grepWholeWord) {
