@@ -5,27 +5,29 @@
 // Copyright 1998-2011 by Neil Hodgson <neilh@scintilla.org>
 // The License.txt file describes the conditions under which this software may be distributed.
 
-#include <stdlib.h>
-#include <string.h>
-#include <ctype.h>
-#include <stdio.h>
-#include <stddef.h>
-#include <fcntl.h>
-#include <time.h>
-#include <locale.h>
+#include <cstddef>
+#include <cstdlib>
+#include <cstring>
+#include <cctype>
+#include <cstdio>
+#include <ctime>
+#include <clocale>
 
 #include <string>
 #include <vector>
-#include <set>
 #include <map>
+#include <set>
 #include <algorithm>
 #include <memory>
 
+#include <fcntl.h>
+
+#include "ILexer.h"
 #include "Scintilla.h"
 #include "SciLexer.h"
-#include "ILexer.h"
 
 #include "GUI.h"
+#include "ScintillaWindow.h"
 
 #if defined(__unix__)
 
@@ -45,13 +47,14 @@ const GUI::gui_char menuAccessIndicator[] = GUI_TEXT("&");
 #include "StyleWriter.h"
 #include "Extender.h"
 #include "SciTE.h"
-#include "IFaceTable.h"
 #include "Mutex.h"
 #include "JobQueue.h"
 #include "Cookie.h"
 #include "Worker.h"
 #include "MatchMarker.h"
+#include "EditorConfig.h"
 #include "SciTEBase.h"
+#include "IFaceTable.h"
 
 void SciTEBase::SetImportMenu() {
 	for (int i = 0; i < importMax; i++) {
@@ -59,7 +62,7 @@ void SciTEBase::SetImportMenu() {
 	}
 	if (!importFiles.empty()) {
 		for (int stackPos = 0; stackPos < static_cast<int>(importFiles.size()) && stackPos < importMax; stackPos++) {
-			int itemID = importCmdID + stackPos;
+			const int itemID = importCmdID + stackPos;
 			if (importFiles[stackPos].IsSet()) {
 				GUI::gui_string entry = localiser.Text("Open");
 				entry += GUI_TEXT(" ");
@@ -83,7 +86,7 @@ void SciTEBase::SetLanguageMenu() {
 		DestroyMenuItem(menuLanguage, languageCmdID + i);
 	}
 	for (unsigned int item = 0; item < languageMenu.size(); item++) {
-		int itemID = languageCmdID + item;
+		const int itemID = languageCmdID + item;
 		GUI::gui_string entry = localiser.Text(languageMenu[item].menuItem.c_str());
 		if (languageMenu[item].menuKey.length()) {
 #if defined(GTK)
@@ -202,6 +205,12 @@ void SciTEBase::ReadLocalPropFile() {
 
 	props.Set("Chrome", "#C0C0C0");
 	props.Set("ChromeHighlight", "#FFFFFF");
+
+	FilePath fileDirectory = filePath.Directory();
+	editorConfig->Clear();
+	if (props.GetInt("editor.config.enable", 0)) {
+		editorConfig->ReadFromDirectory(fileDirectory);
+	}
 }
 
 Colour ColourOfProperty(const PropSetFile &props, const char *key, Colour colourDefault) {
@@ -271,8 +280,8 @@ StyleDefinition SciTEBase::StyleDefinitionFor(int style) {
 		}
 	}
 
-	StyleDefinition sd(ssDefault.c_str());
-	sd.ParseStyleDefinition(ss.c_str());
+	StyleDefinition sd(ssDefault);
+	sd.ParseStyleDefinition(ss);
 	return sd;
 }
 
@@ -310,17 +319,14 @@ void SciTEBase::SetStyleBlock(GUI::ScintillaWindow &win, const char *lang, int s
 			sprintf(key, "style.%s.%0d", lang, style-start);
 			std::string sval = props.GetExpandedString(key);
 			if (sval.length()) {
-				SetOneStyle(win, style, sval.c_str());
+				SetOneStyle(win, style, StyleDefinition(sval));
 			}
 		}
 	}
 }
 
 void SciTEBase::SetStyleFor(GUI::ScintillaWindow &win, const char *lang) {
-	int maxStyle = (1 << win.Call(SCI_GETSTYLEBITS)) - 1;
-	if (maxStyle < STYLE_LASTPREDEFINED)
-		maxStyle = STYLE_LASTPREDEFINED;
-	SetStyleBlock(win, lang, 0, maxStyle);
+	SetStyleBlock(win, lang, 0, STYLE_MAX);
 }
 
 void SciTEBase::SetOneIndicator(GUI::ScintillaWindow &win, int indicator, const IndicatorDefinition &ind) {
@@ -381,7 +387,7 @@ void SciTEBase::ReadAPI(const std::string &fileNameForExtension) {
 
 		// Load files into data
 		for (const std::string &vApiFileName : vApiFileNames) {
-			std::vector<char> contents = FilePath(GUI::StringFromUTF8(vApiFileName)).Read();
+			std::string contents = FilePath(GUI::StringFromUTF8(vApiFileName)).Read();
 			data.insert(data.end(), contents.begin(), contents.end());
 		}
 
@@ -489,6 +495,7 @@ static const char *propertiesToForward[] = {
 	"lexer.css.less.language",
 	"lexer.css.scss.language",
 	"lexer.d.fold.at.else",
+	"lexer.edifact.highlight.un.all",
 	"lexer.errorlist.escape.sequences",
 	"lexer.errorlist.value.separate",
 	"lexer.flagship.styling.within.preprocessor",
@@ -648,6 +655,9 @@ void SciTEBase::ReadProperties() {
 	if (modulePath.length())
 	    wEditor.CallString(SCI_LOADLEXERLIBRARY, 0, modulePath.c_str());
 	language = props.GetNewExpandString("lexer.", fileNameForExtension.c_str());
+	if (wEditor.Call(SCI_GETDOCUMENTOPTIONS) & SC_DOCUMENTOPTION_STYLES_NONE) {
+		language = "";
+	}
 	if (language.length()) {
 		if (StartsWith(language, "script_")) {
 			wEditor.Call(SCI_SETLEXER, SCLEX_CONTAINER);
@@ -657,7 +667,7 @@ void SciTEBase::ReadProperties() {
 				wEditor.CallString(SCI_LOADLEXERLIBRARY, 0, modulePath.c_str());
 				wEditor.CallString(SCI_SETLEXERLANGUAGE, 0, "lpeg");
 				lexLPeg = wEditor.Call(SCI_GETLEXER);
-				const char *lexer = language.c_str() + language.find("_") + 1;
+				const char *lexer = language.c_str() + language.find('_') + 1;
 				wEditor.CallReturnPointer(SCI_PRIVATELEXERCALL, SCI_SETLEXERLANGUAGE,
 					SptrFromString(lexer));
 			}
@@ -672,11 +682,6 @@ void SciTEBase::ReadProperties() {
 
 	lexLanguage = wEditor.Call(SCI_GETLEXER);
 
-	if (StartsWith(language, "script_") || StartsWith(language, "lpeg_"))
-		wEditor.Call(SCI_SETSTYLEBITS, 8);
-	else
-		wEditor.Call(SCI_SETSTYLEBITS, wEditor.Call(SCI_GETSTYLEBITSNEEDED));
-
 	wOutput.Call(SCI_SETLEXER, SCLEX_ERRORLIST);
 
 	const std::string kw0 = props.GetNewExpandString("keywords.", fileNameForExtension.c_str());
@@ -691,7 +696,7 @@ void SciTEBase::ReadProperties() {
 	}
 
 	subStyleBases.clear();
-	int lenSSB = wEditor.CallString(SCI_GETSUBSTYLEBASES, 0, NULL);
+	const int lenSSB = wEditor.CallString(SCI_GETSUBSTYLEBASES, 0, NULL);
 	if (lenSSB) {
 		wEditor.Call(SCI_FREESUBSTYLES);
 
@@ -754,17 +759,21 @@ void SciTEBase::ReadProperties() {
 
 	props.Set("AbbrevPath", pathAbbreviations.AsUTF8().c_str());
 
-	int tech = props.GetInt("technology");
+	const int tech = props.GetInt("technology");
 	wEditor.Call(SCI_SETTECHNOLOGY, tech);
 	wOutput.Call(SCI_SETTECHNOLOGY, tech);
 
+	const int bidirectional = props.GetInt("bidirectional");
+	wEditor.Call(SCI_SETBIDIRECTIONAL, bidirectional);
+	wOutput.Call(SCI_SETBIDIRECTIONAL, bidirectional);
+
 	codePage = props.GetInt("code.page");
 	if (CurrentBuffer()->unicodeMode != uni8Bit) {
 		// Override properties file to ensure Unicode displayed.
 		codePage = SC_CP_UTF8;
 	}
 	wEditor.Call(SCI_SETCODEPAGE, codePage);
-	int outputCodePage = props.GetInt("output.code.page", codePage);
+	const int outputCodePage = props.GetInt("output.code.page", codePage);
 	wOutput.Call(SCI_SETCODEPAGE, outputCodePage);
 
 	characterSet = props.GetInt("character.set", SC_CHARSET_DEFAULT);
@@ -849,9 +858,9 @@ void SciTEBase::ReadProperties() {
 	caretJumps = props.GetInt("caret.policy.yjumps") ? CARET_JUMPS : 0;
 	wEditor.Call(SCI_SETYCARETPOLICY, caretStrict | caretSlop | caretEven | caretJumps, caretZone);
 
-	int visibleStrict = props.GetInt("visible.policy.strict") ? VISIBLE_STRICT : 0;
-	int visibleSlop = props.GetInt("visible.policy.slop", 1) ? VISIBLE_SLOP : 0;
-	int visibleLines = props.GetInt("visible.policy.lines");
+	const int visibleStrict = props.GetInt("visible.policy.strict") ? VISIBLE_STRICT : 0;
+	const int visibleSlop = props.GetInt("visible.policy.slop", 1) ? VISIBLE_SLOP : 0;
+	const int visibleLines = props.GetInt("visible.policy.lines");
 	wEditor.Call(SCI_SETVISIBLEPOLICY, visibleStrict | visibleSlop, visibleLines);
 
 	wEditor.Call(SCI_SETEDGECOLUMN, props.GetInt("edge.column", 0));
@@ -874,7 +883,7 @@ void SciTEBase::ReadProperties() {
 		else	// Have to show selection somehow
 			CallChildren(SCI_SETSELBACK, 1, ColourRGB(0xC0, 0xC0, 0xC0));
 	}
-	int selectionAlpha = props.GetInt("selection.alpha", SC_ALPHA_NOALPHA);
+	const int selectionAlpha = props.GetInt("selection.alpha", SC_ALPHA_NOALPHA);
 	CallChildren(SCI_SETSELALPHA, selectionAlpha);
 
 	std::string selAdditionalFore = props.GetString("selection.additional.fore");
@@ -968,7 +977,7 @@ void SciTEBase::ReadProperties() {
 	wEditor.Call(SCI_AUTOCSETIGNORECASE, autoCompleteIgnoreCase ? 1 : 0);
 	wOutput.Call(SCI_AUTOCSETIGNORECASE, 1);
 
-	int autoCChooseSingle = props.GetInt("autocomplete.choose.single");
+	const int autoCChooseSingle = props.GetInt("autocomplete.choose.single");
 	wEditor.Call(SCI_AUTOCSETCHOOSESINGLE, autoCChooseSingle);
 
 	wEditor.Call(SCI_AUTOCSETCANCELATSTART, 0);
@@ -986,9 +995,9 @@ void SciTEBase::ReadProperties() {
 	jobQueue.clearBeforeExecute = props.GetInt("clear.before.execute");
 	jobQueue.timeCommands = props.GetInt("time.commands");
 
-	int blankMarginLeft = props.GetInt("blank.margin.left", 1);
-	int blankMarginLeftOutput = props.GetInt("output.blank.margin.left", blankMarginLeft);
-	int blankMarginRight = props.GetInt("blank.margin.right", 1);
+	const int blankMarginLeft = props.GetInt("blank.margin.left", 1);
+	const int blankMarginLeftOutput = props.GetInt("output.blank.margin.left", blankMarginLeft);
+	const int blankMarginRight = props.GetInt("blank.margin.right", 1);
 	wEditor.Call(SCI_SETMARGINLEFT, 0, blankMarginLeft);
 	wEditor.Call(SCI_SETMARGINRIGHT, 0, blankMarginRight);
 	wOutput.Call(SCI_SETMARGINLEFT, 0, blankMarginLeftOutput);
@@ -1044,6 +1053,15 @@ void SciTEBase::ReadProperties() {
 
 	wEditor.Call(SCI_CALLTIPUSESTYLE, 32);
 
+	std::string useStripTrailingSpaces = props.GetNewExpandString("strip.trailing.spaces.", ExtensionFileName().c_str());
+	if (useStripTrailingSpaces.length() > 0) {
+		stripTrailingSpaces = atoi(useStripTrailingSpaces.c_str()) != 0;
+	} else {
+		stripTrailingSpaces = props.GetInt("strip.trailing.spaces") != 0;
+	}
+	ensureFinalLineEnd = props.GetInt("ensure.final.line.end") != 0;
+	ensureConsistentLineEnds = props.GetInt("ensure.consistent.line.ends") != 0;
+
 	indentOpening = props.GetInt("indent.opening");
 	indentClosing = props.GetInt("indent.closing");
 	indentMaintain = atoi(props.GetNewExpandString("indent.maintain.", fileNameForExtension.c_str()).c_str());
@@ -1177,7 +1195,7 @@ void SciTEBase::ReadProperties() {
 			break;
 		}
 	}
-	Colour colourFoldFore = ColourFromString(foldFore);
+	const Colour colourFoldFore = ColourFromString(foldFore);
 
 	std::string foldBack = props.GetExpandedString("fold.back");
 	// Set default colour for fill
@@ -1193,12 +1211,12 @@ void SciTEBase::ReadProperties() {
 			break;
 		}
 	}
-	Colour colourFoldBack = ColourFromString(foldBack);
+	const Colour colourFoldBack = ColourFromString(foldBack);
 
 	// Enable/disable highlight for current folding block (smallest one that contains the caret)
-	int isHighlightEnabled = props.GetInt("fold.highlight", 0);
+	const int isHighlightEnabled = props.GetInt("fold.highlight", 0);
 	// Define the colour of highlight
-	Colour colourFoldBlockHighlight = ColourOfProperty(props, "fold.highlight.colour", ColourRGB(0xFF, 0, 0));
+	const Colour colourFoldBlockHighlight = ColourOfProperty(props, "fold.highlight.colour", ColourRGB(0xFF, 0, 0));
 
 	switch (foldSymbols) {
 	case 0:
@@ -1288,7 +1306,7 @@ void SciTEBase::ReadProperties() {
 		wEditor.CallString(SCI_MARKERDEFINEPIXMAP, markerBookmark,
 			bookMarkXPM.c_str());
 	} else if (props.GetString("bookmark.fore").length()) {
-		wEditor.Call(SCI_MARKERDEFINE, markerBookmark, SC_MARK_BOOKMARK);
+		wEditor.Call(SCI_MARKERDEFINE, markerBookmark, props.GetInt("bookmark.symbol", SC_MARK_BOOKMARK));
 	} else {
 		// No bookmark.fore setting so display default pixmap.
 		wEditor.CallPointer(SCI_MARKERDEFINEPIXMAP, markerBookmark, bookmarkBluegem);
@@ -1334,6 +1352,45 @@ void SciTEBase::ReadProperties() {
 		HighlightCurrentWord(true);
 	}
 
+	std::map<std::string, std::string> eConfig = editorConfig->MapFromAbsolutePath(filePath);
+	for (const std::pair<const std::string, std::string> &pss : eConfig) {
+		if (pss.first == "indent_style") {
+			wEditor.Call(SCI_SETUSETABS, pss.second == "tab" ? 1 : 0);
+		} else if (pss.first == "indent_size") {
+			wEditor.Call(SCI_SETINDENT, std::stoi(pss.second));
+		} else if (pss.first == "tab_width") {
+			wEditor.Call(SCI_SETTABWIDTH, std::stoi(pss.second));
+		} else if (pss.first == "end_of_line") {
+			if (pss.second == "lf") {
+				wEditor.Call(SCI_SETEOLMODE, SC_EOL_LF);
+			} else if (pss.second == "cr") {
+				wEditor.Call(SCI_SETEOLMODE, SC_EOL_CR);
+			} else if (pss.second == "crlf") {
+				wEditor.Call(SCI_SETEOLMODE, SC_EOL_CRLF);
+			}
+		} else if (pss.first == "charset") {
+			if (pss.second == "latin1") {
+				CurrentBuffer()->unicodeMode = uni8Bit;
+				codePage = 0;
+			} else {
+				if (pss.second == "utf-8")
+					CurrentBuffer()->unicodeMode = uniCookie;
+				if (pss.second == "utf-8-bom")
+					CurrentBuffer()->unicodeMode = uniUTF8;
+				if (pss.second == "utf-16be")
+					CurrentBuffer()->unicodeMode = uni16BE;
+				if (pss.second == "utf-16le")
+					CurrentBuffer()->unicodeMode = uni16LE;
+				codePage = SC_CP_UTF8;
+			}
+			wEditor.Call(SCI_SETCODEPAGE, codePage);
+		} else if (pss.first == "trim_trailing_whitespace") {
+			stripTrailingSpaces = pss.second == "true";
+		} else if (pss.first == "insert_final_newline") {
+			ensureFinalLineEnd = pss.second == "true";
+		}
+	}
+
 	if (extender) {
 		FilePath defaultDir = GetDefaultDirectory();
 		FilePath scriptPath;
@@ -1387,7 +1444,7 @@ void SciTEBase::ReadFontProperties() {
 	// Set styles
 	// For each window set the global default style, then the language default style, then the other global styles, then the other language styles
 
-	int fontQuality = props.GetInt("font.quality");
+	const int fontQuality = props.GetInt("font.quality");
 	wEditor.Call(SCI_SETFONTQUALITY, fontQuality);
 	wOutput.Call(SCI_SETFONTQUALITY, fontQuality);
 
@@ -1396,12 +1453,12 @@ void SciTEBase::ReadFontProperties() {
 
 	sprintf(key, "style.%s.%0d", "*", STYLE_DEFAULT);
 	std::string sval = props.GetNewExpandString(key);
-	SetOneStyle(wEditor, STYLE_DEFAULT, sval.c_str());
-	SetOneStyle(wOutput, STYLE_DEFAULT, sval.c_str());
+	SetOneStyle(wEditor, STYLE_DEFAULT, StyleDefinition(sval));
+	SetOneStyle(wOutput, STYLE_DEFAULT, StyleDefinition(sval));
 
 	sprintf(key, "style.%s.%0d", languageName, STYLE_DEFAULT);
 	sval = props.GetNewExpandString(key);
-	SetOneStyle(wEditor, STYLE_DEFAULT, sval.c_str());
+	SetOneStyle(wEditor, STYLE_DEFAULT, StyleDefinition(sval));
 
 	wEditor.Call(SCI_STYLECLEARALL, 0, 0);
 
@@ -1419,10 +1476,10 @@ void SciTEBase::ReadFontProperties() {
 		const int subStylesLength = wEditor.Call(SCI_GETSUBSTYLESLENGTH, subStyleBase);
 		for (int subStyle=0; subStyle<subStylesLength; subStyle++) {
 			for (int active=0; active<(diffToSecondary?2:1); active++) {
-				int activity = active * diffToSecondary;
+				const int activity = active * diffToSecondary;
 				sprintf(key, "style.%s.%0d.%0d", languageName, subStyleBase + activity, subStyle+1);
 				sval = props.GetNewExpandString(key);
-				SetOneStyle(wEditor, subStylesStart + subStyle + activity, sval.c_str());
+				SetOneStyle(wEditor, subStylesStart + subStyle + activity, StyleDefinition(sval));
 			}
 		}
 	}
@@ -1435,7 +1492,7 @@ void SciTEBase::ReadFontProperties() {
 
 	sprintf(key, "style.%s.%0d", "errorlist", STYLE_DEFAULT);
 	sval = props.GetNewExpandString(key);
-	SetOneStyle(wOutput, STYLE_DEFAULT, sval.c_str());
+	SetOneStyle(wOutput, STYLE_DEFAULT, StyleDefinition(sval));
 
 	wOutput.Call(SCI_STYLECLEARALL, 0, 0);
 
