@@ -5,13 +5,13 @@
 // Copyright 1998-2005 by Neil Hodgson <neilh@scintilla.org>
 // The License.txt file describes the conditions under which this software may be distributed.
 
-#include <stdlib.h>
-#include <string.h>
+#include <cstdlib>
+#include <cstring>
 
 #include <string>
 #include <vector>
-#include <set>
 #include <map>
+#include <set>
 #include <algorithm>
 
 #include "Scintilla.h"
@@ -133,7 +133,7 @@ void StringList::Set(const std::vector<char> &data) {
 }
 
 namespace {
-	
+
 // Functors used to find elements given a prefix
 
 struct CompareString {
