@@ -9,8 +9,8 @@
 // It is provided "as is" without express or implied warranty.
 ////////////////////////////////////////////////////////////////////////////////
 
-#include <memory.h>
-#include <stdio.h>
+#include <cstring>
+#include <cstdio>
 
 #include "Utf8_16.h"
 
@@ -72,7 +72,7 @@ size_t Utf8_16_Read::convert(char* buf, size_t len) {
 	}
 
 	// Else...
-	size_t newSize = len + len / 2 + 4 + 1;
+	const size_t newSize = len + len / 2 + 4 + 1;
 	if (m_nBufSize != newSize) {
 		delete [] m_pNewBuf;
 		m_pNewBuf = new ubyte[newSize];
@@ -211,7 +211,7 @@ size_t Utf8_16_Write::fwrite(const void* p, size_t _size) {
 		}
 	}
 
-	size_t ret = ::fwrite(m_pBuf,
+	const size_t ret = ::fwrite(m_pBuf,
 		reinterpret_cast<const char*>(pCur) - reinterpret_cast<const char*>(m_pBuf),
 		1, m_pFile);
 
@@ -349,7 +349,7 @@ void Utf16_Iter::operator++() {
 			if (m_pRead >= m_pEnd) {
 				// Have a lead surrogate at end of document with no access to trail surrogate.
 				// May be end of document.
-				--m_pRead;	// With next increment, leave pointer just past buffer 
+				--m_pRead;	// With next increment, leave pointer just past buffer
 			} else {
 				int trail;
 				if (m_eEncoding == eUtf16LittleEndian) {
