@@ -11,8 +11,8 @@
 
 #include <string>
 #include <vector>
-#include <set>
 #include <map>
+#include <set>
 #include <memory>
 
 #undef _WIN32_WINNT
@@ -20,17 +20,17 @@
 #include <windows.h>
 #include <commctrl.h>
 
-#include "Scintilla.h"
 #include "ILexer.h"
+#include "Scintilla.h"
 
 #include "GUI.h"
+#include "ScintillaWindow.h"
 #include "StringList.h"
 #include "StringHelpers.h"
 #include "FilePath.h"
 #include "StyleDefinition.h"
 #include "PropSetFile.h"
 #include "Extender.h"
-#include "DirectorExtension.h"
 #include "SciTE.h"
 #include "Mutex.h"
 #include "JobQueue.h"
@@ -38,6 +38,7 @@
 #include "Worker.h"
 #include "MatchMarker.h"
 #include "SciTEBase.h"
+#include "DirectorExtension.h"
 
 static HWND wDirector = 0;
 static HWND wCorrespondent = 0;
@@ -107,10 +108,10 @@ static void CheckEnvironment(ExtensionAPI *phost) {
 static TCHAR DirectorExtension_ClassName[] = TEXT("DirectorExtension");
 
 static LRESULT HandleCopyData(LPARAM lParam) {
-	COPYDATASTRUCT *pcds = reinterpret_cast<COPYDATASTRUCT *>(lParam);
+	const COPYDATASTRUCT *pcds = reinterpret_cast<COPYDATASTRUCT *>(lParam);
 	// Copy into an temporary buffer to ensure \0 terminated
 	if (pcds->lpData) {
-		std::string dataCopy(static_cast<char *>(pcds->lpData), pcds->cbData);
+		std::string dataCopy(static_cast<const char *>(pcds->lpData), pcds->cbData);
 		DirectorExtension::Instance().HandleStringMessage(dataCopy.c_str());
 	}
 	return 0;
