@@ -311,7 +311,7 @@ static int CodePageFromName(const std::string &encodingName) {
 
 static std::string StringEncode(std::wstring s, int codePage) {
 	if (s.length()) {
-		int cchMulti = ::WideCharToMultiByte(codePage, 0, s.c_str(), static_cast<int>(s.length()), NULL, 0, NULL, NULL);
+		const int cchMulti = ::WideCharToMultiByte(codePage, 0, s.c_str(), static_cast<int>(s.length()), NULL, 0, NULL, NULL);
 		std::string sMulti(cchMulti, 0);
 		::WideCharToMultiByte(codePage, 0, s.c_str(), static_cast<int>(s.size()), &sMulti[0], cchMulti, NULL, NULL);
 		return sMulti;
@@ -322,7 +322,7 @@ static std::string StringEncode(std::wstring s, int codePage) {
 
 static std::wstring StringDecode(std::string s, int codePage) {
 	if (s.length()) {
-		int cchWide = ::MultiByteToWideChar(codePage, 0, s.c_str(), static_cast<int>(s.length()), NULL, 0);
+		const int cchWide = ::MultiByteToWideChar(codePage, 0, s.c_str(), static_cast<int>(s.length()), NULL, 0);
 		std::wstring sWide(cchWide, 0);
 		::MultiByteToWideChar(codePage, 0, s.c_str(), static_cast<int>(s.length()), &sWide[0], cchWide);
 		return sWide;
@@ -419,7 +419,7 @@ void SciTEWin::ReadEmbeddedProperties() {
 
 	HRSRC handProps = ::FindResource(hInstance, TEXT("Embedded"), TEXT("Properties"));
 	if (handProps) {
-		DWORD size = ::SizeofResource(hInstance, handProps);
+		const DWORD size = ::SizeofResource(hInstance, handProps);
 		HGLOBAL hmem = ::LoadResource(hInstance, handProps);
 		if (hmem) {
 			const void *pv = ::LockResource(hmem);
@@ -444,7 +444,7 @@ void SciTEWin::ReadProperties() {
 	SciTEBase::ReadProperties();
 	if (flatterUI) {
 		if (foldColour.empty() && foldHiliteColour.empty()) {
-			Colour lightMargin = ColourRGB(0xF7, 0xF7, 0xF7);
+			const Colour lightMargin = ColourRGB(0xF7, 0xF7, 0xF7);
 			CallChildren(SCI_SETFOLDMARGINCOLOUR, 1, lightMargin);
 			CallChildren(SCI_SETFOLDMARGINHICOLOUR, 1, lightMargin);
 		}
@@ -456,7 +456,7 @@ static FilePath GetSciTEPath(const FilePath &home) {
 		return FilePath(home);
 	} else {
 		GUI::gui_char path[MAX_PATH];
-		if (::GetModuleFileNameW(0, path, ELEMENTS(path)) == 0)
+		if (::GetModuleFileNameW(0, path, static_cast<DWORD>(std::size(path))) == 0)
 			return FilePath();
 		// Remove the SciTE.exe
 		GUI::gui_char *lastSlash = wcsrchr(path, pathSepChar);
@@ -467,12 +467,12 @@ static FilePath GetSciTEPath(const FilePath &home) {
 }
 
 FilePath SciTEWin::GetDefaultDirectory() {
-	GUI::gui_char *home = _wgetenv(GUI_TEXT("SciTE_HOME"));
+	const GUI::gui_char *home = _wgetenv(GUI_TEXT("SciTE_HOME"));
 	return GetSciTEPath(home);
 }
 
 FilePath SciTEWin::GetSciteDefaultHome() {
-	GUI::gui_char *home = _wgetenv(GUI_TEXT("SciTE_HOME"));
+	const GUI::gui_char *home = _wgetenv(GUI_TEXT("SciTE_HOME"));
 	return GetSciTEPath(home);
 }
 
@@ -493,7 +493,7 @@ FilePath SciTEWin::GetSciteUserHome() {
 // Help command lines contain topic!path
 void SciTEWin::ExecuteOtherHelp(const char *cmd) {
 	GUI::gui_string s = GUI::StringFromUTF8(cmd);
-	size_t pos = s.find_first_of('!');
+	const size_t pos = s.find_first_of('!');
 	if (pos != GUI::gui_string::npos) {
 		GUI::gui_string topic = s.substr(0, pos);
 		GUI::gui_string path = s.substr(pos+1);
@@ -523,12 +523,12 @@ void SciTEWin::ExecuteHelp(const char *cmd) {
 
 	if (hHH) {
 		GUI::gui_string s = GUI::StringFromUTF8(cmd);
-		size_t pos = s.find_first_of('!');
+		const size_t pos = s.find_first_of('!');
 		if (pos != GUI::gui_string::npos) {
 			GUI::gui_string topic = s.substr(0, pos);
 			GUI::gui_string path = s.substr(pos + 1);
 			typedef HWND (WINAPI *HelpFn) (HWND, const wchar_t *, UINT, DWORD_PTR);
-			HelpFn fnHHW = (HelpFn)::GetProcAddress(hHH, "HtmlHelpW");
+			HelpFn fnHHW = reinterpret_cast<HelpFn>(::GetProcAddress(hHH, "HtmlHelpW"));
 			if (fnHHW) {
 				XHH_AKLINK ak;
 				ak.cbStruct = sizeof(ak);
@@ -550,7 +550,7 @@ void SciTEWin::ExecuteHelp(const char *cmd) {
 }
 
 void SciTEWin::CopyAsRTF() {
-	Sci_CharacterRange cr = GetSelection();
+	const Sci_CharacterRange cr = GetSelection();
 	std::ostringstream oss;
 	SaveToStreamRTF(oss, static_cast<int>(cr.cpMin), static_cast<int>(cr.cpMax));
 	std::string rtf = oss.str();
@@ -575,7 +575,7 @@ void SciTEWin::CopyPath() {
 		return;
 
 	GUI::gui_string clipText(filePath.AsInternal());
-	size_t blobSize = sizeof(GUI::gui_char)*(clipText.length()+1);
+	const size_t blobSize = sizeof(GUI::gui_char)*(clipText.length()+1);
 	if (::OpenClipboard(MainHWND())) {
 		HGLOBAL hand = ::GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, blobSize);
 		if (hand) {
@@ -639,7 +639,7 @@ HWND SciTEWin::MainHWND() {
 }
 
 void SciTEWin::Command(WPARAM wParam, LPARAM lParam) {
-	int cmdID = ControlIDOfWParam(wParam);
+	const int cmdID = ControlIDOfWParam(wParam);
 	if (wParam & 0x10000) {
 		// From accelerator -> goes to focused pane.
 		menuSource = 0;
@@ -735,7 +735,7 @@ void SciTEWin::OutputAppendEncodedStringSynchronised(const GUI::gui_string &s, i
 	OutputAppendStringSynchronised(sMulti.c_str());
 }
 
-CommandWorker::CommandWorker() : pSciTE(NULL) {
+CommandWorker::CommandWorker() : pSciTE(nullptr) {
 	Initialise(true);
 }
 
@@ -875,7 +875,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 
 	PROCESS_INFORMATION pi = {0, 0, 0, 0};
 
-	// Make a mutable copy as the CreateProcess parameter is mutable 
+	// Make a mutable copy as the CreateProcess parameter is mutable
 	const GUI::gui_string sCommand = GUI::StringFromUTF8(jobToRun.command);
 	std::vector<wchar_t> vwcCommand(sCommand.c_str(), sCommand.c_str() + sCommand.length() + 1);
 
@@ -889,8 +889,10 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 			  startDirectory.AsInternal() : NULL,
 			  &si, &pi);
 
+	const DWORD errCode = ::GetLastError();
 	// if jobCLI "System can't find" - try calling with command processor
-	if ((!running) && (jobToRun.jobType == jobCLI) && (::GetLastError() == ERROR_FILE_NOT_FOUND)) {
+	if ((!running) && (jobToRun.jobType == jobCLI) && (
+		(errCode == ERROR_FILE_NOT_FOUND) || (errCode == ERROR_BAD_EXE_FORMAT))) {
 
 		std::string runComLine = "cmd.exe /c ";
 		runComLine = runComLine.append(jobToRun.command);
@@ -1022,11 +1024,11 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 				} else {
 					running = false;
 				}
-			} else { 
+			} else {
 				// bytesAvail == 0, and if the process
 				// was already dead by the time we did
 				// PeekNamedPipe, there should not be
-				// any more data coming	
+				// any more data coming
 				if (processDead) {
 					running = false;
 				}
@@ -1071,7 +1073,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 				doRepSel = (0 == exitcode);
 
 			if (doRepSel) {
-				int cpMin = static_cast<int>(wEditor.Send(SCI_GETSELECTIONSTART, 0, 0));
+				const int cpMin = static_cast<int>(wEditor.Send(SCI_GETSELECTIONSTART, 0, 0));
 				wEditor.Send(SCI_REPLACESEL,0,SptrFromString(repSelBuf.c_str()));
 				wEditor.Send(SCI_SETSEL, cpMin, cpMin+repSelBuf.length());
 			}
@@ -1080,7 +1082,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 		WarnUser(warnExecuteOK);
 
 	} else {
-		DWORD nRet = ::GetLastError();
+		const DWORD nRet = ::GetLastError();
 		OutputAppendStringSynchronised(">");
 		OutputAppendEncodedStringSynchronised(GetErrorMessage(nRet), codePageOutput);
 		WarnUser(warnExecuteKO);
@@ -1191,7 +1193,7 @@ void SciTEWin::ShellExec(const std::string &cmd, const char *dir) {
 		// it worked!
 		return;
 	}
-	DWORD rc = GetLastError();
+	const DWORD rc = GetLastError();
 
 	std::string errormsg("Error while launching:\n\"");
 	errormsg += mycmdcopy;
@@ -1242,7 +1244,7 @@ void SciTEWin::Execute() {
 
 void SciTEWin::StopExecute() {
 	if (hWriteSubProcess && (hWriteSubProcess != INVALID_HANDLE_VALUE)) {
-		char stop[] = "\032";
+		const char stop[] = "\032";
 		DWORD bytesWrote = 0;
 		::WriteFile(hWriteSubProcess, stop, static_cast<DWORD>(strlen(stop)), &bytesWrote, NULL);
 		Sleep(500L);
@@ -1329,10 +1331,10 @@ void SciTEWin::QuitProgram() {
 }
 
 void SciTEWin::RestorePosition() {
-	int left = propsSession.GetInt("position.left", CW_USEDEFAULT);
-	int top = propsSession.GetInt("position.top", CW_USEDEFAULT);
-	int width = propsSession.GetInt("position.width", CW_USEDEFAULT);
-	int height = propsSession.GetInt("position.height", CW_USEDEFAULT);
+	const int left = propsSession.GetInt("position.left", CW_USEDEFAULT);
+	const int top = propsSession.GetInt("position.top", CW_USEDEFAULT);
+	const int width = propsSession.GetInt("position.width", CW_USEDEFAULT);
+	const int height = propsSession.GetInt("position.height", CW_USEDEFAULT);
 	cmdShow = propsSession.GetInt("position.maximize", 0) ? SW_MAXIMIZE : 0;
 
 	if (left != static_cast<int>(CW_USEDEFAULT) &&
@@ -1572,13 +1574,14 @@ void SciTEWin::DropFiles(HDROP hdrop) {
 	if (hdrop) {
 		const bool tempFilesSyncLoad = props.GetInt("temp.files.sync.load") != 0;
 		GUI::gui_char tempDir[MAX_PATH];
-		DWORD tempDirLen = ::GetTempPath(MAX_PATH, tempDir);
+		const DWORD tempDirLen = ::GetTempPath(MAX_PATH, tempDir);
 		bool isTempFile = false;
 		const int filesDropped = ::DragQueryFile(hdrop, 0xffffffff, NULL, 0);
 		// Append paths to dropFilesQueue, to finish drag operation soon
 		for (int i = 0; i < filesDropped; ++i) {
 			GUI::gui_char pathDropped[MAX_PATH];
-			::DragQueryFileW(hdrop, i, pathDropped, ELEMENTS(pathDropped));
+			::DragQueryFileW(hdrop, i, pathDropped,
+				static_cast<UINT>(std::size(pathDropped)));
 			// Only do this for the first file in the drop op
 			// as all are coming from the same drag location
 			if (i == 0 && tempFilesSyncLoad) {
@@ -1623,7 +1626,7 @@ bool SciTEWin::PreOpenCheck(const GUI::gui_char *arg) {
 	int nbuffers = props.GetInt("buffers");
 	FilePath fpArg(arg);
 
-	if (fileattributes != (DWORD) -1) {	// arg is an existing directory or filename
+	if (fileattributes != INVALID_FILE_ATTRIBUTES) {	// arg is an existing directory or filename
 		// if the command line argument is a directory, use OpenDialog()
 		if (fileattributes & FILE_ATTRIBUTE_DIRECTORY) {
 			OpenDialog(fpArg, GUI::StringFromUTF8(props.GetExpandedString("open.filter")).c_str());
@@ -1666,7 +1669,7 @@ bool SciTEWin::PreOpenCheck(const GUI::gui_char *arg) {
 					GUI::gui_string filterName = GUI::StringFromUTF8(extensions.c_str() + start);
 					GUI::gui_string nameWithExtension = fpArg.AsInternal();
 					nameWithExtension += filterName;
-					if (::GetFileAttributes(nameWithExtension.c_str()) != (DWORD)-1) {
+					if (::GetFileAttributes(nameWithExtension.c_str()) != INVALID_FILE_ATTRIBUTES) {
 						isHandled = true;
 						Open(nameWithExtension.c_str());
 						break;	// Found!
@@ -1761,7 +1764,7 @@ inline bool KeyMatch(const std::string &sKey, int keyval, int modifiers) {
 
 LRESULT SciTEWin::KeyDown(WPARAM wParam) {
 	// Look through lexer menu
-	int modifiers =
+	const int modifiers =
 	    (IsKeyDown(VK_SHIFT) ? SCMOD_SHIFT : 0) |
 	    (IsKeyDown(VK_CONTROL) ? SCMOD_CTRL : 0) |
 	    (IsKeyDown(VK_MENU) ? SCMOD_ALT : 0);
@@ -1809,14 +1812,14 @@ LRESULT SciTEWin::KeyDown(WPARAM wParam) {
 		}
 	}
 
-	return 0l;
+	return 0;
 }
 
 LRESULT SciTEWin::KeyUp(WPARAM wParam) {
 	if (wParam == VK_CONTROL) {
 		EndStackedTabbing();
 	}
-	return 0l;
+	return 0;
 }
 
 void SciTEWin::AddToPopUp(const char *label, int cmd, bool enabled) {
@@ -1837,16 +1840,16 @@ LRESULT SciTEWin::ContextMenuMessage(UINT iMessage, WPARAM wParam, LPARAM lParam
 		// Caused by keyboard so display menu near caret
 		if (wOutput.HasFocus())
 			w = &wOutput;
-		int position = w->Call(SCI_GETCURRENTPOS);
+		const int position = w->Call(SCI_GETCURRENTPOS);
 		pt.x = w->Call(SCI_POINTXFROMPOSITION, 0, position);
 		pt.y = w->Call(SCI_POINTYFROMPOSITION, 0, position);
 		POINT spt = {pt.x, pt.y};
 		::ClientToScreen(HwndOf(*w), &spt);
 		pt = GUI::Point(spt.x, spt.y);
 	} else {
-		GUI::Rectangle rcEditor = wEditor.GetPosition();
+		const GUI::Rectangle rcEditor = wEditor.GetPosition();
 		if (!rcEditor.Contains(pt)) {
-			GUI::Rectangle rcOutput = wOutput.GetPosition();
+			const GUI::Rectangle rcOutput = wOutput.GetPosition();
 			if (rcOutput.Contains(pt)) {
 				w = &wOutput;
 			} else {	// In frame so use default.
@@ -1878,7 +1881,7 @@ void SciTEWin::CheckForScintillaFailure(int statusFailure) {
 
 LRESULT SciTEWin::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 	try {
-		LRESULT uim = uniqueInstance.CheckMessage(iMessage, wParam, lParam);
+		const LRESULT uim = uniqueInstance.CheckMessage(iMessage, wParam, lParam);
 		if (uim != 0) {
 			return uim;
 		}
@@ -2044,7 +2047,7 @@ LRESULT SciTEWin::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 	} catch (GUI::ScintillaFailure &sf) {
 		CheckForScintillaFailure(static_cast<int>(sf.status));
 	}
-	return 0l;
+	return 0;
 }
 
 LRESULT PASCAL SciTEWin::TWndProc(
@@ -2053,7 +2056,7 @@ LRESULT PASCAL SciTEWin::TWndProc(
 	// Find C++ object associated with window.
 	SciTEWin *scite = static_cast<SciTEWin *>(PointerFromWindow(hWnd));
 	// scite will be zero if WM_CREATE not seen yet
-	if (scite == 0) {
+	if (!scite) {
 		if (iMessage == WM_CREATE) {
 			LPCREATESTRUCT cs = reinterpret_cast<LPCREATESTRUCT>(lParam);
 			scite = static_cast<SciTEWin *>(cs->lpCreateParams);
@@ -2081,7 +2084,7 @@ LRESULT ContentWin::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 	case WM_PAINT: {
 			PAINTSTRUCT ps;
 			::BeginPaint(Hwnd(), &ps);
-			GUI::Rectangle rcPaint(ps.rcPaint.left, ps.rcPaint.top, ps.rcPaint.right, ps.rcPaint.bottom);
+			const GUI::Rectangle rcPaint(ps.rcPaint.left, ps.rcPaint.top, ps.rcPaint.right, ps.rcPaint.bottom);
 			Paint(ps.hdc, rcPaint);
 			::EndPaint(Hwnd(), &ps);
 			return 0;
@@ -2138,7 +2141,7 @@ LRESULT ContentWin::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 	}
 	} catch (...) {
 	}
-	return 0l;
+	return 0;
 }
 
 // Convert String from UTF-8 to doc encoding
@@ -2214,13 +2217,15 @@ static void RestrictDLLPath() {
 	if (kernel32) {
 		// SetDefaultDllDirectories is stronger, limiting search path to just the application and
 		// system directories but is only available on Windows 8+
-		SetDefaultDllDirectoriesSig SetDefaultDllDirectoriesFn = (SetDefaultDllDirectoriesSig)::GetProcAddress(
-			kernel32, "SetDefaultDllDirectories");
+		SetDefaultDllDirectoriesSig SetDefaultDllDirectoriesFn =
+			reinterpret_cast<SetDefaultDllDirectoriesSig>(::GetProcAddress(
+			kernel32, "SetDefaultDllDirectories"));
 		if (SetDefaultDllDirectoriesFn) {
 			SetDefaultDllDirectoriesFn(LOAD_LIBRARY_SEARCH_APPLICATION_DIR | LOAD_LIBRARY_SEARCH_SYSTEM32);
 		} else {
-			SetDllDirectorySig SetDllDirectoryFn = (SetDllDirectorySig)::GetProcAddress(
-				kernel32, "SetDllDirectoryW");
+			SetDllDirectorySig SetDllDirectoryFn =
+				reinterpret_cast<SetDllDirectorySig>(::GetProcAddress(
+				kernel32, "SetDllDirectoryW"));
 			if (SetDllDirectoryFn) {
 				// For security, remove current directory from the DLL search path
 				SetDllDirectoryFn(TEXT(""));
