@@ -47,6 +47,8 @@ typedef void *HTHEME;
 #include <commctrl.h>
 #include <richedit.h>
 #include <shlwapi.h>
+// need this header for SHBrowseForFolder
+#include <shlobj.h>
 
 #include <io.h>
 #include <process.h>
@@ -81,9 +83,10 @@ typedef void *HTHEME;
 #endif
 
 #include "Scintilla.h"
-#include "ILexer.h"
+#include "ILoader.h"
 
 #include "GUI.h"
+#include "ScintillaWindow.h"
 
 #include "StringList.h"
 #include "StringHelpers.h"
@@ -130,7 +133,7 @@ public:
 
 	CommandWorker();
 	void Initialise(bool resetToStart);
-	virtual void Execute();
+	void Execute() override;
 };
 
 class Dialog;
@@ -139,13 +142,13 @@ class ContentWin : public BaseWin {
 	SciTEWin *pSciTEWin;
 	bool capturedMouse;
 public:
-	ContentWin() : pSciTEWin(0), capturedMouse(false) {
+	ContentWin() noexcept : pSciTEWin(nullptr), capturedMouse(false) {
 	}
 	void SetSciTE(SciTEWin *pSciTEWin_) {
 		pSciTEWin = pSciTEWin_;
 	}
 	void Paint(HDC hDC, GUI::Rectangle rcPaint);
-	LRESULT WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam);
+	LRESULT WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) override;
 };
 
 struct Band {
@@ -320,7 +323,7 @@ protected:
 	void UserStripShow(const char *description) override;
 	void UserStripSet(int control, const char *value) override;
 	void UserStripSetList(int control, const char *value) override;
-	const char *UserStripValue(int control) override;
+	std::string UserStripValue(int control) override;
 	void UserStripClosed();
 	void ShowBackgroundProgress(const GUI::gui_string &explanation, size_t size, size_t progress) override;
 	BOOL FindMessage(HWND hDlg, UINT message, WPARAM wParam);
@@ -330,6 +333,7 @@ protected:
 	void UIClosed() override;
 	void PerformGrep();
 	void FillCombos(Dialog &dlg);
+	void FillCombosForGrep(Dialog &dlg);
 	BOOL GrepMessage(HWND hDlg, UINT message, WPARAM wParam);
 	static INT_PTR CALLBACK GrepDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
 	void FindIncrement() override;
