@@ -152,7 +152,7 @@ void SciTEWin::Notify(SCNotification *notification) {
 
 			bool bAddSeparator = false;
 			for (int item = 0; item < toolMax; item++) {
-				int itemID = IDM_TOOLS + item;
+				const int itemID = IDM_TOOLS + item;
 				std::string prefix = "command.name.";
 				prefix += StdStringFromInteger(item);
 				prefix += ".";
@@ -250,7 +250,7 @@ void SciTEWin::Notify(SCNotification *notification) {
 					const GUI::Point ptClient = ClientFromScreen(HwndOf(wTabBar), PointOfCursor());
 					const int index = TabAtPoint(HwndOf(wTabBar), ptClient);
 					if (index >= 0) {
-						GUI::gui_string path = buffers.buffers[index].AsInternal();
+						GUI::gui_string path = buffers.buffers[index].file.AsInternal();
 						// Handle '&' characters in path, since they are interpreted in
 						// tooltips.
 						size_t amp = 0;
@@ -276,7 +276,7 @@ void SciTEWin::Notify(SCNotification *notification) {
 		if ((notification->nmhdr.idFrom == IDM_RUNWIN) &&
 		        jobQueue.IsExecuting() &&
 		        hWriteSubProcess) {
-			char chToWrite = static_cast<char>(notification->ch);
+			const char chToWrite = static_cast<char>(notification->ch);
 			if (chToWrite != '\r') {
 				DWORD bytesWrote = 0;
 				::WriteFile(hWriteSubProcess, &chToWrite,
@@ -347,10 +347,10 @@ void SciTEWin::ShowOutputOnMainThread() {
  * Resize the content windows, embedding the editor and output windows.
  */
 void SciTEWin::SizeContentWindows() {
-	GUI::Rectangle rcInternal = wContent.GetClientPosition();
+	const GUI::Rectangle rcInternal = wContent.GetClientPosition();
 
-	int w = rcInternal.Width();
-	int h = rcInternal.Height();
+	const int w = rcInternal.Width();
+	const int h = rcInternal.Height();
 	heightOutput = NormaliseSplit(heightOutput);
 
 	if (splitVertical) {
@@ -367,7 +367,7 @@ void SciTEWin::SizeContentWindows() {
  * Resize the sub-windows, ie. the toolbar, tab bar, status bar. And call @a SizeContentWindows.
  */
 void SciTEWin::SizeSubWindows() {
-	GUI::Rectangle rcClient = wSciTE.GetClientPosition();
+	const GUI::Rectangle rcClient = wSciTE.GetClientPosition();
 	bool showTab = false;
 
 	//::SendMessage(MainHWND(), WM_SETREDRAW, false, 0); // suppress flashing
@@ -398,7 +398,7 @@ void SciTEWin::SizeSubWindows() {
 	bands[bandFind].visible = findStrip.visible;
 	bands[bandReplace].visible = replaceStrip.visible;
 
-	GUI::Rectangle rcSB = wStatusBar.GetPosition();
+	const GUI::Rectangle rcSB = wStatusBar.GetPosition();
 	bands[bandStatus].height = rcSB.Height() - 2;	// -2 hides a top border
 	bands[bandStatus].visible = sbVisible;
 
@@ -426,14 +426,14 @@ void SciTEWin::SizeSubWindows() {
 	int yPos = rcClient.top;
 	for (const Band &band : bands) {
 		if (band.visible) {
-			GUI::Rectangle rcToSet(rcClient.left, yPos, rcClient.right, yPos + band.height);
+			const GUI::Rectangle rcToSet(rcClient.left, yPos, rcClient.right, yPos + band.height);
 			if (hdwp)
 				hdwp = ::DeferWindowPos(hdwp, HwndOf(band.win),
 				0, rcToSet.left, rcToSet.top, rcToSet.Width(), rcToSet.Height(),
 				SWP_NOZORDER|SWP_NOACTIVATE|SWP_SHOWWINDOW);
 			yPos += band.height;
 		} else {
-			GUI::Rectangle rcToSet(rcClient.left, rcClient.top - 41, rcClient.Width(), rcClient.top - 40);
+			const GUI::Rectangle rcToSet(rcClient.left, rcClient.top - 41, rcClient.Width(), rcClient.top - 40);
 			if (hdwp)
 				hdwp = ::DeferWindowPos(hdwp, HwndOf(band.win),
 				0, rcToSet.left, rcToSet.top, rcToSet.Width(), rcToSet.Height(),
@@ -478,7 +478,7 @@ void SciTEWin::SetMenuItem(int menuNumber, int position, int itemID,
 	}
 
 	const UINT typeFlags = (text[0]) ? MF_STRING : MF_SEPARATOR;
-	if (::GetMenuState(hmenu, itemID, MF_BYCOMMAND) == (UINT)(-1)) {
+	if (::GetMenuState(hmenu, itemID, MF_BYCOMMAND) == static_cast<UINT>(-1)) {
 		// Not present so insert
 		::InsertMenuW(hmenu, position, MF_BYPOSITION | typeFlags, itemID, sTextMnemonic.c_str());
 	} else {
@@ -560,8 +560,8 @@ void SciTEWin::LocaliseMenu(HMENU hmenu) {
 				if (mii.dwTypeData) {
 					GUI::gui_string text(mii.dwTypeData);
 					GUI::gui_string accel(mii.dwTypeData);
-					size_t len = text.length();
-					size_t tab = text.find(GUI_TEXT("\t"));
+					const size_t len = text.length();
+					const size_t tab = text.find(GUI_TEXT("\t"));
 					if (tab != GUI::gui_string::npos) {
 						text.erase(tab, len - tab);
 						accel.erase(0, tab + 1);
@@ -650,7 +650,7 @@ static LRESULT PASCAL TabWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM
 	switch (iMessage) {
 
 	case WM_LBUTTONDOWN: {
-			GUI::Point pt = PointFromLong(lParam);
+			const GUI::Point pt = PointFromLong(lParam);
 			st_iLastClickTab = TabAtPoint(hWnd, pt);
 		}
 		break;
@@ -782,7 +782,7 @@ static LRESULT PASCAL TabWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM
 
 						HPEN pen = ::CreatePen(0,1,RGB(255, 0, 0));
 						HPEN penOld = static_cast<HPEN>(::SelectObject(hDC, pen));
-						COLORREF colourNearest = ::GetNearestColor(hDC, RGB(255, 0, 0));
+						const COLORREF colourNearest = ::GetNearestColor(hDC, RGB(255, 0, 0));
 						HBRUSH brush = ::CreateSolidBrush(colourNearest);
 						HBRUSH brushOld = static_cast<HBRUSH>(::SelectObject(hDC, brush));
 						::Polygon(hDC, tab < st_iDraggingTab ? ptsLeftArrow : ptsRightArrow, 7);
@@ -819,7 +819,7 @@ void SciTEWin::Creation() {
 	               &contents);
 	wContent.Show();
 
-	wEditor.SetID(::CreateWindowEx(
+	wEditor.SetScintilla(::CreateWindowEx(
 	              0,
 	              TEXT("Scintilla"),
 	              TEXT("Source"),
@@ -836,7 +836,7 @@ void SciTEWin::Creation() {
 	wEditor.Call(SCI_USEPOPUP, 0);
 	WindowSetFocus(wEditor);
 
-	wOutput.SetID(::CreateWindowEx(
+	wOutput.SetScintilla(::CreateWindowEx(
 	              0,
 	              TEXT("Scintilla"),
 	              TEXT("Run"),
@@ -872,7 +872,7 @@ void SciTEWin::Creation() {
 
 	::SendMessage(hwndToolBar, TB_BUTTONSTRUCTSIZE, sizeof(TBBUTTON), 0);
 	::SendMessage(hwndToolBar, TB_SETBITMAPSIZE, 0, tbLarge ? MAKELPARAM(24, 24) : MAKELPARAM(16, 16));
-	::SendMessage(hwndToolBar, TB_LOADIMAGES, 
+	::SendMessage(hwndToolBar, TB_LOADIMAGES,
 	              tbLarge ? IDB_STD_LARGE_COLOR : IDB_STD_SMALL_COLOR,
 	              reinterpret_cast<LPARAM>(HINST_COMMCTRL));
 
@@ -881,11 +881,11 @@ void SciTEWin::Creation() {
 	if (tbLarge) {
 		addbmp.nID = IDR_CLOSEFILE24;
 	}
-	
+
 	::SendMessage(hwndToolBar, TB_ADDBITMAP, 1, reinterpret_cast<LPARAM>(&addbmp));
 
-	TBBUTTON tbb[ELEMENTS(bbs)];
-	for (unsigned int i = 0;i < ELEMENTS(bbs);i++) {
+	TBBUTTON tbb[std::size(bbs)];
+	for (unsigned int i = 0;i < std::size(bbs);i++) {
 		if (bbs[i].cmd == IDM_CLOSE)
 			tbb[i].iBitmap = STD_PRINT + 1;
 		else
@@ -900,7 +900,7 @@ void SciTEWin::Creation() {
 		tbb[i].iString = 0;
 	}
 
-	::SendMessage(hwndToolBar, TB_ADDBUTTONS, ELEMENTS(bbs), reinterpret_cast<LPARAM>(tbb));
+	::SendMessage(hwndToolBar, TB_ADDBUTTONS, std::size(bbs), reinterpret_cast<LPARAM>(tbb));
 
 	wToolBar.Show();
 
@@ -1027,7 +1027,7 @@ void SciTEWin::Creation() {
 	                 hInstance,
 	                 0);
 	wStatusBar.Show();
-	int widths[] = { 4000 };
+	const int widths[] = { 4000 };
 	// Perhaps we can define a syntax to create more parts,
 	// but it is probably an overkill for a marginal feature
 	::SendMessage(HwndOf(wStatusBar),
