@@ -6,8 +6,6 @@
 // The License.txt file describes the conditions under which this software may be distributed.
 
 #include "SciTEWin.h"
-// need this header for SHBrowseForFolder
-#include <shlobj.h>
 
 /**
  * Flash the given window for the asked @a duration to visually warn the user.
@@ -51,7 +49,7 @@ static void PlayThisSound(
 
 		if (hMM) {
 			typedef BOOL (WINAPI *MMFn) (LPCSTR, HMODULE, DWORD);
-			MMFn fnMM = (MMFn)::GetProcAddress(hMM, "PlaySoundA");
+			MMFn fnMM = reinterpret_cast<MMFn>(::GetProcAddress(hMM, "PlaySoundA"));
 			if (fnMM) {
 				bPlayOK = fnMM(sound, NULL, SND_ASYNC | SND_FILENAME);
 			}
@@ -234,7 +232,7 @@ bool SciTEWin::OpenDialog(const FilePath &directory, const GUI::gui_char *filesF
 
 	if (!openWhat[0]) {
 		StringCopy(openWhat, localiser.Text("Custom Filter").c_str());
-		openWhat[ELEMENTS(openWhat)-2] = L'\0';
+		openWhat[std::size(openWhat)-2] = L'\0';
 		// 2 NULs as there are 2 strings here: the display string and the filter string
 		openWhat[wcslen(openWhat) + 1] = L'\0';
 	}
@@ -251,7 +249,7 @@ bool SciTEWin::OpenDialog(const FilePath &directory, const GUI::gui_char *filesF
 	ofn.nMaxFile = maxBufferSize;
 	ofn.lpstrFilter = openFilter.c_str();
 	ofn.lpstrCustomFilter = openWhat;
-	ofn.nMaxCustFilter = ELEMENTS(openWhat);
+	ofn.nMaxCustFilter = static_cast<DWORD>(std::size(openWhat));
 	ofn.nFilterIndex = filterDefault;
 	GUI::gui_string translatedTitle = localiser.Text("Open File");
 	ofn.lpstrTitle = translatedTitle.c_str();
@@ -302,7 +300,7 @@ FilePath SciTEWin::ChooseSaveName(const FilePath &directory, const char *title,
 		ofn.hwndOwner = MainHWND();
 		ofn.hInstance = hInstance;
 		ofn.lpstrFile = saveName;
-		ofn.nMaxFile = ELEMENTS(saveName);
+		ofn.nMaxFile = static_cast<DWORD>(std::size(saveName));
 		GUI::gui_string translatedTitle = localiser.Text(title);
 		ofn.lpstrTitle = translatedTitle.c_str();
 		ofn.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_NOCHANGEDIR;
@@ -384,7 +382,7 @@ void SciTEWin::LoadSessionDialog() {
 	ofn.hwndOwner = MainHWND();
 	ofn.hInstance = hInstance;
 	ofn.lpstrFile = openName;
-	ofn.nMaxFile = ELEMENTS(openName);
+	ofn.nMaxFile = static_cast<DWORD>(std::size(openName));
 	ofn.lpstrFilter = GUI_TEXT("Session (.session)\0*.session\0");
 	GUI::gui_string translatedTitle = localiser.Text("Load Session");
 	ofn.lpstrTitle = translatedTitle.c_str();
@@ -406,7 +404,7 @@ void SciTEWin::SaveSessionDialog() {
 	ofn.hInstance = hInstance;
 	ofn.lpstrDefExt = GUI_TEXT("session");
 	ofn.lpstrFile = saveName;
-	ofn.nMaxFile = ELEMENTS(saveName);
+	ofn.nMaxFile = static_cast<DWORD>(std::size(saveName));
 	GUI::gui_string translatedTitle = localiser.Text("Save Current Session");
 	ofn.lpstrTitle = translatedTitle.c_str();
 	ofn.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_NOCHANGEDIR;
@@ -451,9 +449,9 @@ void SciTEWin::Print(
 	pdlg.hDevNames = hDevNames;
 
 	// See if a range has been selected
-	Sci_CharacterRange crange = GetSelection();
-	int startPos = static_cast<int>(crange.cpMin);
-	int endPos = static_cast<int>(crange.cpMax);
+	const Sci_CharacterRange crange = GetSelection();
+	const int startPos = static_cast<int>(crange.cpMin);
+	const int endPos = static_cast<int>(crange.cpMax);
 
 	if (startPos == endPos) {
 		pdlg.Flags |= PD_NOSELECTION;
@@ -608,7 +606,7 @@ void SciTEWin::Print(
 	}
 
 	LONG lengthDoc = wEditor.Call(SCI_GETLENGTH);
-	LONG lengthDocMax = lengthDoc;
+	const LONG lengthDocMax = lengthDoc;
 	LONG lengthPrinted = 0;
 
 	// Requested to print selection
@@ -652,7 +650,7 @@ void SciTEWin::Print(
 	SetFileProperties(propsPrint);
 
 	while (lengthPrinted < lengthDoc) {
-		bool printPage = (!(pdlg.Flags & PD_PAGENUMS) ||
+		const bool printPage = (!(pdlg.Flags & PD_PAGENUMS) ||
 		             ((pageNum >= pdlg.nFromPage) && (pageNum <= pdlg.nToPage)));
 
 		char pageString[32];
@@ -667,7 +665,7 @@ void SciTEWin::Print(
 				::SetTextColor(hdc, sdHeader.ForeAsLong());
 				::SetBkColor(hdc, sdHeader.BackAsLong());
 				::SelectObject(hdc, fontHeader);
-				UINT ta = ::SetTextAlign(hdc, TA_BOTTOM);
+				const UINT ta = ::SetTextAlign(hdc, TA_BOTTOM);
 				RECT rcw = {frPrint.rc.left, frPrint.rc.top - headerLineHeight - headerLineHeight / 2,
 				            frPrint.rc.right, frPrint.rc.top - headerLineHeight / 2};
 				rcw.bottom = rcw.top + headerLineHeight;
@@ -697,7 +695,7 @@ void SciTEWin::Print(
 				::SetTextColor(hdc, sdFooter.ForeAsLong());
 				::SetBkColor(hdc, sdFooter.BackAsLong());
 				::SelectObject(hdc, fontFooter);
-				UINT ta = ::SetTextAlign(hdc, TA_TOP);
+				const UINT ta = ::SetTextAlign(hdc, TA_TOP);
 				RECT rcw = {frPrint.rc.left, frPrint.rc.bottom + footerLineHeight / 2,
 				            frPrint.rc.right, frPrint.rc.bottom + footerLineHeight + footerLineHeight / 2};
 				::ExtTextOutW(hdc, frPrint.rc.left + 5, frPrint.rc.bottom + footerLineHeight / 2,
@@ -853,11 +851,8 @@ void SciTEWin::UserStripSetList(int control, const char *value) {
 	userStrip.SetList(control, value);
 }
 
-const char *SciTEWin::UserStripValue(int control) {
-	std::string val = userStrip.GetValue(control);
-	char *ret = new char[val.size() + 1];
-	strcpy(ret, val.c_str());
-	return ret;
+std::string SciTEWin::UserStripValue(int control) {
+	return userStrip.GetValue(control);
 }
 
 void SciTEWin::UserStripClosed() {
@@ -962,6 +957,8 @@ BOOL SciTEWin::FindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 			if (ShouldClose(found)) {
 				::EndDialog(hDlg, IDOK);
 				wFindReplace.Destroy();
+			} else {
+				FillCombos(dlg);
 			}
 			return TRUE;
 		} else if (ControlIDOfWParam(wParam) == IDFINDINSTYLE) {
@@ -969,6 +966,7 @@ BOOL SciTEWin::FindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 				findInStyle = dlg.Checked(IDFINDINSTYLE);
 				dlg.Enable(IDFINDSTYLE, findInStyle);
 			}
+			FillCombos(dlg);
 			return TRUE;
 		}
 	}
@@ -994,12 +992,16 @@ BOOL SciTEWin::HandleReplaceCommand(int cmd, bool reverseDirection) {
 	int replacements = 0;
 	if (cmd == IDOK) {
 		FindNext(reverseDirection);
+		FillCombos(dlg);
 	} else if (cmd == IDREPLACE) {
 		ReplaceOnce();
+		FillCombos(dlg);
 	} else if ((cmd == IDREPLACEALL) || (cmd == IDREPLACEINSEL)) {
 		replacements = ReplaceAll(cmd == IDREPLACEINSEL);
+		FillCombos(dlg);
 	} else if (cmd == IDREPLACEINBUF) {
 		replacements = ReplaceInBuffers();
+		FillCombos(dlg);
 	}
 	GUI::gui_string replDone = GUI::StringFromInteger(replacements);
 	dlg.SetItemText(IDREPLDONE, replDone.c_str());
@@ -1110,7 +1112,7 @@ void SciTEWin::Find() {
 		findStrip.visible = true;
 		SizeSubWindows();
 		findStrip.SetIncrementalBehaviour(props.GetInt("find.strip.incremental"));
-		findStrip.Show();
+		findStrip.ShowStrip();
 	} else {
 		if (searchStrip.visible || replaceStrip.visible)
 			return;
@@ -1170,6 +1172,11 @@ void SciTEWin::PerformGrep() {
 
 void SciTEWin::FillCombos(Dialog &dlg) {
 	dlg.FillComboFromMemory(IDFINDWHAT, memFinds, true);
+	if (dlg.Item(IDREPLACEWITH))    // Replace combo is presented
+		dlg.FillComboFromMemory(IDREPLACEWITH, memReplaces,true);
+}
+
+void SciTEWin::FillCombosForGrep(Dialog &dlg) {
 	dlg.FillComboFromMemory(IDFILES, memFiles, true);
 	dlg.FillComboFromMemory(IDDIRECTORY, memDirectory, true);
 }
@@ -1184,6 +1191,7 @@ BOOL SciTEWin::GrepMessage(HWND hDlg, UINT message, WPARAM wParam) {
 	case WM_INITDIALOG:
 		LocaliseDialog(hDlg);
 		FillCombos(dlg);
+		FillCombosForGrep(dlg);
 		dlg.SetItemTextU(IDFINDWHAT, props.GetString("find.what"));
 		dlg.SetItemTextU(IDDIRECTORY, props.GetString("find.directory"));
 		if (props.GetNewExpandString("find.command") == "") {
@@ -1228,6 +1236,7 @@ BOOL SciTEWin::GrepMessage(HWND hDlg, UINT message, WPARAM wParam) {
 			matchCase = dlg.Checked(IDMATCHCASE);
 
 			FillCombos(dlg);
+			FillCombosForGrep(dlg);
 
 			PerformGrep();
 			if (props.GetInt("find.in.files.close.on.find", 1)) {
@@ -1340,7 +1349,7 @@ void SciTEWin::Replace() {
 		replaceStrip.visible = true;
 		SizeSubWindows();
 		replaceStrip.SetIncrementalBehaviour(props.GetInt("replace.strip.incremental"));
-		replaceStrip.Show();
+		replaceStrip.ShowStrip();
 		havefound = false;
 	} else {
 		if (searchStrip.visible || findStrip.visible)
@@ -1506,13 +1515,13 @@ BOOL SciTEWin::TabSizeMessage(HWND hDlg, UINT message, WPARAM wParam) {
 		} else if ((ControlIDOfWParam(wParam) == IDCONVERT) ||
 			(ControlIDOfWParam(wParam) == IDOK)) {
 			BOOL bOK;
-			int tabSize = static_cast<int>(::GetDlgItemInt(hDlg, IDTABSIZE, &bOK, FALSE));
+			const int tabSize = static_cast<int>(::GetDlgItemInt(hDlg, IDTABSIZE, &bOK, FALSE));
 			if (tabSize > 0)
 				wEditor.Call(SCI_SETTABWIDTH, tabSize);
-			int indentSize = static_cast<int>(::GetDlgItemInt(hDlg, IDINDENTSIZE, &bOK, FALSE));
+			const int indentSize = static_cast<int>(::GetDlgItemInt(hDlg, IDINDENTSIZE, &bOK, FALSE));
 			if (indentSize > 0)
 				wEditor.Call(SCI_SETINDENT, indentSize);
-			bool useTabs = static_cast<bool>(::IsDlgButtonChecked(hDlg, IDUSETABS));
+			const bool useTabs = static_cast<bool>(::IsDlgButtonChecked(hDlg, IDUSETABS));
 			wEditor.Call(SCI_SETUSETABS, useTabs);
 			if (ControlIDOfWParam(wParam) == IDCONVERT) {
 				ConvertIndentation(tabSize, useTabs);
@@ -1666,9 +1675,9 @@ BOOL SciTEWin::AboutMessage(HWND hDlg, UINT message, WPARAM wParam) {
 		LocaliseDialog(hDlg);
 		GUI::ScintillaWindow ss;
 		HWND hwndCredits = ::GetDlgItem(hDlg, IDABOUTSCINTILLA);
-		LONG_PTR subclassedProc = ::SetWindowLongPtr(hwndCredits, GWLP_WNDPROC, reinterpret_cast<LONG_PTR>(CreditsWndProc));
+		const LONG_PTR subclassedProc = ::SetWindowLongPtr(hwndCredits, GWLP_WNDPROC, reinterpret_cast<LONG_PTR>(CreditsWndProc));
 		::SetWindowLongPtr(hwndCredits, GWLP_USERDATA, subclassedProc);
-		ss.SetID(hwndCredits);
+		ss.SetScintilla(hwndCredits);
 		SetAboutMessage(ss, staticBuild ? "Sc1  " : "SciTE");
 		}
 		return TRUE;
