@@ -36,16 +36,18 @@ GUI::gui_string ClassNameOfWindow(HWND hWnd) {
 		return GUI::gui_string();
 }
 
-static void SetFontHandle(const GUI::Window &w, HFONT hfont) {
+namespace {
+
+void SetFontHandle(const GUI::Window &w, HFONT hfont) {
 	SetWindowFont(HwndOf(w), hfont, 0);
 }
 
-static void CheckButton(const GUI::Window &wButton, bool checked) {
+void CheckButton(const GUI::Window &wButton, bool checked) {
 	::SendMessage(HwndOf(wButton),
 		BM_SETCHECK, checked ? BST_CHECKED : BST_UNCHECKED, 0);
 }
 
-static SIZE SizeButton(const GUI::Window &wButton) {
+SIZE SizeButton(const GUI::Window &wButton) {
 	SIZE sz = { 0, 0 };
 #ifdef BCM_GETIDEALSIZE
 	// Push buttons can be measured with BCM_GETIDEALSIZE.
@@ -57,7 +59,7 @@ static SIZE SizeButton(const GUI::Window &wButton) {
 	return sz;
 }
 
-static int WidthText(HFONT hfont, const GUI::gui_char *text) {
+int WidthText(HFONT hfont, const GUI::gui_char *text) {
 	HDC hdcMeasure = ::CreateCompatibleDC(NULL);
 	HFONT hfontOriginal = static_cast<HFONT>(::SelectObject(hdcMeasure, hfont));
 	RECT rcText = {0,0, 2000, 2000};
@@ -68,50 +70,66 @@ static int WidthText(HFONT hfont, const GUI::gui_char *text) {
 	return width;
 }
 
-static int WidthControl(GUI::Window &w) {
-	GUI::Rectangle rc = w.GetPosition();
+int WidthControl(GUI::Window &w) {
+	const GUI::Rectangle rc = w.GetPosition();
 	return rc.Width();
 }
 
-static GUI::gui_string ControlGText(GUI::Window w) {
+GUI::gui_string ControlGText(GUI::Window w) {
 	return TextOfWindow(HwndOf(w));
 }
 
-static std::string ControlText(GUI::Window w) {
+std::string ControlText(GUI::Window w) {
 	const GUI::gui_string gsText = ControlGText(w);
 	return GUI::UTF8FromString(gsText);
 }
 
-static std::string ComboSelectionText(GUI::Window w) {
-	GUI::gui_string gsText;
-	HWND wT = HwndOf(w);
-	const LRESULT selection = ::SendMessageW(wT, CB_GETCURSEL, 0, 0);
+std::string ComboSelectionText(GUI::Window w) {
+	HWND combo = HwndOf(w);
+	const int selection = ComboBox_GetCurSel(combo);
 	if (selection != CB_ERR) {
-		const LRESULT len = ::SendMessageW(wT, CB_GETLBTEXTLEN, selection, 0) + 1;
-		std::vector<GUI::gui_char> itemText(len);
-		const LRESULT lenActual = ComboBox_GetLBText(wT, selection, &itemText[0]);
+		const int len = ComboBox_GetLBTextLen(combo, selection);
+		GUI::gui_string itemText(len+1, L'\0');
+		const int lenActual = ComboBox_GetLBText(combo, selection, &itemText[0]);
 		if (lenActual != CB_ERR) {
-			gsText = GUI::gui_string(&itemText[0], lenActual);
+			itemText.pop_back(); // Remove NUL
+			return GUI::UTF8FromString(itemText);
 		}
 	}
-	return GUI::UTF8FromString(gsText.c_str());
+	return std::string();
 }
 
-static void SetComboFromMemory(GUI::Window w, const ComboMemory &mem) {
+enum class ComboSelection { all, atEnd };
+
+void SetComboText(GUI::Window w, const std::string &s, ComboSelection selection) {
 	HWND combo = HwndOf(w);
-	::SendMessage(combo, CB_RESETCONTENT, 0, 0);
+	GUI::gui_string text = GUI::StringFromUTF8(s);
+	ComboBox_SetText(combo, text.c_str());
+	if (selection == ComboSelection::all) {
+		::SendMessage(combo, CB_SETEDITSEL, 0, MAKELPARAM(0, -1));
+	} else {
+		const size_t textLength = text.length();
+		::SendMessage(combo, CB_SETEDITSEL, 0, MAKELPARAM(textLength, textLength));
+	}
+}
+
+void SetComboFromMemory(GUI::Window w, const ComboMemory &mem) {
+	HWND combo = HwndOf(w);
+	ComboBox_ResetContent(combo);
 	for (int i = 0; i < mem.Length(); i++) {
 		GUI::gui_string gs = GUI::StringFromUTF8(mem.At(i));
-		ComboBox_AddString(HwndOf(w), gs.c_str());
+		ComboBox_AddString(combo, gs.c_str());
 	}
 }
 
+}
+
 LRESULT PASCAL BaseWin::StWndProc(
     HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam) {
 	// Find C++ object associated with window.
 	BaseWin *base = static_cast<BaseWin *>(::PointerFromWindow(hWnd));
 	// scite will be zero if WM_CREATE not seen yet
-	if (base == 0) {
+	if (!base) {
 		if (iMessage == WM_CREATE) {
 			LPCREATESTRUCT cs = reinterpret_cast<LPCREATESTRUCT>(lParam);
 			base = static_cast<BaseWin *>(cs->lpCreateParams);
@@ -162,7 +180,8 @@ static DWORD GetVersion(LPCTSTR lpszDllName) {
     DWORD dwVersion = 0;
     HINSTANCE hinstDll = ::LoadLibrary(lpszDllName);
     if (hinstDll) {
-        DLLGETVERSIONPROC pDllGetVersion = (DLLGETVERSIONPROC)::GetProcAddress(hinstDll, "DllGetVersion");
+        DLLGETVERSIONPROC pDllGetVersion = reinterpret_cast<DLLGETVERSIONPROC>(
+			::GetProcAddress(hinstDll, "DllGetVersion"));
 
         if (pDllGetVersion) {
             DLLVERSIONINFO dvi;
@@ -226,7 +245,7 @@ GUI::Window Strip::CreateButton(const char *text, size_t ident, bool check) {
 		case IDDIRECTIONUP: resNum = IDBM_UP; break;
 		}
 
-		UINT flags = (GetVersion(TEXT("COMCTL32")) >= PACKVERSION(6,0)) ?
+		const UINT flags = (GetVersion(TEXT("COMCTL32")) >= PACKVERSION(6,0)) ?
 			(LR_DEFAULTSIZE) : (LR_DEFAULTSIZE|LR_LOADMAP3DCOLORS);
 		HBITMAP bm = static_cast<HBITMAP>(::LoadImage(
 			::GetModuleHandle(NULL), MAKEINTRESOURCE(resNum + resDifference), IMAGE_BITMAP,
@@ -240,7 +259,7 @@ GUI::Window Strip::CreateButton(const char *text, size_t ident, bool check) {
 	if (!check) {
 		const SIZE sz = SizeButton(w);
 		if (sz.cx > 0) {
-			GUI::Rectangle rc(0,0, sz.cx + 2 * WidthText(fontText, TEXT(" ")), sz.cy);
+			const GUI::Rectangle rc(0,0, sz.cx + 2 * WidthText(fontText, TEXT(" ")), sz.cy);
 			w.SetPosition(rc);
 		}
 	}
@@ -357,8 +376,8 @@ RECT RECTFromRectangle(GUI::Rectangle r) {
 }
 
 void Strip::Paint(HDC hDC) {
-	GUI::Rectangle rcStrip = GetClientPosition();
-	RECT rc = RECTFromRectangle(rcStrip);
+	const GUI::Rectangle rcStrip = GetClientPosition();
+	const RECT rc = RECTFromRectangle(rcStrip);
 	HBRUSH hbrFace = CreateSolidBrush(::GetSysColor(COLOR_3DFACE));
 	::FillRect(hDC, &rc, hbrFace);
 	::DeleteObject(hbrFace);
@@ -430,12 +449,12 @@ int Strip::Lines() const {
 }
 
 void Strip::InvalidateClose() {
-	RECT rc = RECTFromRectangle(CloseArea());
+	const RECT rc = RECTFromRectangle(CloseArea());
 	::InvalidateRect(Hwnd(), &rc, TRUE);
 }
 
 bool Strip::MouseInClose(GUI::Point pt) {
-	GUI::Rectangle rcClose = CloseArea();
+	const GUI::Rectangle rcClose = CloseArea();
 	return rcClose.Contains(pt);
 }
 
@@ -545,7 +564,7 @@ LRESULT Strip::CustomDraw(NMHDR *pnmh) {
 		rbmi.bmiHeader.biSize = sizeof (BITMAPINFOHEADER);
 		::GetDIBits(pcd->hdc, hBitmap, 0, 0, NULL, &rbmi, DIB_RGB_COLORS);
 
-		DWORD colourTransparent = RGB(0xC0,0xC0,0xC0);
+		const DWORD colourTransparent = RGB(0xC0,0xC0,0xC0);
 
 		// Offset from button edge to contents.
 		const int xOffset = ((rcButton.right - rcButton.left) - rbmi.bmiHeader.biWidth) / 2 + 1;
@@ -658,9 +677,9 @@ LRESULT Strip::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 
 	case WM_NOTIFY: {
 			NMHDR *pnmh = reinterpret_cast<LPNMHDR>(lParam);
-			if (pnmh->code == static_cast<unsigned int>(NM_CUSTOMDRAW)) {
+			if (pnmh->code == NM_CUSTOMDRAW) {
 				return CustomDraw(pnmh);
-			} else if (pnmh->code == static_cast<unsigned int>(TTN_GETDISPINFO)) {
+			} else if (pnmh->code == TTN_GETDISPINFO) {
 				NMTTDISPINFOW *pnmtdi = reinterpret_cast<LPNMTTDISPINFO>(lParam);
 				const int idButton = static_cast<int>(
 					(pnmtdi->uFlags & TTF_IDISHWND) ?
@@ -681,7 +700,7 @@ LRESULT Strip::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 		return ::DefWindowProc(Hwnd(), iMessage, wParam, lParam);
 	}
 
-	return 0l;
+	return 0;
 }
 
 void Strip::ShowPopup() {
@@ -718,7 +737,7 @@ void BackgroundStrip::Size() {
 	if (!visible)
 		return;
 	Strip::Size();
-	GUI::Rectangle rcArea = LineArea(0);
+	const GUI::Rectangle rcArea = LineArea(0);
 
 	const int progWidth = 200;
 
@@ -761,7 +780,7 @@ LRESULT BackgroundStrip::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 		return Strip::WndProc(iMessage, wParam, lParam);
 	} catch (...) {
 	}
-	return 0l;
+	return 0;
 }
 
 void BackgroundStrip::SetProgress(const GUI::gui_string &explanation, size_t size, size_t progress) {
@@ -809,7 +828,7 @@ void SearchStrip::Creation() {
 
 	wButton = CreateButton(textFindNext, IDC_INCFINDBTNOK);
 
-	GUI::Rectangle rcButton = wButton.GetPosition();
+	const GUI::Rectangle rcButton = wButton.GetPosition();
 	lineHeight = rcButton.Height() + space + 1;
 }
 
@@ -829,7 +848,7 @@ void SearchStrip::Size() {
 	if (!visible)
 		return;
 	Strip::Size();
-	GUI::Rectangle rcArea = LineArea(0);
+	const GUI::Rectangle rcArea = LineArea(0);
 
 	GUI::Rectangle rcButton = rcArea;
 	rcButton.top -= 1;
@@ -893,7 +912,7 @@ void SearchStrip::Next(bool select) {
 bool SearchStrip::Command(WPARAM wParam) {
 	if (entered)
 		return false;
-	int control = ControlIDOfWParam(wParam);
+	const int control = ControlIDOfWParam(wParam);
 	const int subCommand = static_cast<int>(wParam >> 16);
 	if (((control == IDC_INCFINDTEXT) && (subCommand == EN_CHANGE)) ||
 		(control == IDC_INCFINDBTNOK)) {
@@ -917,12 +936,12 @@ LRESULT SearchStrip::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 		return Strip::WndProc(iMessage, wParam, lParam);
 	} catch (...) {
 	}
-	return 0l;
+	return 0;
 }
 
 LRESULT FindReplaceStrip::EditColour(HWND hwnd, HDC hdc) {
 	if (GetDlgItem(static_cast<HWND>(GetID()),IDFINDWHAT) == ::GetParent(hwnd)) {
-		if (pSearcher->FindHasText() && 
+		if (pSearcher->FindHasText() &&
 			(incrementalBehaviour != simple) &&
 			pSearcher->failedfind) {
 			return NoMatchColour(hdc);
@@ -982,7 +1001,7 @@ void FindStrip::Creation() {
 	SetFontHandle(wText, fontText);
 	wText.Show();
 
-	GUI::Rectangle rcCombo = wText.GetPosition();
+	const GUI::Rectangle rcCombo = wText.GetPosition();
 	lineHeight = rcCombo.Height() + space + 1;
 
 	wButton = CreateButton(textFindNext, IDOK);
@@ -1004,7 +1023,7 @@ void FindStrip::Size() {
 	if (!visible)
 		return;
 	Strip::Size();
-	GUI::Rectangle rcArea = LineArea(0);
+	const GUI::Rectangle rcArea = LineArea(0);
 
 	GUI::Rectangle rcButton = rcArea;
 	rcButton.top -= 1;
@@ -1097,6 +1116,9 @@ void FindStrip::Next(bool markAll, bool invertDirection) {
 	const bool found = pSearcher->FindNext(pSearcher->reverseFind ^ invertDirection) >= 0;
 	if (pSearcher->ShouldClose(found)) {
 		Close();
+	} else {
+ 		SetComboFromMemory(wText, pSearcher->memFinds);
+		SetComboText(wText, pSearcher->findWhat, ComboSelection::atEnd);
 	}
 }
 
@@ -1115,15 +1137,15 @@ void FindStrip::ShowPopup() {
 	for (int i=SearchOption::tWord; i<=SearchOption::tUp; i++) {
 		AddToPopUp(popup, toggles[i].label, toggles[i].cmd, pSearcher->FlagFromCmd(toggles[i].cmd));
 	}
-	GUI::Rectangle rcButton = wButton.GetPosition();
-	GUI::Point pt(rcButton.left, rcButton.bottom);
+	const GUI::Rectangle rcButton = wButton.GetPosition();
+	const GUI::Point pt(rcButton.left, rcButton.bottom);
 	popup.Show(pt, *this);
 }
 
 bool FindStrip::Command(WPARAM wParam) {
 	if (entered)
 		return false;
-	int control = ControlIDOfWParam(wParam);
+	const int control = ControlIDOfWParam(wParam);
 	const int subCommand = static_cast<int>(wParam >> 16);
 	if (control == IDOK) {
 		if (incrementalBehaviour == simple) {
@@ -1166,13 +1188,12 @@ void FindStrip::CheckButtons() {
 	entered--;
 }
 
-void FindStrip::Show() {
+void FindStrip::ShowStrip() {
 	pSearcher->failedfind = false;
 	Focus();
 	pSearcher->SetCaretAsStart();
 	SetComboFromMemory(wText, pSearcher->memFinds);
-	::SetWindowText(HwndOf(wText), GUI::StringFromUTF8(pSearcher->findWhat).c_str());
-	::SendMessage(HwndOf(wText), CB_SETEDITSEL, 0, MAKELPARAM(0, -1));
+	SetComboText(wText, pSearcher->findWhat, ComboSelection::all);
 	CheckButtons();
 	pSearcher->ScrollEditorIfNeeded();
 	MarkIncremental();
@@ -1192,7 +1213,7 @@ void ReplaceStrip::Creation() {
 	SetFontHandle(wText, fontText);
 	wText.Show();
 
-	GUI::Rectangle rcCombo = wText.GetPosition();
+	const GUI::Rectangle rcCombo = wText.GetPosition();
 	lineHeight = rcCombo.Height() + space + 1;
 
 	wStaticReplace = CreateText(textReplacePrompt);
@@ -1367,15 +1388,19 @@ void ReplaceStrip::ShowPopup() {
 	for (int i=SearchOption::tWord; i<=SearchOption::tWrap; i++) {
 		AddToPopUp(popup, toggles[i].label, toggles[i].cmd, pSearcher->FlagFromCmd(toggles[i].cmd));
 	}
-	GUI::Rectangle rcButton = wCheckWord.GetPosition();
-	GUI::Point pt(rcButton.left, rcButton.bottom);
+	const GUI::Rectangle rcButton = wCheckWord.GetPosition();
+	const GUI::Point pt(rcButton.left, rcButton.bottom);
 	popup.Show(pt, *this);
 }
 
 void ReplaceStrip::HandleReplaceCommand(int cmd, bool reverseFind) {
 	pSearcher->SetFind(ControlText(wText).c_str());
+	SetComboFromMemory(wText, pSearcher->memFinds);
+	SetComboText(wText, pSearcher->findWhat, ComboSelection::atEnd);
 	if (cmd != IDOK) {
 		pSearcher->SetReplace(ControlText(wReplace).c_str());
+ 		SetComboFromMemory(wReplace, pSearcher->memReplaces);
+		SetComboText(wReplace, pSearcher->replaceWhat, ComboSelection::atEnd);
 	}
 	//int replacements = 0;
 	if (cmd == IDOK) {
@@ -1397,7 +1422,7 @@ void ReplaceStrip::HandleReplaceCommand(int cmd, bool reverseFind) {
 bool ReplaceStrip::Command(WPARAM wParam) {
 	if (entered)
 		return false;
-	int control = ControlIDOfWParam(wParam);
+	const int control = ControlIDOfWParam(wParam);
 	switch (control) {
 
 	case IDFINDWHAT:
@@ -1454,13 +1479,12 @@ void ReplaceStrip::CheckButtons() {
 	entered--;
 }
 
-void ReplaceStrip::Show() {
+void ReplaceStrip::ShowStrip() {
 	pSearcher->failedfind = false;
 	Focus();
 	pSearcher->SetCaretAsStart();
 	SetComboFromMemory(wText, pSearcher->memFinds);
-	::SetWindowText(HwndOf(wText), GUI::StringFromUTF8(pSearcher->findWhat).c_str());
-	::SendMessage(HwndOf(wText), CB_SETEDITSEL, 0, MAKELPARAM(0, -1));
+	SetComboText(wText, pSearcher->findWhat, ComboSelection::all);
 	SetComboFromMemory(wReplace, pSearcher->memReplaces);
 
 	CheckButtons();
@@ -1547,7 +1571,7 @@ void UserStrip::Size() {
 				rcSize.bottom = rcSize.top + lineHeight - 3;
 			if (ctl.controlType == UserControl::ucCombo)
 				rcSize.bottom = rcSize.top + 180;
-			GUI::Rectangle rcControl(left, topWithFix, left + ctl.widthAllocated, topWithFix + rcSize.Height());
+			const GUI::Rectangle rcControl(left, topWithFix, left + ctl.widthAllocated, topWithFix + rcSize.Height());
 			ctl.w.SetPosition(rcControl);
 			left += ctl.widthAllocated + 4;
 
@@ -1617,10 +1641,10 @@ static StripCommand NotificationToStripCommand(int notification) {
 bool UserStrip::Command(WPARAM wParam) {
 	if (entered)
 		return false;
-	int control = ControlIDOfWParam(wParam);
+	const int control = ControlIDOfWParam(wParam);
 	const int notification = HIWORD(wParam);
 	if (extender) {
-		StripCommand sc = NotificationToStripCommand(notification);
+		const StripCommand sc = NotificationToStripCommand(notification);
 		if (sc != scUnknown)
 			return extender->OnUserStrip(control, sc);
 	}
@@ -1632,7 +1656,7 @@ LRESULT UserStrip::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 		return Strip::WndProc(iMessage, wParam, lParam);
 	} catch (...) {
 	}
-	return 0l;
+	return 0;
 }
 
 int UserStrip::Lines() const {
@@ -1736,7 +1760,7 @@ void UserStrip::SetList(int control, const char *value) {
 			const GUI::gui_string sValue = GUI::StringFromUTF8(value);
 			const std::vector<GUI::gui_string> listValues = ListFromString(sValue);
 			HWND combo = HwndOf(ctl->w);
-			::SendMessage(combo, CB_RESETCONTENT, 0, 0);
+			ComboBox_ResetContent(combo);
 			for (const GUI::gui_string &gs : listValues) {
 				ComboBox_AddString(combo, gs.c_str());
 			}
