@@ -67,7 +67,7 @@ protected:
 	void SetTheme();
 	virtual LRESULT EditColour(HWND hwnd, HDC hdc);
 	virtual LRESULT CustomDraw(NMHDR *pnmh);
-	virtual LRESULT WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam);
+	LRESULT WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) override;
 	virtual void ShowPopup();
 public:
 	bool visible;
@@ -75,6 +75,7 @@ public:
 		closeSize.cx = 11;
 		closeSize.cy = 11;
 	}
+	virtual ~Strip() = default;
 	virtual int Height() {
 		return lineHeight * Lines() + space - 1;
 	}
@@ -86,15 +87,16 @@ class BackgroundStrip : public Strip {
 public:
 	BackgroundStrip() {
 	}
-	virtual void Creation();
-	virtual void Destruction();
-	virtual void Close();
+	virtual ~BackgroundStrip() = default;
+	void Creation() override;
+	void Destruction() override;
+	void Close() override;
 	void Focus();
-	virtual bool KeyDown(WPARAM key);
-	virtual bool Command(WPARAM wParam);
-	virtual void Size();
-	virtual bool HasClose() const;
-	virtual LRESULT WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam);
+	bool KeyDown(WPARAM key) override;
+	bool Command(WPARAM wParam) override;
+	void Size() override;
+	bool HasClose() const override;
+	LRESULT WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) override;
 	void SetProgress(const GUI::gui_string &explanation, size_t size, size_t progress);
 };
 
@@ -105,11 +107,12 @@ protected:
 public:
 	SearchStripBase() : pSearcher(0), hbrNoMatch(0) {
 	}
+	virtual ~SearchStripBase() = default;
 	void SetSearcher(Searcher *pSearcher_) {
 		pSearcher = pSearcher_;
 	}
-	virtual void Creation();
-	virtual void Destruction();
+	void Creation() override;
+	void Destruction() override;
 	LRESULT NoMatchColour(HDC hdc);
 };
 
@@ -120,17 +123,18 @@ class SearchStrip : public SearchStripBase {
 public:
 	SearchStrip() {
 	}
-	virtual void Creation();
-	virtual void Destruction();
-	virtual void Close();
+	virtual ~SearchStrip() = default;
+	void Creation() override;
+	void Destruction() override;
+	void Close() override;
 	void Focus();
-	virtual bool KeyDown(WPARAM key);
+	bool KeyDown(WPARAM key) override;
 	void Next(bool select);
-	virtual bool Command(WPARAM wParam);
-	virtual void Size();
-	virtual void Paint(HDC hDC);
-	virtual LRESULT EditColour(HWND hwnd, HDC hdc);
-	virtual LRESULT WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam);
+	bool Command(WPARAM wParam) override;
+	void Size() override;
+	void Paint(HDC hDC) override;
+	LRESULT EditColour(HWND hwnd, HDC hdc) override;
+	LRESULT WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) override;
 };
 
 class FindReplaceStrip : public SearchStripBase {
@@ -143,14 +147,15 @@ protected:
 	GUI::Window wCheckBE;
 	GUI::Window wCheckWrap;
 	enum IncrementalBehaviour { simple, incremental, showAllMatches };
-	IncrementalBehaviour incrementalBehaviour; 
+	IncrementalBehaviour incrementalBehaviour;
 	FindReplaceStrip() : incrementalBehaviour(simple) {
 	}
-	virtual LRESULT EditColour(HWND hwnd, HDC hdc);
+	LRESULT EditColour(HWND hwnd, HDC hdc) override;
 	enum ChangingSource { changingEdit, changingCombo };
 	void NextIncremental(ChangingSource source);
 public:
-	virtual void Close();
+	virtual ~FindReplaceStrip() = default;
+	void Close() override;
 	void SetIncrementalBehaviour(int behaviour);
 	void MarkIncremental();
 };
@@ -162,18 +167,19 @@ class FindStrip : public FindReplaceStrip {
 public:
 	FindStrip() {
 	}
-	virtual void Creation();
-	virtual void Destruction();
+	virtual ~FindStrip() = default;
+	void Creation() override;
+	void Destruction() override;
 	void Focus();
-	virtual bool KeyDown(WPARAM key);
+	bool KeyDown(WPARAM key) override;
 	void Next(bool markAll, bool invertDirection);
 	void AddToPopUp(GUI::Menu &popup, const char *label, int cmd, bool checked) const;
-	virtual void ShowPopup();
-	virtual bool Command(WPARAM wParam);
-	virtual void Size();
-	virtual void Paint(HDC hDC);
+	void ShowPopup() override;
+	bool Command(WPARAM wParam) override;
+	void Size() override;
+	void Paint(HDC hDC) override;
 	void CheckButtons();
-	void Show();
+	void ShowStrip();
 };
 
 class ReplaceStrip : public FindReplaceStrip {
@@ -186,19 +192,20 @@ class ReplaceStrip : public FindReplaceStrip {
 public:
 	ReplaceStrip() {
 	}
-	virtual void Creation();
-	virtual void Destruction();
-	virtual int Lines() const;
+	virtual ~ReplaceStrip() = default;
+	void Creation() override;
+	void Destruction() override;
+	int Lines() const override;
 	void Focus();
-	virtual bool KeyDown(WPARAM key);
+	bool KeyDown(WPARAM key) override;
 	void AddToPopUp(GUI::Menu &popup, const char *label, int cmd, bool checked) const;
-	virtual void ShowPopup();
+	void ShowPopup() override;
 	void HandleReplaceCommand(int cmd, bool reverseFind = false);
-	virtual bool Command(WPARAM wParam);
-	virtual void Size();
-	virtual void Paint(HDC hDC);
+	bool Command(WPARAM wParam) override;
+	void Size() override;
+	void Paint(HDC hDC) override;
 	void CheckButtons();
-	void Show();
+	void ShowStrip();
 };
 
 class StripDefinition;
@@ -211,16 +218,17 @@ public:
 	UserStrip() : extender(nullptr), pSciTEWin(nullptr) {
 		lineHeight = 26;
 	}
-	virtual void Creation();
-	virtual void Destruction();
-	virtual void Close();
+	virtual ~UserStrip() = default;
+	void Creation() override;
+	void Destruction() override;
+	void Close() override;
 	void Focus();
-	virtual bool KeyDown(WPARAM key);
-	virtual bool Command(WPARAM wParam);
-	virtual void Size();
-	virtual bool HasClose() const;
-	virtual LRESULT WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam);
-	virtual int Lines() const;
+	bool KeyDown(WPARAM key) override;
+	bool Command(WPARAM wParam) override;
+	void Size() override;
+	bool HasClose() const override;
+	LRESULT WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) override;
+	int Lines() const override;
 	void SetDescription(const char *description);
 	void SetExtender(Extension *extender_);
 	void SetSciTE(SciTEWin *pSciTEWin_);
